// Persistence of Vision Ray Tracer Scene Description File
// File: Airlock.inc
// Vers: 3.5
// Desc: Spaceship Airlock
// Date: 9/6/2004
// Auth: Ray Benjamin
//

#ifndef (__airlock__)
  #declare __airlock__ = 1;


#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end
#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end


// Weathering Texture
#declare T_Weathering = texture {
  pigment {
    julia <0.353, 0.288>, 30
      interior 1, 1
 
    color_map {
      [0 Clear]
      [.7 Clear]
      [.75 Black]
      [.77 Black]
      [.775 Brown]
      [.79 Clear]
      [.8 Clear]
      [1.0 Clear]
    }
    //turbulence 1
    //scale <10,10,30>
  } // pigment
  normal {agate}
}

#declare N_Weathering = normal { agate }
#declare T_Ship_White = texture {
  pigment { White }
  normal {N_Weathering}
}


///////////////////////////////////////

// This macro makes a box with the supplied
// png file as the image map placed on XZ plane
// or top of the box.
#macro Make_Image_Map_BoxTop(W,H,D,pngFile)
box {0, <W,H,D>
  texture {
    pigment {
    
      image_map {
        png pngFile
        map_type 0
        interpolate 2
      } // image_map
      scale <W,D,1>
      rotate 90*x
    } // pigment
  
  } // texture
} // box
#end


//
// Equipment Handles
//
// used for removing and replacing equipment

#macro Make_Handle(Handle_W,Handle_H, Handle_R)
  union {
    cylinder {0, Handle_W*x, Handle_R translate Handle_H*y} // cross piece
    sphere {Handle_H*y Handle_R}
    sphere {<Handle_W,Handle_H,0> Handle_R}
    cylinder {0, Handle_H*y, Handle_R} // vert
    cylinder {0, Handle_H*y, Handle_R translate Handle_W*x}
  }
#end // handle macro

// Take Hold
#declare Take_Hold_L = 1*ft;
#declare Take_Hold_H = 5*in;
#declare Take_Hold_R = .5*in;
#declare Take_Hold = object {
  Make_Handle(Take_Hold_L,Take_Hold_H,Take_Hold_R)
  texture { pigment {Yellow} }
  //normal {N_Weathering}
  translate <(-Take_Hold_L/2),0,0>
}

//
// Air Lock Controls
//

// Labeled Key

#macro Make_Key(PngFile)
  #local Key_W = .75*inch;
  #local Key_H = Key_W;
  #local Key_D = .75*inch;
union {
  box { 0, <Key_W, Key_H, Key_D> pigment { White }}
  box { <0,Key_H,0>, <Key_W, Key_H+.01, Key_D> 
    texture {
      pigment {
        image_map {
          png PngFile       // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
          map_type 0        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
          interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
        } // image_map
        rotate 90*x
        scale <Key_W, 1, Key_D>
      } // pigment
    }
    //finish { F_MetalC }
  }
} // union
#end // macro Make_Key()

#declare One_Key = Make_Key("One.png");
#declare Two_Key = Make_Key("Two.png");
#declare Three_Key = Make_Key("Three.png");
#declare Four_Key = Make_Key("Four.png");
#declare Five_Key = Make_Key("Five.png");
#declare Six_Key = Make_Key("Six.png");
#declare Seven_Key = Make_Key("Seven.png");
#declare Eight_Key = Make_Key("Eight.png");
#declare Nine_Key = Make_Key("Nine.png");
#declare Zero_Key = Make_Key("Zero.png");
#declare Period_Key = Make_Key("Period.png");
#declare Plus_Key = Make_Key("Plus.png");
#declare Minus_Key = Make_Key("Minus.png");
#declare Enter_Key = Make_Key("Enter.png");
#declare Pound_Key = Make_Key("Pound.png");
#declare Asterisk_Key = Make_Key("Asterisk.png");

// KeyPad

#declare Keypad = union {
  #local Key_W = .75*inch;
  #local Key_H = 1*inch;
  #local Key_D = .75*inch;
  #local Spacing = .25*inch;
  #local Current_X = 0;
  #local Current_Z = 0;
  
  #local KP = union {
    object {One_Key}
    #local Current_X = Current_X + Key_W + Spacing;
    object {Two_Key translate <Current_X, 0, Current_Z>} 
    #local Current_X = Current_X + Key_W + Spacing;
    object {Three_Key translate <Current_X, 0, Current_Z>} 
    #local Current_X = Current_X + Key_W + Spacing;
    object {Plus_Key translate <Current_X, 0, Current_Z>} 
                        
    // new row
    
    #local Current_X = 0;
    #local Current_Z = Current_Z - Key_D - Spacing;
    object {Four_Key translate <Current_X, 0, Current_Z>} 
    #local Current_X = Current_X + Key_W + Spacing;
    object {Five_Key translate <Current_X, 0, Current_Z>} 
    #local Current_X = Current_X + Key_W + Spacing;
    object {Six_Key translate <Current_X, 0, Current_Z>} 
    #local Current_X = Current_X + Key_W + Spacing;
    object {Minus_Key translate <Current_X, 0, Current_Z>} 
    
    // new row
    #local Current_X = 0;
    #local Current_Z = Current_Z - Key_D - Spacing;
    object {Seven_Key translate <Current_X, 0, Current_Z>} 
    #local Current_X = Current_X + Key_W + Spacing;
    object {Eight_Key translate <Current_X, 0, Current_Z>} 
    #local Current_X = Current_X + Key_W + Spacing;
    object {Nine_Key translate <Current_X, 0, Current_Z>} 
    #local Current_X = Current_X + Key_W + Spacing;
    object {Asterisk_Key translate <Current_X, 0, Current_Z>} 
              
    // new row
    #local Current_X = 0;
    #local Current_Z = Current_Z - Key_D - Spacing;
    object {Pound_Key translate <Current_X, 0, Current_Z>} 
    #local Current_X = Current_X + Key_W + Spacing;
    object {Zero_Key translate <Current_X, 0, Current_Z>} 
    #local Current_X = Current_X + Key_W + Spacing;
    object {Period_Key translate <Current_X, 0, Current_Z>}
    #local Current_X = Current_X + Key_W + Spacing;
    object {Enter_Key translate <Current_X, 0, Current_Z>} 
    
    #declare Key_Pad_D = -Current_Z + Spacing;
    #declare Key_Pad_W = Key_W*4+Spacing*5;
    translate <Spacing,0,Key_Pad_D>
  }
  // KP Handles
  #declare KP_Handle_W = 3;
  #declare KP_Handle_H = .5;
  #declare KP_Handle_R = .125;
  #declare KP_Handle = Make_Handle(KP_Handle_W,KP_Handle_H,KP_Handle_R);
  //
  // keypad base
  #declare KP_Base_W = 6;
  #declare KP_Base_H = 1;
  #declare KP_Base_D = 6;
  #declare Key_Pad_D = Key_Pad_D + Key_D + Spacing;
  
  #declare KP_Offset_W = (KP_Base_W - Key_Pad_W)/2;
  #declare KP_Offset_D = (KP_Base_D - Key_Pad_D)/2;

  difference {
   box {0 <KP_Base_W,KP_Base_H,KP_Base_D>}
   box {<0,0,0>, <Key_Pad_W,.75,Key_Pad_D > translate <KP_Offset_W,.75,KP_Offset_D>}
   texture {T_Silver_3C}
  }
  object {KP translate <KP_Offset_W,.25,KP_Offset_D>}
  object {KP_Handle 
    translate <KP_Base_W/2-(KP_Handle_W/2),KP_Base_H,KP_Base_D-KP_Offset_D/2>
    texture {T_Silver_3D}
  } 
  object {KP_Handle 
    translate <KP_Base_W/2-(KP_Handle_W/2),KP_Base_H,KP_Offset_D/2>
    texture {T_Silver_3D}
  } 
}

// AirLock Button
#declare Airlock_Button_R = 1;

#macro Make_Round_Button(PngFile,R)
sphere {
 0 R
 texture {
   pigment {
        image_map {
          png PngFile       // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
          map_type 0        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
          interpolate 0     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
        } // image_map
        rotate 90*x
        translate <-.5,0,-.5>
        scale <R*2, 1, R*2>
   } // pigment
 } // texture
 finish { Glossy }
 scale <1,.5,1>
}
#end // macro Make_Airlock_Button()

#declare Airlock_Open_Button = Make_Round_Button("Open.png",1);
#declare Airlock_Close_Button = Make_Round_Button("Close.png",1);

// Wheel Lock

#declare Wheel_Lock_Major_R = 6*inch;
#declare Wheel_Lock_Minor_R = .5*inch;
#declare Wheel_Lock_Hub_H = 4*inch;
#declare Wheel_Lock_Hub_R = 1*inch;
#declare Wheel_Lock_Hub_Socket_W = 1*inch;
#declare Wheel_H = Wheel_Lock_Hub_H; // - Wheel_Lock_Minor_R;
#declare Wheel_Spoke_L = Wheel_Lock_Major_R;

#declare Wheel_Lock_Spoke = cylinder {<Wheel_Lock_Hub_R,0,0>, <Wheel_Spoke_L,0,0>, Wheel_Lock_Minor_R/2
  texture {T_Silver_3C}
  //texture {T_Weathering}
  translate <0,Wheel_H,0>
}

#declare Wheel_Lock = union {
  torus {Wheel_Lock_Major_R Wheel_Lock_Minor_R texture {T_Silver_3C} translate Wheel_H*y}
  // hub
  difference {
    cylinder {0, <0,Wheel_Lock_Hub_H,0> Wheel_Lock_Hub_R}
    box {0 , <Wheel_Lock_Hub_Socket_W,Wheel_Lock_Hub_H,Wheel_Lock_Hub_Socket_W>
      translate <-Wheel_Lock_Hub_Socket_W/2,Wheel_Lock_Hub_H/2,-Wheel_Lock_Hub_Socket_W/2>
    }
    texture {T_Silver_3C}
  } // difference
  // spokes
  object {Wheel_Lock_Spoke}
  object {Wheel_Lock_Spoke rotate -180*y}
  object {Wheel_Lock_Spoke rotate 90*y}
  object {Wheel_Lock_Spoke rotate -90*y}
}


// Airlock Status Decal
#declare Airlock_Status_Decal_W = 12;
#declare Airlock_Status_Decal_H = .25;
#declare Airlock_Status_Decal_D = 3;
#declare Airlock_Status_Decal = 
Make_Image_Map_BoxTop(Airlock_Status_Decal_W,Airlock_Status_Decal_H,Airlock_Status_Decal_D,"AirlockStatusDecal.png");

// Status Panel
#declare Panel_Display_W = 12;
#declare Panel_Display_D = 6;
#declare Panel_Display_H = .25;
#declare Status_Panel_Display = 
Make_Image_Map_BoxTop(Panel_Display_W, Panel_Display_H, Panel_Display_D,"AirlockStatusPanel.png");

#declare Display_Mounting_W = Panel_Display_W + 8;
#declare Display_Mounting_H = 1;
#declare Display_Mounting_D = Panel_Display_D + 5;

#declare Status_Mount_X = (Display_Mounting_W - Panel_Display_W)/2;
#declare Status_Mount_Y = .65;
#declare Status_Mount_Z = 1;
#declare Status_Panel_Display_Pos = <Status_Mount_X, Status_Mount_Y, Status_Mount_Z>;
#declare Airlock_Status_Decal_Pos = <Status_Mount_X, Display_Mounting_H-Airlock_Status_Decal_H*.9, Status_Mount_Z+Panel_Display_D + .25>;

#declare Mounting_Handle_W = 8*inch;
#declare Mounting_Handle_H = 1*inch;
#declare Mounting_Handle_R = .125;
#declare Mounting_Handle = object {
  Make_Handle(Mounting_Handle_W,Mounting_Handle_H,Mounting_Handle_R)
  texture {T_Silver_3C}
  rotate -90*y
}
#declare Mounting_Handle_Pos_A = <.5,Display_Mounting_H,(Display_Mounting_D-Mounting_Handle_W)/2>;
#declare Mounting_Handle_Pos_B = Mounting_Handle_Pos_A + (Display_Mounting_W-1)*x;

#declare Display_Mounting = union {
  difference {
    box {0 <Display_Mounting_W, Display_Mounting_H, Display_Mounting_D>}
    box {0 <Panel_Display_W, 2, Panel_Display_D> translate Status_Panel_Display_Pos}
    texture {T_Silver_2D}
    texture {T_Weathering}
  }
  object {Airlock_Status_Decal texture {T_Weathering} translate Airlock_Status_Decal_Pos}
  object {Status_Panel_Display texture {T_Weathering} translate Status_Panel_Display_Pos}
  // Door Buttons
  union {
    // External
    object {Airlock_Open_Button 
      translate <Display_Mounting_W - (Airlock_Button_R + 1), Display_Mounting_H, Panel_Display_D*3/4>
    }
    object {Airlock_Close_Button 
      translate <Display_Mounting_W - (Airlock_Button_R + 1), Display_Mounting_H, Panel_Display_D/4>
    }
    // Internal
    object {Airlock_Open_Button 
      translate <(Airlock_Button_R + 1), Display_Mounting_H, Panel_Display_D*3/4>
    }
    object {Airlock_Close_Button 
      translate <(Airlock_Button_R + 1), Display_Mounting_H, Panel_Display_D/4>
    }
              
    translate <0,0,1>
  }
  // Access Handles
  object {Mounting_Handle translate Mounting_Handle_Pos_A}
  object {Mounting_Handle translate Mounting_Handle_Pos_B}
  
  // Wheel Lock
  object {Wheel_Lock translate <Display_Mounting_W/2,0,-Wheel_Lock_Major_R>}
  
  // Take Holds
  object {Take_Hold rotate 90*y translate -6*inch*x}
  object {Take_Hold rotate 90*y translate (Display_Mounting_W+6*inch)*x}
  
  translate <-Display_Mounting_W/2,0,-Display_Mounting_D>
}





// Lever Lock

// Power Outlet

#declare Air_Lock_Control_Panel = object {Display_Mounting}


//
// Air Lock
//
#declare Airlock_Door_H = 1*ft;
#declare Airlock_Door_R = 5*ft;
#declare Airlock_Door_Left = cylinder { 0, <0,Airlock_Door_H,0> Airlock_Door_R 
  clipped_by {
    plane { x, 0}
  }
  texture {
    pigment {
      gradient x
      color_map {
        [0 Black]
        [.5 Black]
        [.5 Yellow]
        [1.0 Yellow]
      } // color_map
      rotate -45*y
      scale foot
    } // pigment
  } // texture
  //normal { N_Weathering }
} // AirLock_Door_Left

#declare Airlock_Door_Right = cylinder { 0, <0,Airlock_Door_H,0> Airlock_Door_R 
  clipped_by {
    plane { -x, 0}
  }
  texture {
    pigment {
      gradient x
      color_map {
        [0 Yellow]
        [.5 Yellow]
        [.5 Black]
        [1.0 Black]
      } // color_map
      rotate 45*y
      scale foot
    } // pigment
  } // texture
  //texture {T_Weathering}
  //normal { N_Weathering }
} // AirLock_Door_Left

#declare Air_Lock_Frame_H = 4*ft;
#declare Air_Lock_Frame_Width = 3*ft;
#declare Air_Lock_Frame_Outer_R = Airlock_Door_R + Air_Lock_Frame_Width;
#declare Air_Lock_Frame_Inner_R = Airlock_Door_R;
#declare T_Air_Lock_Frame = texture {
  pigment {
    radial
    color_map {
        [0 Yellow]
        [.5 Yellow]
        [.5 Black]
        [1.0 Black]
    }
    frequency 10
    //rotate -90*x
  }
}
texture {T_Weathering}

#declare Air_Lock_Frame = difference {
  cylinder {0, Air_Lock_Frame_H*y, Air_Lock_Frame_Outer_R 
    texture {T_Air_Lock_Frame}
  }
  cylinder {0, (Air_Lock_Frame_H+1)*y, Air_Lock_Frame_Inner_R 
    pigment {Gray90}
    //normal {N_Weathering}
  }
}

#declare Air_Lock = union {
  object {Air_Lock_Frame}
  // door
  union {
    object {Airlock_Door_Left}
    object {Airlock_Door_Right}
    translate (Air_Lock_Frame_H - Airlock_Door_H - 6*inch)*y
  }
  // control panel
  object { Air_Lock_Control_Panel translate <0,Air_Lock_Frame_H-.5,-(Air_Lock_Frame_Inner_R+6*in)>}
  // Take Hold Points
  union {
    object {Take_Hold translate <0,0,Air_Lock_Frame_Inner_R+2*inch>}
    object {Take_Hold translate <0,0,Air_Lock_Frame_Inner_R+2*inch> rotate 180*y}
    object {Take_Hold translate <0,0,Air_Lock_Frame_Inner_R+2*inch> rotate 90*y}
    object {Take_Hold translate <0,0,Air_Lock_Frame_Inner_R+2*inch> rotate -95*y}
    object {Take_Hold translate <0,0,Air_Lock_Frame_Inner_R+2*inch> rotate 45*y}
    object {Take_Hold translate <0,0,Air_Lock_Frame_Inner_R+2*inch> rotate -45*y}
    object {Take_Hold translate <0,0,Air_Lock_Frame_Inner_R+2*inch> rotate 135*y}
    object {Take_Hold translate <0,0,Air_Lock_Frame_Inner_R+2*inch> rotate -135*y}
    translate Air_Lock_Frame_H*y
  }
  // Wheel Lock
  //object {Wheel_Lock translate <0,Air_Lock_Frame_H,-(Air_Lock_Frame_Inner_R+Air_Lock_Frame_Width/2+6*inch)>}
}


////////----------------------------------------------------
#end // ifndef __airlock__