// Persistence of Vision Ray Tracer Scene Description File
// File: CalcDreamsOffice.inc
// Vers: 3.5
// Desc: Office Part of CalculatedDreams
// Date: 8/26/2004
// Auth: Ray Benjamin

light_group {


#include "office_desk.inc"


// ---------------------------------------
// Office Stuff
// ----------------------------------------

////////////////////////////////////////////
// Floor
////////////////////////////////////////////

#macro MakeBoard(Length,Width)
  object {
    Round_Box_Merge(<0,-1*inch,0>,<Width,0,Length>,.1*inch)
    texture {T_Wood20 rotate 4*x scale 10*inch translate rand(R1)*Width*10*inch*x}
  }
#end // macro


#local Board_Width = 4*inch;
#local Board_Length = 20*foot;

#declare Floor = union {
  object {MakeBoard(Board_Length,Board_Width) translate -2*foot*z}
  object {MakeBoard(Board_Length,Board_Width) translate <Board_Width,0,-2*foot>}
  object {MakeBoard(Board_Length,Board_Width) translate <Board_Width*2,0,-2*foot>}
  #local ct = 1;
  #while (ct < 40)
    object {MakeBoard(Board_Length,Board_Width) translate <-Board_Width*ct,0,-2*foot>}
    #local ct = ct + 1;
  #end
  normal {dents 1}
}

// Office Walls

#declare Wall_H = 12*ft;
#declare Wall_D = 6*inch;
#declare T_Wall = texture {
  pigment {White}
}

#macro Make_Wall(Wall_L)
  box { <0,0,0>, <Wall_L, Wall_H, Wall_D> texture {T_Wall} }
#end

#declare Office_Walls = union {
  object {Make_Wall(5*ft) translate <0,0,10*ft>}
  object {Make_Wall(10*ft) rotate -90*y}
}

// Mug
#local Mug_Radius = 1.5*inch;
#local Mug_Height = 3*inch;
#local Mug_Thickness = .25*inch;

#declare T_Mug_Outside = texture {
  pigment { rgb <0,0,.6> } // dark blue
  finish { 
    reflection .25
    specular 1
    roughness .01
  }
}

#declare T_Mug_Inside = texture {
  pigment { rgb <.8,.8,.8> } // very light gray
}

#declare Mug = union {
  #local Inner_Radius = Mug_Radius-Mug_Thickness/2;
  difference {
    Round_Cylinder_Merge(<0,0,0>, <0,Mug_Height,0>, Mug_Radius, .1*inch)
    Round_Cylinder_Merge(<0,.25*inch,0>, <0,Mug_Height+.01*inch>, Inner_Radius, .1*inch)
    texture {T_Mug_Outside}
  }
}

////////// LCD Computer Screen //////////////////

#declare T_Monitor_Bezel = texture {
  pigment { Gray50 }
}

#local Screen_W = 23*inch;
#local Screen_H = 20*inch;
#local Screen_D = .25*inch;

#declare T_Screen = texture {
  pigment {
    image_map {
      png "marsscreen.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 0        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
      // once           // for no repetitive tiling
      // [filter N V]   // N=all or color index # (0...N), V= value (0.0...1.0)
      // [transmit N V] // N=all or color index # (0...N), V= value (0.0...1.0)
      // [use_color | use_index]
    } // image_map
      scale <Screen_W, Screen_H,1>
  }
}

#declare LCD_Computer_Screen = union {
  #local Display_W = 20*inch;
  #local Display_H = 17*inch;
  #local Display_D = .5*inch;
  #local Display_Off_X = (Screen_W - Display_W)/2;
  #local Display_Off_Y = (Screen_H - Display_H)/2;
  #local Display_Off_Z = 1/8*inch;
  // Bezel
  difference {
    box {<0,0,0>, <Screen_W,Screen_H,Screen_D>}
    box {<0,0,-1>, <Display_W, Display_H, 1> translate <Display_Off_X,Display_Off_Y,0>}
    texture {T_Monitor_Bezel}
  }
  // LCD Screen
  box { <0,0,0>, <Display_W, Display_H, Display_D >
    texture {T_Screen}
    translate <Display_Off_X, Display_Off_Y, Display_Off_Z>
  }
}


// Desk Scene
  #declare Room_Depth = -5*ft;
  
  light_source { <10*ft,4*ft,-10*ft> White
    fade_distance 10*ft
    fade_power 2
  }
  //object { Office_Walls translate <-5*ft,-27.5*in, Room_Depth> }
  //object { LCD_Computer_Screen rotate -0*y translate <-Screen_W*2/3,3*inch,0> }
  //object { Desk translate <-Desk_Width/2,-Desk_Height,-Desk_Depth/2>}
  //object { Mug translate <-16*inch,1*inch,-2*inch> }
  
  //object { Floor translate <6*ft, -27.5*in, Room_Depth> }

} // Office Scene light group


