// Persistence of Vision Ray Tracer Scene Description File
// File: CalcDreamsOffice.inc
// Vers: 3.5
// Desc: Office Part of CalculatedDreams
// Date: 8/26/2004
// Auth: Ray Benjamin

#ifndef (ShowOffice)
  #declare ShowOffice = true;
#end

#ifndef (UseAreaLights)
  #declare UseAreaLights = true;
#end
  
#if (ShowOffice)

#include "shapes.inc"

#include "office_desk.inc"

#include "Pencil.inc"



// ---------------------------------------
// Office Stuff
// ----------------------------------------

////////////////////////////////////////////
// Floor
////////////////////////////////////////////

#macro MakeBoard(Length,Width)
  object {
    Round_Box_Merge(<0,-1*inch,0>,<Width,0,Length>,.1*inch)
    texture {T_Wood20 rotate 4*x scale 10*inch translate rand(R1)*Width*10*inch*x}
  }
#end // macro


#local Board_Width = 4*inch;
#local Board_Length = 20*foot;

#declare Floor = union {
  object {MakeBoard(Board_Length,Board_Width) translate -2*foot*z}
  object {MakeBoard(Board_Length,Board_Width) translate <Board_Width,0,-2*foot>}
  object {MakeBoard(Board_Length,Board_Width) translate <Board_Width*2,0,-2*foot>}
  #local ct = 1;
  #while (ct < 40)
    object {MakeBoard(Board_Length,Board_Width) translate <-Board_Width*ct,0,-2*foot>}
    #local ct = ct + 1;
  #end
  normal {dents 1}
}

#include "Exit_Sign.inc"

// Office Walls

#declare Wall_H = 12*ft;
#declare Wall_D = 6*inch;

#declare T_Wall = texture {
  pigment {White}
}

// Make Stucco -- from the Stucco Example in Advanced Scenes
#declare L2=0; //clock;  //Set amount of decay of stucco, higher value is more decay. animate
#declare PWrink= pigment { //Mortar
   wrinkles
   scale 0.25
   colour_map {
      [0, rgb 0.5]
      [L2, rgb 0.96]
   }
}
#declare Stucco= pigment {    //Stucco
   granite
   scale 0.05
   colour_map {
      [0, rgb 0.96]
      [1, rgb 1]
   }
}

#declare StuMor= pigment {       //Stucco & Mortar
   pigment_pattern{
      wrinkles
      scale 0.25
   }
   pigment_map {
      [L2, PWrink]
      [L2, Stucco]
   }
}

#declare HF_Stucco=height_field {  //Turn it into a hightfield
   function 500, 500{               //500,500 for test higher is better, but watch the memory
      pigment{StuMor }
   }
   translate -0.5
   rotate -90*x
   scale 1
   pigment {                         //Use the mortar to colour up the stucco
      pigment_pattern {
         wrinkles
         scale 0.25
      }
      color_map {
         [L2, rgb 0.5]
         [L2, rgb <1,1,0.95>]
      }
      warp {planar z, 0}
      translate <-0.5, -0.5, 0>
      rotate <180,0,0>
      scale 1
   }
   //scale 1/20
}

#macro Make_Wall(Wall_L)
  //box { <0,0,0>, <Wall_L, Wall_H, Wall_D> texture {T_Stucco} }
  object {HF_Stucco scale <Wall_L,Wall_H,Wall_D> translate <Wall_L/2,Wall_H/2,Wall_D/2>}
#end

#declare Office_Walls = union {
  object {Make_Wall(5*ft) translate <0,0,10*ft>}
  object {Make_Wall(10*ft) rotate -90*y}
  //object {Exit_Sign scale .8 translate <4*ft,6*ft+5*inch,10*ft-.5*inch>}
  no_shadow
}

////////////// Cubicle Dividers ////////////////////////////

#declare P_Cloth1 = pigment {
  gradient x
  color_map {
    [0 rgb <.2,.5,.8>]
    [0.7 rgb <.2,.5,.8>]
    [.7 White]
    [1.0 White]
  }
  frequency 8
}

#declare P_Cloth2 = pigment {
  gradient z
  color_map {
    [0 rgb <.2,.5,.8>]
    [0.7 rgb <.2,.5,.8>]
    [.7 White]
    [1.0 White]
  }
  frequency 8
}

#declare P_Cloth = pigment {
  checker
  pigment {P_Cloth1}
  pigment {P_Cloth2}
  translate .5*x
  translate .5*z
  scale 1/2.5/16
}

#declare T_Cloth = texture { 
  pigment {P_Cloth}
  finish { 
    ambient .3 
    diffuse .5
  } 
}

#declare T_Divider_Metal = texture {
  pigment { Tan }
  finish {
    brilliance 5
    specular .03125
    reflection { .01, .02 }
    conserve_energy
  }
}

#declare Divider_W = 5*ft;
#declare Divider_H = 5.5*ft;
#declare Divider_T = 2.5*inch;
#declare Divider_Edge_T = .75*in;

#declare LCorner = <-Divider_W/2, -0.05, -Divider_H/2>;  #declare RCorner = <Divider_W/2, 0.05, Divider_H/2>;
#declare Office_Divider_Panel = union {
   
  intersection {
  	isosurface {
  		function { f_mesh1(x,y,z, 1/20, 1/20, 1/20, 0.005, 1/10) }
  			//P0= X frequency
  			//P1= Z frequency
  			//P2= scale (X-Z)
  			//P3= amplitude
  			//P4= scale Y
  		threshold  0.001
  		contained_by { box { LCorner, RCorner } }
  		max_gradient 0.2
  		texture { T_Cloth scale 1 }
  		no_shadow
  		scale 4
  	}
  	
  	box { LCorner*0.9999, RCorner*0.9999  //texture { pigment {Gray50} } 
  	}
  	scale 1  
  	rotate x*-90
    translate <Divider_W/2,Divider_H/2,0.05>
  }
  
  merge {
    box {<-Divider_Edge_T,0,.25>, <Divider_W+Divider_Edge_T,Divider_H,Divider_T-.25> }
    box {<0,0,.25>, <Divider_W,Divider_H + Divider_Edge_T,Divider_T-.25>}
    cylinder { <0,0,.25>, <0,0,Divider_T-.25>, Divider_Edge_T translate <0,Divider_H,0>}
    cylinder { <0,0,.25>, <0,0,Divider_T-.25>, Divider_Edge_T translate <Divider_W,Divider_H,0>}
    texture {T_Divider_Metal}
  }
}

// Mug
#include "Ceramic_Mug.inc"

/////////////////// Speaker ////////////////////////////////

#include "Speakers.inc"

//////////////// Wedding Picture ////////////////
#include "Wedding_Picture.inc"
/////////////////////////////////////////////////

#include "Pen.inc"

//////////// Paper Clip //////////////////

#include "Paperclip.inc"


/////////////////////// Thumb Tack /////////////////////////

#include "ThumbTack.inc"

//////////// Coffee Stain Decals ////////////////

#local Coffee_Stain_W = 3.125*in + .125*in;
#local Coffee_Stain_D = Coffee_Stain_W;
#declare Coffee_Stain = box { <0,0,0>, <Coffee_Stain_W,.01,Coffee_Stain_D>
  no_shadow
  pigment {
    image_map {
      png "stain_coffee.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 0        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
      // once           // for no repetitive tiling
      // filter all .1
      // [filter N V]   // N=all or color index # (0...N), V= value (0.0...1.0)
      //transmit all .1
      // [transmit N V] // N=all or color index # (0...N), V= value (0.0...1.0)
      // [use_color | use_index]
    } // image_map
    rotate 90*x
    scale <Coffee_Stain_W, 1, Coffee_Stain_D>
  }
  rotate -0*x
  translate <-Coffee_Stain_W/2,.01*in,-Coffee_Stain_D/2> 
}

///////////// Web Cam ///////////////////////////
#include "webcam.inc"

////////////// Pop Can
#include "PopCan.inc"

////////////// Stapler
#include "Stapler.inc"

/////////////// PostIt Notes
#include "Postit.inc"

// Paper
#include "Paper.inc"

#declare Plain_Paper = object {
  Paper
  scale <8.5*inch,1,11*inch>
  translate .05*inch*y
}

#declare T_Dreamer_Design = texture {
  pigment {
    image_map {
      png "VoidDreamerDesign.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 0        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
    } // image map
    //translate <-.5,-.5,0>
    rotate 90*x
  } // pigment
} // texture
 
#declare Posted_Paper = object {
  //Paper
  box {<0,0,0>, <1,.01,1>}
  texture {T_Dreamer_Design}
  scale <8.5*inch, 1, 11*inch>
}

#declare T_SpaceTug_Sketch = texture {
  pigment {
    image_map {
      png "SpaceTugSketch.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 0        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
    } // image map
    //translate <-.5,-.5,0>
    rotate 90*x
    //rotate 180*y
  } // pigment
} // texture
 
#declare SpaceTugSketch_Paper = object {
  box {<0,0,0>, <1,.01,1>}
  texture {T_SpaceTug_Sketch}
  scale <8.5*inch, 1, 11*inch>
}

////////// LCD Computer Screen //////////////////

#if (ShowLCD)
  #include "LCD_Monitor.inc"
#end

#if (ShowPencil)
  //#include "Pencil.inc"
#end

light_group {

// Desk Scene
  #declare Room_Depth = -5*ft;
  #declare Light_Pos = <1*ft,12*ft,-10*ft>;          
  #declare Light_Color = White*.85;

#if (UseAreaLights)
  // An area light (creates soft shadows)
  // WARNING: This special light can significantly slow down rendering times!
  light_source {
    Light_Pos                 // light's position (translated below)
    color Light_Color       // light's color
    area_light
    <24, 0, 0> <0, 0, 24> // lights spread out across this distance (x * z)
    8, 8                // total number of lights in grid (4x*4z = 16 lights)
    //adaptive 3          // 0,1,2,3...
    jitter              // adds random softening of light
    circular            // make the shape of the light circular
    //orient              // orient light
    fade_distance 15*ft
    fade_power 2
  }
#else  
  light_source { Light_Pos 
    Light_Color
    fade_distance 15*ft
    fade_power 2
  }
#end  
  // Fill Light -- testing
  //light_source {<0,1*ft,-10*ft> White}
  
  #if (LCD_Frame_Only)
    object { LCD_Monitor rotate -0*y translate <-4*in,0,0> no_shadow }
  #else
    #if (ShowLCD)
      object { LCD_Monitor rotate -0*y translate <-4*in,0,0> }
    #end
    #if (ShowWalls)
      object { Office_Walls translate <-5*ft,-27.5*in, Room_Depth> }
      #if (ShowDivider)
        object { Office_Divider_Panel translate <-5*ft,-27.5*in,Desk_Depth+2*in> }
      #end
    #end
    
    object { Desk translate <-Desk_Width/2,-(Desk_Height+1),-Desk_Depth/2>}
    #local Mug_Pos = <-15*in, .05, -2*in>;
    #if (ShowSteam)
      object { Steaming_Mug translate Mug_Pos }
    #else
      object { Saturn_Mug translate Mug_Pos}
    #end
    object { Coffee_Stain translate (Mug_Pos + <-1*in,0.05, -1*in>) } // close to mug
    object { Coffee_Stain rotate 20*y translate (Mug_Pos + <0,0.05,-.5*in>)} // under mug

    object { Web_Cam_Clipped translate <0,22*inch,0> }
    object { Rectangular_Speaker rotate -18*y translate <-21*in,0*inch,6*inch> }  // left
    object { Rectangular_Speaker rotate 45*y translate <9*in,0*inch,1*inch> }  // right
    
    object { Wedding_Picture rotate 30*y translate <14*inch,0,5*inch>}
    
    object { Horizontal_Pen rotate 22*y translate <10*in,0,-7*in>} // pen on right side
    
    object { Horizontal_Pen_Body rotate -95*y translate <-11*in,0,-4*in> } // pen on left side
    object { Horizontal_Pen_Body rotate 250*y translate <6*in,0,2*in>} // pen parallel to stapler on right
    
    object { Paper_Clip rotate -23*y translate <0,0,-5*in> }
    object { Paper_Clip rotate 103*y translate <-5*in, 0, -5*in> }
    object { Bent_Paper_Clip rotate 38.7*y translate <8.7*inch,0,-1.2*inch>} // detail 2 view
    
    
    object { PopCan rotate -20*y translate <12.5*in, 0*in, -5*in>} // on right, foreground
    object { PopCan rotate -97*y translate <9*in,0*in,5*in>} 
    
    object { Stapler rotate 70*y translate <3*in, 0*in, 1*in> }
    
    object { PostIt_Notes rotate -8*y translate <3*inch,0,-5*in> }
    object { Povcomp_Postit rotate <-90,0,50> translate <7.75*inch,3.5*inch,-.025*inch>}  // on monitor
    
    object { Plain_Paper translate <-16*inch,0,-5*inch> pigment {White}}  // paper on desk with coffee stain 
    object { SpaceTugSketch_Paper rotate -78*y translate <1*inch,0,-14*inch> } // paper with spacetug sketch
    
    object { Posted_Paper rotate -90*x rotate 3*z translate <-46*inch,20*inch,Desk_Depth+2*in - .05> } // paper on divider wall
    object { Thumb_Tack_Blue rotate -90*x translate <-42*inch,29*inch,Desk_Depth+2*in>}
    // Thumb tacks on divider wall
    union {
      object { Thumb_Tack_Red rotate -90*x translate <0,0,0>}
      object { Thumb_Tack_White rotate -90*x translate <0,2*in,0>}
      object { Thumb_Tack_Yellow rotate -90*x translate <-1*inch, 1*in, 0>}
      object { Thumb_Tack_Clear rotate -90*x translate <-1.5*inch,-1*in, 0>}
      object { Thumb_Tack_Blue rotate -90*x translate <-.25*inch,-1.5*in,0>}
      object { Thumb_Tack_Yellow rotate -90*x translate <-4*inch,-3*inch,0>}
      object { Thumb_Tack_Red rotate -90*x translate <-7*inch,-3*inch,0>}
      translate <-36*inch,24*in,Desk_Depth+2*in>
    }
    
    // thumb tacks on desk
    object {Make_Horizontal_Thumb_Tack(Red) translate <-7.2*inch,0,-5*inch> }    
    object {Make_Horizontal_Thumb_Tack(Blue) rotate -22*y translate <-12*inch,0,0*inch> }    
    object {Make_Horizontal_Thumb_Tack(Green) rotate 87*y translate <-9.2*inch,0,-3*inch> }
    object {Make_Horizontal_Thumb_Tack(White) rotate 98*y translate <-10.0*inch,0, -4*inch> }
    object {Make_Horizontal_Thumb_Tack(Clear) rotate 41*y translate <9.2*inch,0,-3.5*inch>} // in detail 2 view    
    
    
    object { Floor translate <6*ft, -27.5*in, Room_Depth-1*ft> }
  #end // if frame only
} // Office Scene light group

#local Pencil_Pos = <7.25*in,0,-4*in>;
#local Pencil_Rotation = <0,-25,0>;

#if (ShowPencil)
  light_source { Light_Pos White
    fade_distance 10*ft
    fade_power 2
  }
  object { New_Pencil rotate Pencil_Rotation translate Pencil_Pos }
#else
  light_source { Light_Pos White
    fade_distance 10*ft
    fade_power 2
  }
  object { Old_Unsharpened_Pencil rotate Pencil_Rotation translate Pencil_Pos }
#end

#end // if ShowOffice

//    object { New_Pencil rotate -25*y translate <6*in,0,-4*in> translate <0,14*in,-12*inch>}

// light_source { <0,20*inch,-40*inch> color White }