// Persistence of Vision Ray Tracer Scene Description File
// File: CalculatedDreams.pov
// Vers: 3.5
// Desc: Basic Scene Example
// Date: 8/18/2004
// Auth: Ray Benjamin
//

//
// Closeups must be 320x240 squares


#version 3.5;

#include "colors.inc"
// several different gold colors, finishes and textures
#include "golds.inc"

// various metal colors, finishes and textures
// brass, copper, chrome, silver
#include "metals.inc"
// macros for generating various shapes
//
#include "shapes.inc" 
#include "shapes2.inc"
// various (mostly layered) wood textures
// T_Wood1 - T_Wood35
#include "woods.inc"
// various glass finishes, colors and interiors
//
#include "glass.inc"
// some standard finishes
#include "finish.inc"

#include "grid.inc"
#include "utils.inc"

////////// Scene Controls

#declare View = 1; // 0 = main, 1 = detail 1 or 2 = detail 2 are legal values

#declare UseRadiosity = true;
#declare UseAreaLights = false;
#declare ShowSpace = true;
#declare Show_Stars = true;
#declare Show_Ship = true;
#declare Show_Tug = true;
#declare Ship_Scale_Factor = 240;
#declare Engine_On = false;
#declare Ship_Lights = true;
#declare Show_Lights = true;
#declare Show_Cockpit_Light = true;
#declare ShowOffice = true;
#declare ShowWalls = true;
#declare ShowDivider = true;
#declare ShowLCD = true;
#declare ShowPencil = false;
#declare ShowSteam = true;
#declare Show_Dirt = true;
#declare Show_Scuffs = true;
#declare LCD_Frame_Only = false;

#declare R1 = seed(1232);

global_settings {
  max_trace_level 256
  assumed_gamma 1.0
  ambient_light .3
  #if (UseRadiosity)
    radiosity {
      pretrace_start 0.08           // start pretrace at this size
      pretrace_end   0.04           // end pretrace at this size
      count 35                      // higher -> higher quality (1..1600) [35]
      nearest_count 5               // higher -> higher quality (1..10) [5]
      error_bound 1.8               // higher -> smoother, less accurate [1.8]
      recursion_limit 3             // how much interreflections are calculated (1..5+) [3]
      low_error_factor .5           // reduce error_bound during last pretrace step
      gray_threshold 0.0            // increase for weakening colors (0..1) [0]
      minimum_reuse 0.015           // reuse of old radiosity samples [0.015]
      brightness 1                  // brightness of radiosity effects (0..1) [1]
  
      adc_bailout 0.01/2
      //normal on                   // take surface normals into account [off]
      //media on                    // take media into account [off]
      //save_file "file_name"       // save radiosity data
      //load_file "file_name"       // load saved radiosity data
      //always_sample off           // turn sampling in final trace off [on]
      //max_sample 1.0              // maximum brightness of samples
    }
  #end
}

//////// Units /////////////////
#declare inch = 1;
#declare foot = 12;
#declare in = inch;
#declare ft = foot;
#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end


////////// My Includes 


///////////////////////

// ---------------------------------------- 

#declare ZoomIn_MaintainAspectRatioWhenZooming = 0;
#include "screen.inc"
#include "zoomin.inc"

// Normal Scene Camera ---
Set_Camera_Location(<0,10*inch,-2.5*foot>)
Set_Camera_Look_At(<0*in,10*inch,0>)

// --------------------------------
// Zooms
// Detail 2 Zoom 
#if (View = 2)
  #local Starting_Column = 0.59375;
  #local Starting_Row = .75;
  #local Ending_Column = Starting_Column + 0.2;
  #local Ending_Row = Starting_Row + 0.2;
  #local ZoomStr = concat("+SC", str(Starting_Column,-3,-1), 
                         " +SR", str(Starting_Row,-3,-1), 
                         " +EC", str(Ending_Column,-3,-1), 
                         " +ER", str(Ending_Row,-3,-1))
                         
  #debug concat("\n\n\nRendering with the string: ", ZoomStr, "\n\n\n")                       
  
  Set_Zoom_Area_ScreenInc(ZoomStr)
  Update_Camera_Zoom()
#end // if detail2
         
///// DETAIL ONE ZOOM
#if (View = 1)
  #local Starting_Column = 0.752230; //0.748230;
  #local Starting_Row = 0.243706; // 0.223706;
  #local Ending_Column = Starting_Column + .15;
  #local Ending_Row = Starting_Row + .2666667;
  #local ZoomStr = concat("+SC", str(Starting_Column,-3,-1), 
                         " +SR", str(Starting_Row,-3,-1), 
                         " +EC", str(Ending_Column,-3,-1), 
                         " +ER", str(Ending_Row,-3,-1))
  Set_Camera_Aspect(1200,1600)
  //Set_Zoom_Area_ScreenInc("+sc0.748230 +sr0.223706 +ec0.890909 +er0.517529") // CURRENT CORRECT ONE
  Set_Zoom_Area_ScreenInc(ZoomStr)
  Update_Camera_Zoom()
#end // if detail1

//-- Closeup of SpaceTug --
//Set_Camera_Location(<5*inch,11*inch,-1.5*foot>)
//Set_Camera_Look_At(<5*inch,13*inch,0>)


// Camera for looking down at scene
//Set_Camera_Location(<0,24*in,-11*in>)

//camera { location  <0.0, 10*inch, -2.5*foot>  look_at   <0.0, 10.0*in,  0.0>} // scene camera
//camera { location  <0.0, 10*inch, -2.2*foot>  look_at   <0.0, 10.0*in,  0.0>} // scene camera
//camera { location  <0.0, 0, 0>  look_at   <0.0, 0.0,  1.0> translate <-4*in,12*inch,-17*in>} // computer screen closeup
//camera { location  <0.0, 0, 0>  look_at   <0.0, 0.0,  1.0> translate <-4*in,12*inch,0*in>} // computer wallpaper

//camera { location  <0.0, 2*ft, -5*foot>  look_at   <0.0, 10.0*in,  0.0>} // scene camera farther back
//camera { location  <5*ft, 2*ft, -15*ft>  look_at   <0.0, 10.0*in,  0.0>} // big picture
//camera { location  <0.0, 10*inch, 12*in>  look_at   <0.0, 10.0*in,  5.0*ft>} // look at cargo
//camera { location <0, 0, 0> look_at <0,0,1> translate <0,.5*inch,-6*inch> } // Look at pen on desk   



#if (Show_Stars)
  /*
  #declare galaxy_seed = (clock - .5) * 2e5 + 1.25;
  */
  #declare galaxy_declare_only = true;
  #declare debug_options = false;
  #declare star_scale = .5;
  #declare star_distance = 10000*m;
  #declare galaxy_bgnebula = off;
  #declare galaxy_nebula_sphere = 0;
  #declare galaxy_intensity = .5;
  #declare galaxy_bg = off;
  #include "GALAXY.INC"
  #include "GALAXY.BG"
  
  sky_sphere {
    pigment {BGStars1}
    //pigment {BGMilkyWay rotate z * 45}
    //pigment {BGNebula5 scale 2}
    }
#else
  sky_sphere {
    pigment { Black }
  }
#end 

#if (ShowOffice)
  #include "CalcDreamOffice.inc"
#end // if showoffice
#if (ShowSpace)
  #include "CalcDreamSpace.inc"
#end // showspace

//object {XYZ_Axis}