// Persistence of Vision Ray Tracer Scene Description File
// File: Cargo_Module.pov
// Vers: 3.5
// Desc: Testing Cargo
// Date: 9/11/2004
// Auth: Ray Benjamin

#ifndef (__cargo_module__)
#declare __cargo_module__ = 1;

#ifndef(Show_Scuffs)
  #declare Show_Scuffs = 1;
#end

#ifndef (Show_Glass)
  #declare Show_Glass = true;
#end

#ifndef(T_Scuffs)
  #declare T_Scuffs = texture {
      pigment {
          granite
          color_map {
            /*
              [0.0 White]
              [0.08 White]
              [0.08 Clear]
              [0.8 Clear]
              [0.92 Black]
              [1.0 Gray10]
            */
            [0 White]
            [.15 Clear]
            [.75 Clear]
            [1 Black]
          } // color_map
          turbulence 1
          scale 1000
          scale <30,1,1>
      } // pigment
      finish { ambient 0 }
  } // texture T_Scuffs
#end


#ifndef(T_Ship_White)
#declare T_Ship_White = 
texture {
  pigment { White }
  finish {ambient 0}
}
texture {T_Scuffs}

#end

#ifndef (T_Caution)
  // Yellow Stripe texture
  #declare T_CautionT = texture {
    pigment {
      gradient x
      color_map {
        [0 Yellow]
        [.5 Yellow]
        [.5 Black]
        [1.0 Black]
        [1.0 Yellow]
      }
    }
    finish {ambient 0}
    scale 2*foot
  }

#if (Show_Scuffs)
    #declare T_Caution =
    texture {T_CautionT}
    texture {T_Scuffs}
#else
    #declare T_Caution = texture {T_CautionT}
#end

#end


// Maneuvering Jets

// Small Jets used to help maneuver cargo and the engines as well as the whole ship when docking

#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end

#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end

#ifndef (Ship_Lights)
  #declare Ship_Lights = true;
#end

// Used to make sure that lighting is scaled properly
#ifndef (Ship_Scale_Factor)
  #declare Ship_Scale_Factor = 1;
#end

#ifndef(R1)
  #declare R1 = seed(3993);
#end


#declare Jet_Length = 1*ft;
#declare Jet_R = 3*inch;
#declare Jet_Mount_R = 3*inch;

#declare T_Jet = texture {
  pigment { Red }
}

#declare Jet = difference {
  cone { 0, 0, <0,Jet_Length,0>, Jet_R}
  cone { <0,.1*inch,0>, 0, <0, Jet_Length + 1, 0>, Jet_R }
  texture {T_Jet}
  }

#declare Jets = union {
  object {Jet}
  object {Jet rotate 90*x}
  object {Jet rotate -90*x}
  object {Jet rotate 90*z}
  object {Jet rotate -90*z}
  sphere { 0, Jet_Mount_R texture {T_Ship_White}}
  cone {<0,0,0>, 0, <0,-Jet_Length/2,0>, 3*inch texture {T_Ship_White}}
  translate Jet_Length/2*y
}


#declare SpaceShip_Pos = <-50*ft,0,100*ft>;



#declare Parabolic_Mirror = lathe {
    quadratic_spline
    5, // # of points
    <-1,.5>
    <0,0> // list of <u,v> points
    <1,.5>
    <2,2>
    <3,4.5>
    texture {T_Chrome_3D finish {ambient 0}}
}

#macro Make_Flood_Light(LightColor)                                                                                        
union {
  #if (Ship_Lights)
     object {Parabolic_Mirror translate 2*y}
     light_source {
       <0,3,0>
       color LightColor
       spotlight
       point_at <0,10,0>
       radius 30
       tightness 0
       falloff 35
       fade_power 2
       fade_distance 25*ft/Ship_Scale_Factor
       looks_like {
         sphere { <0,0,0>, .5
           pigment { LightColor * 2}
           finish {ambient LightColor*2}
           hollow
           no_shadow
         } // sphere
       } // looks_like
     } // light source
 #end // if Ship_Lights
   difference {
     cylinder {<0,0,0>, <0,2.5,0>, 1}
     cylinder {<0,.1,0>, <0,2.6,0>, .95}
     pigment {White}
   }
}
#end // macro

#local T_Cargo_Light = texture {
  pigment {White}
  finish {ambient 0}
}

#declare Cargo_Light = union {
  union {
    object {Make_Flood_Light(White*.25) rotate -117*z}
    object {Make_Flood_Light(White*.25) rotate 117*z}
    translate <0,40*in,0>
  }
  cylinder {<0,0,0>, <0,40*in,0>, 1
    texture {T_Cargo_Light}
  }
}

// Running Light

#macro Make_Real_Running_Light(LightColor)

#local Light_R = 3*inch;
#local Glass_T = .25*inch;
#local Light_H = 6*inch;

union {
  light_source {
    <0,0,0>
    color LightColor
    fade_power 2
    fade_distance 10*ft/Ship_Scale_Factor
    looks_like {
      sphere {<0,0,0>, Light_R
        pigment { LightColor * 2 }
        finish { ambient LightColor * 2 }
        hollow
        no_shadow
      } // sphere
    } // looks_like
    translate Light_H*y
  }  // light_source
  // Light Bulb Base
  cylinder { <0,0,0>, <0,Light_H,0>, 1*inch pigment {White} no_shadow}
  // Glass Cover
  #if (Show_Glass)
    merge {
      #local Glass_R = Light_R + 1*inch;
      #local Glass_I_R = Glass_R - Glass_T;
      Ring(Glass_R,Glass_I_R, Light_H)
      difference {
        sphere {0, Glass_R}
        sphere {0, Glass_I_R}
        translate Light_H*y
      }
      hollow
      texture {T_Glass1}
      interior {
        ior 1.32
      }
    }
  #end
}  
#end // macro

#macro Make_Fake_Running_Light(LightColor)

#local Light_R = 3*inch;
#local Glass_T = .25*inch;
#local Light_H = 6*inch;

union {
  sphere { <0,0,0> Light_R
        pigment { LightColor * 2 }
        finish { ambient LightColor * 2 }
  translate Light_H*y
  }
  // Light Bulb Base
  cylinder { <0,0,0>, <0,Light_H,0>, 1*inch pigment {White}}
  // Glass Cover
  #ifndef (Show_Glass)
    #declare Show_Glass = true;
  #end
  #if (Show_Glass)
    union {
      #local Glass_R = Light_R + 1*inch;
      #local Glass_I_R = Glass_R - Glass_T;
      Ring(Glass_R,Glass_I_R, Light_H)
      difference {
        sphere {0, Glass_R}
        sphere {0, Glass_I_R}
        translate Light_H*y
      }
      hollow
      texture {T_Glass1}
      interior {
        ior 1.32
      }
    }
  #end
}  
#end // macro

#include "Docking_Ring.inc"

/////////// Cargo Container //////////
#declare Cargo_Container_L = 100*ft;
#declare Cargo_Container_R = 20*ft;
 
#declare T_Cargo_Container_Cyl = texture { T_Ship_White }
#declare T_Cargo_Container_Bands = texture { T_Ship_White }
#declare T_Cargo_Container_End = texture { T_Ship_White }

#declare Cargo_Container_End = union {
 sphere { <0,0,0>, Cargo_Container_R texture {T_Cargo_Container_End translate <rand(R1)*100,rand(R1)*100,rand(R1)*100>} }
 object { Make_Fake_Running_Light(Yellow) translate Cargo_Container_R*y}
}

#declare AM_Cargo_Container_End = union {
 sphere { <0,0,0>, Cargo_Container_R texture {T_Cargo_Container_End translate <rand(R1)*100,rand(R1)*100,rand(R1)*100>} }
 #local Band_R = 20*cm;
 #local Detail_Band = torus {Cargo_Container_R Band_R texture {T_Silver_3D finish {ambient 0}}}
 object {Detail_Band translate Band_R*y}
 union {
   object {Detail_Band rotate 90*x}
   object {Detail_Band rotate 90*z}
   clipped_by {
     plane {-y, 0}
   }
 }
}

#declare Freight_Cargo_End = union {
 sphere { <0,0,0>, Cargo_Container_R texture {T_Cargo_Container_End translate <rand(R1)*100,rand(R1)*100,rand(R1)*100>} }
 object {Docking_Ring translate Cargo_Container_R*y}
}

#declare Gas_Cargo_End = union {
 sphere { <0,0,0>, Cargo_Container_R texture {T_Cargo_Container_End translate <rand(R1)*100,rand(R1)*100,rand(R1)*100>} }
 object {Nozzle_Docking_Ring translate Cargo_Container_R*y}
 #local Band_R = 20*cm;
 #local Detail_Band = torus {Cargo_Container_R Band_R texture {T_Silver_3D finish {ambient 0}}}
 object {Detail_Band translate Band_R*y}
 union {
   object {Detail_Band rotate 90*x}
   object {Detail_Band rotate 90*z}
   clipped_by {
     plane {-y, 0}
   }
 }
}
  
//////////// Cargo Take Hold Rods ///////////////
// Used by Cargo Loading Tugs
//
#declare Cargo_Take_Hold_R = 5*cm;
#declare Cargo_Take_Hold_L = Cargo_Container_L;
#declare Cargo_Take_Hold_H = .5*m;
#declare T_Cargo_Take_Hold = texture { T_Silver_3D finish {ambient 0}}

#declare Cargo_Take_Hold = union {
  #local Take_Hold_Support = cylinder {0, -Cargo_Take_Hold_H*x Cargo_Take_Hold_R}
  cylinder {0, Cargo_Take_Hold_L*y, Cargo_Take_Hold_R}
  object {Take_Hold_Support}
  sphere {0, Cargo_Take_Hold_R}
  object {Take_Hold_Support translate Cargo_Take_Hold_L*y}
  sphere {0, Cargo_Take_Hold_R translate Cargo_Take_Hold_L*y}
  object {Take_Hold_Support translate Cargo_Take_Hold_L/5*y}
  object {Take_Hold_Support translate Cargo_Take_Hold_L*2/5*y}
  object {Take_Hold_Support translate Cargo_Take_Hold_L*3/5*y}
  object {Take_Hold_Support translate Cargo_Take_Hold_L*4/5*y}
  translate Cargo_Take_Hold_H*x
}

// MakeCargo Macro
//
// pngFile - the filename of the file containing the image map to be placed on the cargo container.
//           the container is 100 x 20 so the file should be 100 high by 20*2*Pi wide.
//
// CargoType - Indicates specialization of the cargo
//             0 - Generic
//             1 - Freight - gets cargo docking ring
//             2 - Anti-Matter - has ejection pods
//             3 - Pressurized Gas or Liquid -- gets pipe hookup - female ---- 
//             
#macro MakeCargo(pngFile,CargoType,Use_Real_Lights)
#local Label_L = Cargo_Container_L;
#local Label_R = Cargo_Container_R;
#local Label_IR = Cargo_Container_R;
#local T_TextureT = texture {
  pigment {
    image_map {
      png pngFile // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 2        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
       once           // for no repetitive tiling
    } // image_map
  scale <1, Label_L,1>
  } // pigment 
  finish {ambient 0}
}
#if (Show_Scuffs)
  #local T_Texture = 
  texture {T_TextureT}
  texture {T_Scuffs}
#else
  #local T_Texture = texture {T_TextureT}
#end

union {
  union {
    #if (CargoType = 2) // AntiMatter
      union {
        cylinder {0, <0,Cargo_Container_L,0,>, Cargo_Container_R}
        #local Num_Pods = 4;
        #local Escape_Pod_Cap_R = Cargo_Container_R/2;
        #local Escape_Pod_H = 10*cm + Cargo_Container_R;
        #local Gap = (Cargo_Container_L-(Escape_Pod_Cap_R*Num_Pods*2))/(Num_Pods + 1);
        #local Spacing = Gap + Escape_Pod_Cap_R;
        #local Delta = Gap + Escape_Pod_Cap_R*2;
        #local Top_Ring_Minor_R = 5*cm;
        #local Escape_Pod_Cap = union {
          sphere {-Escape_Pod_H*z, Escape_Pod_Cap_R scale <1,1,.5>}
          cylinder {0, -Escape_Pod_H*z, Escape_Pod_Cap_R}
          torus {Escape_Pod_Cap_R, Top_Ring_Minor_R rotate 90*x translate -Escape_Pod_H*z} 
          //pigment {Red}
        }
        object {Escape_Pod_Cap translate Spacing*y}
        #local Spacing = Spacing + Delta; 
        object {Escape_Pod_Cap translate Spacing*y} 
        #local Spacing = Spacing + Delta; 
        object {Escape_Pod_Cap translate Spacing*y} 
        #local Spacing = Spacing + Delta; 
        object {Escape_Pod_Cap translate Spacing*y} 
        texture {T_Texture}
      }
    #else
      cylinder {0, <0,Cargo_Container_L,0,>, Cargo_Container_R texture {T_Texture}}
    #end
    /////// Take Holds /////////////////////////
    union {
      object {Cargo_Take_Hold 
        translate Cargo_Container_R*x
        rotate 20*y
      }
      object {Cargo_Take_Hold 
        translate Cargo_Container_R*x
        rotate -20*y
      }
    }
    union {
      object {Cargo_Take_Hold 
        translate Cargo_Container_R*x
        rotate 20*y
      }
      object {Cargo_Take_Hold 
        translate Cargo_Container_R*x
        rotate -20*y
      }
      rotate 180*y
    }
    ///////////////////////////////////////////
    
    #if (CargoType = 0)
      object {Cargo_Container_End rotate 180*z}
      object {Cargo_Container_End translate Cargo_Container_L*y}
    #end
    #if (CargoType = 1) // freight
      object {Freight_Cargo_End rotate 180*z}
      object {Freight_Cargo_End translate Cargo_Container_L*y}
    #end
    #if (CargoType = 2) // Anti-Matter
      object {AM_Cargo_Container_End rotate 180*z}
      object {AM_Cargo_Container_End translate Cargo_Container_L*y}
    #end
    #if (CargoType = 3) // Gas or Liquid
      object {Gas_Cargo_End rotate 180*z}
      object {Gas_Cargo_End translate Cargo_Container_L*y}
    #end
    texture {T_Ship_White}
    rotate -90*z // make it oriented along X-axis
    rotate 90*x
  } // union
  // cargo lights
  object { Cargo_Light translate <Cargo_Container_L/2,Cargo_Container_R,0> }
  object { Cargo_Light translate <Cargo_Container_L/2,Cargo_Container_R,0> rotate 180*x}
  // running lights
  #local Red_Light = Make_Real_Running_Light(Red);
  #local Green_Light = Make_Real_Running_Light(Green);
  object { Red_Light translate <Cargo_Container_L/2, Cargo_Container_R,0> rotate -90*x}
  object { Green_Light translate <Cargo_Container_L/2, Cargo_Container_R,0> rotate 90*x}
  translate -Cargo_Container_L*x
}
#end // macro MakeCargo


// An assembly of 4 cargo containers
#declare Cargo_Offset = Cargo_Container_R*1.6;
#declare Cargo_Spacing = 20*ft;
#declare Cargo_Quad_Len = Cargo_Container_L + Cargo_Container_R*2 + Cargo_Spacing;

#macro Make_Random_Cargo()
  #local DecalCount = 8;  
  #local value = int(rand(R1)*DecalCount);
  #local RealLights = 0;
  #local GenericCargo = 0;
  #local PressurizedCargo = 1;
  #local AMCargo = 2;
  #local LiquidGasCargo = 3;
  #if (value = 0)
    MakeCargo("He3DecalFull.png", LiquidGasCargo,RealLights)
  #end
  #if (value = 1)
    MakeCargo("HDecalFull.png", LiquidGasCargo,RealLights)
  #end
  #if (value = 2)
    MakeCargo("O2DecalFull.png",LiquidGasCargo,RealLights)
  #end
  #if (value = 3)
    MakeCargo("AMDecalFull.png",AMCargo,RealLights)
  #end
  #if (value = 4)
    MakeCargo("N2DecalFull.png",LiquidGasCargo,RealLights)
  #end
  #if (value = 5)
    MakeCargo("H2ODecalFull.png",LiquidGasCargo,RealLights)
  #end
  #if (value = 6)
    MakeCargo("SiO2DecalFull.png",GenericCargo,RealLights)
  #end
  #if (value = 7)
    MakeCargo("USSDecalFull.png",PressurizedCargo,RealLights)
  #end
#end



//--------- END INCLUDE
#end // ifndef __cargo_module__