// Persistence of Vision Ray Tracer Scene Description File
// File: Cargo_Quad.inc
// Vers: 3.5
// Desc: This is the framework that carries the cargo modules for Cargo_Module.inc in SpaceShip.inc 
// Date: 9/19/2004
// Auth: Ray Benjamin
//

#version 3.5;

#include "colors.inc"
#include "shapes.inc"
#include "shapes2.inc"
// various (mostly layered) wood textures
// T_Wood1 - T_Wood35
#include "woods.inc"
// several different gold colors, finishes and textures
#include "golds.inc"

// various metal colors, finishes and textures
// brass, copper, chrome, silver
#include "metals.inc"
#include "shapes.inc"
// various glass finishes, colors and interiors
//
#include "glass.inc"
// some standard finishes
#include "finish.inc"

#include "grid.inc"
#include "utils.inc"

#ifndef(__Cargo_Quad__)
#declare __Cargo_Quad__ = 1;

#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end
#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end

#ifndef(R1)
  #declare R1 = seed(3939);
#end


///////////////// Cargo Quad Include //////////////////////

#include "Cargo_Module.inc"


/////////// Create Cargo Quad Cradle ///////////////

#declare T_Decay = texture {
  pigment {
    bozo
    color_map {
     [0 Clear]
     [.8 Clear]
     [.95 Brown]
     [1.0 Brown]
    } // color_map
  } // pigment
  finish {
    Dull
    ambient 0
  }
}

#local T_Yellow = texture {
  //T_Silver_2D
  pigment {Yellow}
  finish {ambient 0}
}

#local T_Scrapes = texture {
    gradient x
    texture_map {
      [0 T_Yellow]
      [.5 T_Yellow]
      [.5 pigment {Black}]
      [.55 pigment {Black}]
      [.56 T_Silver_2D]
      [1.0 T_Silver_2D]
    }
    turbulence .5
    scale 3
}

#local T_Yellow_Aged = texture {
  bozo
  texture_map {
    [0 T_Yellow]
    [.5 T_Yellow]
    [.5 T_Scrapes]
    [1.0 T_Scrapes]
  }
  scale 3
}


#declare Cradle_Center_R = 6*inch;
#declare Cradle_Arm_R = 6*inch;
#declare Cradle_Arm_Offset = Cargo_Offset - Cargo_Container_R;

#declare T_Quad_Cradle = texture {
  pigment { Yellow }
  finish {ambient 0}

#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end

///////////// Utility Macros /////////////////
#macro Make_Handle(Handle_W,Handle_H, Handle_R)
  union {
    cylinder {0, Handle_W*x, Handle_R translate Handle_H*y} // cross piece
    sphere {Handle_H*y Handle_R}
    sphere {<Handle_W,Handle_H,0> Handle_R}
    cylinder {0, Handle_H*y, Handle_R} // vert
    cylinder {0, Handle_H*y, Handle_R translate Handle_W*x}
    translate <-Handle_W/2,0,0>
  }
#end // handle macro
/////////////////////////////////////

#declare Cradle_Arm = union {
  difference {
    torus { Cargo_Container_R Cradle_Arm_R }  // Torus from which we cut a section
    cylinder {<0,-(Cradle_Arm_R+1),0>, <0,Cradle_Arm_R + 1,0>, Cargo_Container_R texture {T_Yellow_Aged}} // cut out inner half
    rotate -90*x
    translate Cargo_Container_R*y
    clipped_by {                      // clip it into the short section we need
      plane { -x, 0 rotate -45*z}
    }
    translate -Cargo_Container_R*y
    rotate -45*z
    translate Cargo_Container_R*y
    rotate -90*y
  }
  cylinder {<0,0,0>, <0,-Cradle_Arm_Offset,0>, Cradle_Arm_R}
  #local Connector_R = 2.5*ft;
  // hemi circle that connects cradle arms
  difference {
    torus { Connector_R Cradle_Arm_R texture {T_Quad_Cradle} }
    difference {
      cylinder {-(Cradle_Arm_R+1)*y, (Cradle_Arm_R+1)*y, Connector_R+Cradle_Arm_R+1}
      cylinder {-(Cradle_Arm_R+2)*y, (Cradle_Arm_R+2)*y, Connector_R}
      texture {T_Yellow_Aged}
    }
    clipped_by {plane {-x,0}}
  
    rotate 90*z
    translate (Cargo_Container_R+9*ft)*y
    rotate -33.75*x 
    translate -Cargo_Container_R*y
    //texture {T_Yellow_Aged}
  }
  texture {T_Quad_Cradle}
}

#declare Cradle_Arm_Assembly = union {
  union {
    object {Cradle_Arm}
    object {Cradle_Arm translate Cargo_Container_L/3*x }
    object {Cradle_Arm translate Cargo_Container_L*2/3*x }
    object {Cradle_Arm translate Cargo_Container_L*x }
    translate Cradle_Arm_Offset*y
  }
  // Guide Rails
  #local Rail = difference {
    cylinder {<-Cradle_Arm_R,0,0>, <Cargo_Container_L+Cradle_Arm_R,0,0>, Cradle_Arm_R texture {T_Quad_Cradle}}
    box {<-(Cradle_Arm_R+1),0,-Cradle_Arm_R + 1>, <Cargo_Container_L+Cradle_Arm_R+1,Cradle_Arm_R+1,Cradle_Arm_R+1>
      texture {T_Yellow_Aged}
    } // cutting box
  } // difference
  union {
    object {Rail
      translate -(Cargo_Container_R)*y
      rotate 45*x
      translate (Cargo_Container_R+Cradle_Arm_Offset)*y
      texture {T_Yellow_Aged}
      }
    cylinder {Rail
      translate -(Cargo_Container_R)*y
      rotate -45*x
      translate (Cargo_Container_R+Cradle_Arm_Offset)*y
      texture {T_Yellow_Aged}
      }
    texture {T_Quad_Cradle}
  } // union
}

#local Cradle_Brace_R = Cradle_Arm_Offset-Cradle_Arm_R; 
#declare Cradle_Brace = torus {
  Cradle_Brace_R
  Cradle_Arm_R
  rotate 90*z
  texture {T_Quad_Cradle}
}

#local Cable_Drum_R = 1*m;
#local Cable_Drum_W = 1.5*m;
#local Cable_Motor_R = .5*m;
#local Cable_Motor_W = Cable_Drum_W;

#local C_Cable_Light_Color = color  Yellow;

#local T_Cable_Drum = texture {
  pigment {White}
  finish {
    ambient 0
  }
#if (Show_Scuffs)
}
texture {T_Scuffs}

#else
}
#end

#local T_Cable_Motor = texture {
  //pigment {Gray50}
  pigment {
    image_map {
      png "MotorDecal.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 2        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
    } // image_map
    translate -.5*y
    scale <1,Cable_Drum_W,1>
  }
  
  finish {
    ambient 0
    metallic                    
  }

#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end

#local T_Cap = texture {
  pigment { Gray50 }
  finish {
    ambient 0
    metallic
  }
#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end


#local T_Drum_Handle = texture {T_Yellow_Aged}

#local Drum_Handle = object {
  #local W = 1*m;
  #local H = 10*cm;
  #local R = 2*cm;
  Make_Handle(W,H,R)
  texture {T_Drum_Handle}
}

#local Motor_Handle = object {
  #local W = .5*m;
  #local H = 10*cm;
  #local R = 2*cm;
  Make_Handle(W,H,R)
  texture {T_Drum_Handle}
}

#local Cable_Drum = union {
  cylinder { -Cable_Drum_W/2*y, Cable_Drum_W/2*y, Cable_Drum_R}
  #local Band_R = 5*cm;
  #local Band = torus { Cable_Drum_R Band_R }
  object {Band translate .25*Cable_Drum_R*y}
  object {Band translate -.25*Cable_Drum_R*y}
  #local Cap = union {
    sphere {0 Cable_Drum_R scale <1,.25,1>}
    object {Drum_Handle translate <0,Cable_Drum_R*.25,.25*Cable_Drum_R>}
    object {Drum_Handle translate <0,Cable_Drum_R*.25,-.25*Cable_Drum_R>}
    rotate 90*y
  }
  object {Cap rotate 180*x translate -Cable_Drum_W/2*y}
  object {Cap translate Cable_Drum_W/2*y}
  texture {T_Cable_Drum}
  rotate 90*x
}

#local Cable_Motor = union {
  cylinder { -Cable_Motor_W/2*y, Cable_Motor_W/2*y, Cable_Motor_R texture {T_Cable_Motor}}
  #local Cap = union {
    sphere {0, Cable_Motor_R scale <1,.25,1>}
    object {Motor_Handle translate <0,Cable_Motor_R*.25,.25*Cable_Motor_R>}
    object {Motor_Handle translate <0,Cable_Motor_R*.25,-.25*Cable_Motor_R>}
    rotate 90*y
  }
  object {Cap rotate 180*x translate -Cable_Motor_W/2*y texture {T_Cap}}
  object {Cap translate Cable_Motor_W/2*y texture {T_Cap}}
  rotate 90*x
}

#local Fuel_Cell_Tank_R = .5*m;
#local T_Fuel_Cell_Tank = texture {
  T_Gold_2C
  finish { ambient 0 }
#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end

#local Fuel_Cell_Tank = sphere {
  0, Fuel_Cell_Tank_R
  texture {T_Fuel_Cell_Tank}
}

#local Fuel_Cell_H = 50*cm;
#local Fuel_Cell_R = 100*cm;
#local T_Fuel_Cell = texture {
  pigment { White }
  finish { ambient 0 }
#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end

#local Fuel_Cell = object {
  Round_Cylinder_Merge(<0,0,0>, <0,Fuel_Cell_H,0>, Fuel_Cell_R, 5*cm)
  texture {T_Fuel_Cell}
}

#local T_Quad_Connector_Ext = texture {
  //T_Silver_2D
  pigment {Yellow}
  finish {ambient 0}
}

#local T_Quad_Scrapes = texture {
  //T_Silver_2D
    gradient x
    texture_map {
      [0 T_Quad_Connector_Ext]
      [.5 T_Quad_Connector_Ext]
      [.5 pigment {Black}]
      [.55 pigment {Black}]
      [.56 T_Silver_2D]
      [1.0 T_Silver_2D]
    }
    turbulence .5
    scale 3
}

#local T_Quad_Connector_Int = texture {
  bozo
  texture_map {
    [0 T_Quad_Connector_Ext]
    [.5 T_Quad_Connector_Ext]
    [.5 T_Quad_Scrapes]
    [1.0 T_Quad_Scrapes]
  }
  scale 3
}

#local FQC_R = 1*m;
#local FQC_T = 10*cm;

#local Female_Quad_Connector = union {
   difference {
    sphere {0, FQC_R texture {T_Quad_Connector_Ext}}
    sphere {0, FQC_R - FQC_T texture {T_Quad_Connector_Int}}
    //cutaway_textures
    clipped_by {
      plane {x,0}
    }
  }
  torus {FQC_R-FQC_T/2, FQC_T/2 rotate 90*z texture {T_Quad_Connector_Int}}
  scale <1,1,.75>
}

#declare Cable_Assembly = union {
  union {
    object {Cable_Drum translate Cable_Drum_R*y }
    object {Cable_Motor translate <-(Cable_Drum_R+Cable_Motor_R),Cable_Motor_R,0>}
    // Gear box
    object {
      Round_Box_Merge(<0,0,0>,<Cable_Drum_R+Cable_Motor_R,Cable_Drum_R,Cable_Drum_W>,5*cm)
      texture {T_Cap}
      translate <-(Cable_Drum_R+Cable_Motor_R),0,-Cable_Drum_W/2>
    }
    // Fuel Cell Fuel Supply
    object { Fuel_Cell_Tank translate <-(Cable_Drum_R + Cable_Motor_R),-Fuel_Cell_Tank_R,0>}
    object { Fuel_Cell translate <0,-Fuel_Cell_H,0>}
    //
    object {Make_Real_Running_Light(Yellow) translate (Cable_Drum_R*2)*y}
    object {Make_Real_Running_Light(Yellow) rotate 180*x translate (-Fuel_Cell_H)*y}
    translate -Cradle_Center_R*y
  } // union
  object {Female_Quad_Connector translate (Cargo_Spacing/2 + FQC_R)*x}
}

#local Cargo_End_L = Cargo_Spacing + Cargo_Container_R;
#local R_End_Brace = cylinder {<0,Cradle_Brace_R,0>, <Cargo_End_L,0,0> Cradle_Center_R texture {T_Quad_Cradle}}
#local L_End_Brace = object {R_End_Brace rotate 180*y}

#declare Quad_Cradle = union {
  union {
    cylinder {<-(Cargo_End_L),0,0>, <Cargo_Container_L+Cargo_End_L,0,0>, Cradle_Center_R }
    sphere {0 Cradle_Center_R  translate -(Cargo_End_L)*x}
    sphere {0, Cradle_Center_R translate (Cargo_Container_L+Cargo_End_L)*x}
    texture {T_Quad_Cradle}
  }
  object {Cradle_Arm_Assembly}
  object {Cradle_Arm_Assembly rotate -90*x}
  object {Cradle_Arm_Assembly rotate 90*x}
  object {Cradle_Arm_Assembly rotate -180*x}
  
  // tori that add to the structural integrity
  object {Cradle_Brace}
  object {Cradle_Brace translate Cargo_Container_L/3*x}
  object {Cradle_Brace translate Cargo_Container_L*2/3*x}
  object {Cradle_Brace translate Cargo_Container_L*x}

  // linear braces
  object {R_End_Brace translate Cargo_Container_L*x}
  object {R_End_Brace translate Cargo_Container_L*x rotate 90*x}
  object {R_End_Brace translate Cargo_Container_L*x rotate -90*x}
  object {R_End_Brace translate Cargo_Container_L*x rotate 180*x}
  

  translate -Cargo_Container_L*x
}

#declare Lit_Quad_Cradle = union {
  union {
    union {
      cylinder {<-(Cargo_Spacing+Cargo_Container_R),0,0>, <Cargo_Container_L+Cargo_Spacing+Cargo_Container_R,0,0>, Cradle_Center_R }
      sphere {0 Cradle_Center_R  translate -(Cargo_Container_R + Cargo_Spacing)*x}
      //sphere {0, Cradle_Center_R translate (Cargo_Container_L+Cargo_Spacing+Cargo_Container_R)*x}
      texture {T_Quad_Cradle}
    }
    object {Cable_Assembly translate (Cargo_Container_L + Cargo_Container_R + Cargo_Spacing/2)*x}
  }
  object {Cradle_Arm_Assembly}
  object {Cradle_Arm_Assembly rotate -90*x}
  object {Cradle_Arm_Assembly rotate 90*x}
  object {Cradle_Arm_Assembly rotate -180*x}
  
  object {Cradle_Brace}
  object {Cradle_Brace translate Cargo_Container_L/3*x}
  object {Cradle_Brace translate Cargo_Container_L*2/3*x}
  object {Cradle_Brace translate Cargo_Container_L*x}

  // End Braces
  object {R_End_Brace translate Cargo_Container_L*x}
  object {R_End_Brace translate Cargo_Container_L*x rotate 90*x}
  object {R_End_Brace translate Cargo_Container_L*x rotate -90*x}
  object {R_End_Brace translate Cargo_Container_L*x rotate 180*x}
  object {L_End_Brace }
  object {L_End_Brace rotate 90*x}
  object {L_End_Brace rotate -90*x}
  object {L_End_Brace rotate 180*x}
  

  translate -Cargo_Container_L/2*x
  rotate 180*y
  translate -Cargo_Container_L/2*x
}

#declare Cargo_Quad_Length = (Cargo_Spacing + Cargo_Container_R + Cradle_Center_R)*2 + Cargo_Container_L;

///////////////////////////////////////////////////////////

#macro Make_Random_Cargo_Quad()
  #local Missing_Cargo_Chance = .99;
  union {
    object {Quad_Cradle}
    #if (rand(R1) < Missing_Cargo_Chance)
      object {Make_Random_Cargo() translate Cargo_Offset*y}
    #end
    #if (rand(R1) < Missing_Cargo_Chance)
      object {Make_Random_Cargo() translate Cargo_Offset*y rotate 90*x}
    #end
    #if (rand(R1) < Missing_Cargo_Chance)
      object {Make_Random_Cargo() translate Cargo_Offset*y rotate -90*x}
    #end
    #if (rand(R1) < Missing_Cargo_Chance)
      object {Make_Random_Cargo() translate Cargo_Offset*y rotate 180*x}
    #end
    rotate 90*y
  }
#end // macro

#macro Make_Cargo_Quad(Cargo1,Cargo2,Cargo3,Cargo4)
  union {
    object {Quad_Cradle}
    object {Cargo1 translate Cargo_Offset*y}
    object {Cargo2 translate Cargo_Offset*y rotate 90*x}
    object {Cargo3 translate Cargo_Offset*y rotate -90*x}
    object {Cargo4 translate Cargo_Offset*y rotate 180*x}
    /*
    bounded_by {
      cylinder {
        0, Cargo_Quad_Length*x,
        Cradle_Arm_Offset+Cargo_Container_R*2 + Cradle_Center_R+1*m // radius
        translate -(Cargo_Spacing+Cargo_Container_R + Cradle_Center_R + Cargo_Container_L)*x 
        //pigment { rgbf <1,0,0,.5> }
      }
    }
    */
    rotate 90*y
  }
#end // macro

#macro Make_Partial_Cargo_Quad(Cargo1,Cargo2,Cargo3)
  union {
    object {Lit_Quad_Cradle}
    //object {Cargo1 translate Cargo_Offset*y}
    object {Cargo1 translate Cargo_Offset*y rotate 90*x}
    object {Cargo2 translate Cargo_Offset*y rotate -90*x}
    object {Cargo3 translate Cargo_Offset*y rotate 180*x}
    rotate 90*y
  }
#end // macro

//////////////////////////////////////////////////////////
// Bulk Cargo Modules
#declare T_End_Texture = texture { 
 pigment {White} 
 finish {ambient 0}
 
#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end

#declare T_Detail = texture {
 T_Chrome_2D
 finish {ambient 0}
 
#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end


#macro Make_Bulk_Cargo(pngFile)
  #local T_Texture = texture {
    pigment {
      image_map {
        png pngFile // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
        map_type 2        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
        interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
         once           // for no repetitive tiling
      } // image_map
    scale <1, Cargo_Container_L,1>
    } // pigment 
    finish {ambient 0}
  }
  #local Radius_Mult = 2.5;
  #local Bulk_R = Radius_Mult * Cargo_Container_R;
  union {
    // Main Cylinder
    union {
      cylinder {0, Cargo_Container_L*y, 
        Cargo_Container_R*Radius_Mult // Radius
      }
      #local Detail_R = .25*m;
      // Detail Pipes
      #local Detail_Pipe = cylinder {0, Cargo_Container_L*y, Detail_R 
        //texture {T_Detail}
        translate Bulk_R*x
      }
      object {Detail_Pipe}
      object {Detail_Pipe rotate 180*y}
      object {Detail_Pipe rotate 120*y}
      object {Detail_Pipe rotate -120*y}
      object {Detail_Pipe rotate 90*y}
      object {Detail_Pipe rotate -90*y}
      object {Detail_Pipe rotate 45*y}
      object {Detail_Pipe rotate -45*y}
      texture {T_Texture}
      rotate -90*z
    }
    #local EndCap = union {
      union {
        sphere { 0, Bulk_R texture {T_End_Texture}}
        torus { Bulk_R Detail_R texture {T_Detail}}
        scale <1/Radius_Mult,1,1>
      } // union
      torus { Bulk_R Detail_R 
        texture {T_Detail}
        rotate 90*z
      }
    } // union
    object {EndCap }
    object {EndCap translate Cargo_Container_L*x}
    // Central Support Rod
    cylinder {-(Cargo_Spacing + Cargo_Container_R)*x,
              (Cargo_Container_L + Cargo_Container_R + Cargo_Spacing)*x,
              Cradle_Center_R // radius
      texture {T_Quad_Cradle}
    }
    // Running Lights
    #local Red_Light = Make_Real_Running_Light(Red);
    #local Green_Light = Make_Real_Running_Light(Green);
    object { Red_Light translate <Cargo_Container_L/2, Bulk_R+Detail_R,0> rotate -90*x}
    object { Green_Light translate <Cargo_Container_L/2, Bulk_R+Detail_R,0> rotate 90*x}
    
    translate -Cargo_Container_L*x
    rotate 90*y
  }
#end
//////////////////////////////////////////////////////////


/////------------------------------------------------------
#end // ifndef __Cargo_Quad__