// Persistence of Vision Ray Tracer Scene Description File
// File: Ceramic_Mug.inc
// Vers: 3.5
// Desc: A Blue Ceramic Mug
// Date: 9/4/2004
// Auth: Ray Benjamin

#include "shapes.inc"

// Mug
#local Mug_Radius = 3.125/2*inch;
#local Mug_Height = 3.5*inch;
#local Mug_Thickness = .25*inch;
#local Mug_Handle_R = .375*inch/2;
#local Mug_Handle_Start_H = .75*inch;
#local Mug_Handle_End_H = 3*inch;
#local Mug_Handle_H = Mug_Handle_End_H - Mug_Handle_Start_H;
#local Mug_Handle_D = 1.5*inch;

#local CM_Saturn =  color_map {
      [0  rgb <255/256, 194/256, 87/256>] // first band
      [.15 rgb <255/256, 194/256, 87/256>] //
      [.16 rgb <234/256, 177/256, 87/256>] // 1.5
      [.19 rgb <234/256, 177/256, 87/256>] //
      [.2 rgb < 221/256, 165/256, 76/256> ] // second band
      [.26 rgb < 221/256, 165/256, 76/256> ] //
      [.27 rgb < 201/256, 151/256, 77/256 > ] // third band
      [.31 rgb < 201/256, 151/256, 77/256 > ] //
      [.32 rgb <224/256, 169/256, 87/256 > ] // fourth
      [.36 rgb <224/256, 169/256, 87/256 > ] //
      [.37 rgb <221/256, 166/256, 85/256> ] // fifth
      [.38 rgb <221/256, 166/256, 85/256> ] //
      [.39 rgb <227/256, 169/256, 91/256> ] // sixth
      [.40 rgb <227/256, 169/256, 91/256> ] //
      [.41 rgb <219/256, 163/256, 87/256> ] // seventh
      [.44 rgb <219/256, 163/256, 87/256> ] //
      [.45 rgb <218/256, 165/256, 100/256> ] // eight
      [.49 rgb <218/256, 165/256, 100/256> ] //
      [.50 rgb <215/256, 163/256, 87/256> ] // nine
      [.57 rgb <215/256, 163/256, 87/256> ] // 
      [.58 rgb <214/256, 158/256, 84/256> ] // ten
      [.67 rgb <214/256, 158/256, 84/256> ] // 
      [.68 rgb <209/256, 163/256, 101/256>] // eleven
      [.75 rgb <209/256, 163/256, 101/256>] //
      [.76 rgb <188/256, 145/256, 92/256> ] // twelve
      [.85 rgb <188/256, 145/256, 92/256> ] // 
      [.86 rgb <174/256, 131/256, 86/256> ] // thirteen
      [.88 rgb <174/256, 131/256, 86/256> ] // thirteen
      
      [.9 rgb <160/256, 123/256, 90/256> ]
      [.95 rgb <58/256, 46/256, 51/256>]
      [1 rgb <49/256, 38/256, 46/256>]
    }

#local Band_Function = function(y) { abs(y)}
#local T_Saturn = texture {
  pigment {
    function { Band_Function(y) }
    color_map { CM_Saturn }
    turbulence .1
    scale Mug_Height
    scale <3,1,3>
  } // pigment
  finish { 
    reflection .04
    specular .75
    roughness .01
  }
} // texture

#declare T_Mug_Outside = texture {
  pigment { rgb <.7,.5,.05> } // mustard yellow
  finish { 
    reflection .04
    specular .75
    roughness .01
  }
}

#declare T_Mug_Decal = texture {
  pigment {
    image_map {
      png "MugDecal.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 2        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
    } // image_map 
    scale <1,Mug_Height,1>
    rotate -90*y
  }
  finish { 
    reflection .04
    specular .75
    roughness .01
  }
}

#declare T_Mug_Inside = texture {
  pigment { rgb <.8,.8,.8> } // very light gray
  finish { 
    reflection .05
    specular 1
    roughness .01
  }
}

#declare Mug_Handle = // create a curved tube object translating a sphere along a certain path
sphere_sweep {
  //linear_spline                 // linear curve
  cubic_spline                // alternative spline curves
  //b_spline
  9,                            // number of specified sphere positions
  <-1,1,0>,                            Mug_Handle_R*1.2
  <0,0,0>,                             Mug_Handle_R*1.5
  <Mug_Handle_D/2,0,0>,                Mug_Handle_R
  <Mug_Handle_D*7/8,Mug_Handle_H/5,0>,   Mug_Handle_R
  <Mug_Handle_D,Mug_Handle_H/2,0>      Mug_Handle_R    // center point
  <Mug_Handle_D*7/8,Mug_Handle_H*4/5,0>, Mug_Handle_R
  <Mug_Handle_D/2,Mug_Handle_H,0>,     Mug_Handle_R
  <0, Mug_Handle_H, 0>,                Mug_Handle_R*1.5                
  <-1, Mug_Handle_H-1, 0>,               Mug_Handle_R*1.2
  //tolerance 0.001             // optional 
  texture {T_Mug_Outside}
  scale <1,1,2>
}



#declare Mug = difference {
  #local Inner_Radius = Mug_Radius-Mug_Thickness/2;
  merge {
    Round_Cylinder_Merge(<0,0,0>, <0,Mug_Height,0>, Mug_Radius, .1*inch)
    object {Mug_Handle translate <Mug_Radius,Mug_Handle_Start_H,0>}
    texture {T_Mug_Outside}
  }
  object {
    Round_Cylinder_Merge(<0,.25*inch,0>, <0,Mug_Height+.1*inch>, Inner_Radius, .1*inch)
    texture {T_Mug_Inside}
  }
}

#declare Saturn_Mug = difference {
  #local Inner_Radius = Mug_Radius-Mug_Thickness/2;
  merge {
    Round_Cylinder_Merge(<0,0,0>, <0,Mug_Height,0>, Mug_Radius, .1*inch)
    object {Mug_Handle translate <Mug_Radius,Mug_Handle_Start_H,0> texture {T_Saturn}}
    texture {T_Saturn}
  }
  object {
    Round_Cylinder_Merge(<0,.25*inch,0>, <0,Mug_Height+.1*inch>, Inner_Radius, .1*inch)
    texture {T_Mug_Inside}
  }
}


#declare Steam_H = 4*in;
#declare Steam_Bottom_R = 1.5*in;
#declare Steam_Top_R = 2*in;

#declare M_Steam = material {
  texture {
    pigment { Clear }
  }
  interior {
    media {
      scattering { 3 .5 }
      
      density {
        //cylindrical
        spiral1 1
        density_map {
          [0 rgb 0]
          [0.7 rgb 0]
          [0.9 rgb <1,1,1>]
          [1 rgb <1,1,1>]
        } // density map
        rotate 90*x
        turbulence .7
        scale <1,1,1>
      } // density 
      /*
      density {
        cylindrical
        scale 2
      }
      */
      density {
        spherical
        scale <2,4,2>
      }
            
    }  // media
  } // interior
} // material

#declare Steam = cone {
  <0,0,0> Steam_Bottom_R
  <0,Steam_H,0> Steam_Top_R
  hollow
  material { M_Steam }
  translate Mug_Height*y
}

#declare Steaming_Mug = union {
  object {Saturn_Mug}
  object {Steam}
}
