// Persistence of Vision Ray Tracer Scene Description File
// File: Comm_Laser.pov
// Vers: 3.5
// Desc: Communications Laser
// Date: 9/7/2004
// Auth: Ray Benjamin
//


#ifndef (__comm_dish__)
#declare __comm_dish__ = 1;

#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end

#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end

// Parabaloid
#local Dish_H = 1*m;

#declare T_Comm_Equip = texture { pigment { rgb<1,1,1> }}

#declare Receiver_Pole_H = 1.2*m;
#declare Receiver_Pole_R = 3*cm;
#declare Receiver_Ball_R = 10*cm;
#declare Receiver_Cone_H = 10*cm;
#declare Receiver_Cone_R = 7*cm;

#declare Receiver_Pole = union {
  cylinder {0, Receiver_Pole_H*y, Receiver_Pole_R }
  cone { <0,0,0>, Receiver_Pole_R, <0,Receiver_Cone_H,0> Receiver_Cone_R translate Receiver_Pole_H*y}
  texture {T_Comm_Equip}
}

#declare Receiver_R = 20*cm;
#declare Receiver_T = 1*cm;

#declare Receiver_Ring_Major_R = 20*cm;
#declare Receiver_Ring_Minor_R = 1.5*cm;
#declare Receiver_Ring_H = 12*cm;

#declare Receiver_Support_R = 1.5*cm;
#declare Receiver_Support_L = 171*cm;
#declare Receiver_Support_Angle = -20.5;
#declare Receiver_Support_H = Receiver_Ring_H + Receiver_R + -9*cm;

#declare Receiver_Support = cylinder {0 Receiver_Support_L*x Receiver_Support_R }

#declare Receiver_Support_Assembly = merge {
  object { Receiver_Support rotate Receiver_Support_Angle*z}
  object { Receiver_Support rotate Receiver_Support_Angle*z rotate 90*y}
  object { Receiver_Support rotate Receiver_Support_Angle*z rotate -90*y}
  object { Receiver_Support rotate Receiver_Support_Angle*z rotate 180*y}
  translate Receiver_Support_H*y
}

#declare Receiver = merge {
  difference {
    sphere {<0,0,0> Receiver_R}
    sphere {<0,0,0> Receiver_R - Receiver_T}
    clipped_by {
      plane {-y,-4*cm}
    }
  }
  torus {Receiver_Ring_Major_R Receiver_Ring_Minor_R translate Receiver_Ring_H*y}
  object {Receiver_Support_Assembly}
  texture {T_Comm_Equip}
  translate (Receiver_Pole_H + 15*cm)*y
}

#declare Receiver_Lip_Major_R = 1.6*m;
#declare Receiver_Lip_T = 3*cm;
#declare Receiver_Lip_H = 1*m;

#declare Comm_Dish_Reflector = union {
  object {Paraboloid_Y clipped_by {plane {y, 1*m} }  texture {T_Gold_1A}  scale <10,1,10>}
  object {Receiver_Pole}
  object {Receiver}
  // Dish Lip Support
  torus {Receiver_Lip_Major_R Receiver_Lip_T texture {T_Comm_Equip} translate Receiver_Lip_H*y}
}

#local Dish_Stand_H = 1*m;
#local Dish_Stand_R = 3*cm;
#local Dish_Stand_Upright = cylinder {0, Dish_Stand_H*y, Dish_Stand_R pigment {White}}

#declare Dish_Support_L = 2*m;
#declare Dish_Support_R = 2*cm;
#declare Dish_Support_Angle = -60;

#declare Dish_Support = cylinder {0 Dish_Support_L*x Dish_Support_R}
#declare Dish_Supports = union {
  object {Dish_Support rotate Dish_Support_Angle*z}
  object {Dish_Support rotate Dish_Support_Angle*z rotate 90*y}
  object {Dish_Support rotate Dish_Support_Angle*z rotate -90*y}
  object {Dish_Support rotate Dish_Support_Angle*z rotate 180*y}
  texture {T_Comm_Equip}
}

#declare Comm_Dish = union {
  object {Comm_Dish_Reflector rotate -20*z translate Dish_Stand_H*y}
  object {Dish_Stand_Upright}
  object {Dish_Supports translate Dish_Stand_H*y}
}


////////////////-----------------------------------------------------
#end // ifndef __comm_dish__