// Persistence of Vision Ray Tracer Scene Description File
// File: Comm_Laser.pov
// Vers: 3.5
// Desc: Communications Laser
// Date: 9/7/2004
// Auth: Ray Benjamin
//


#ifndef (__comm_laser__)


#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end

#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end

#declare Laser_Body_L = 25*cm;
#declare Laser_Body_H = 10*cm;
#declare Laser_Body_D = 10*cm;

#declare Laser_Power_LED = sphere {0 2*mm pigment { Red } finish { Luminous } }

#declare Laser_Body = union {
  box {0 <Laser_Body_L,Laser_Body_H,Laser_Body_D> pigment { White }}
  object {Laser_Power_LED translate <1*cm,Laser_Body_H/2,0>} 
  translate -<Laser_Body_L/3,Laser_Body_H/2,Laser_Body_D/2>
  rotate 23*z
}

#declare Comm_Laser_Housing_R = .5*m;
#declare Comm_Laser_Housing_T = .5*cm;

#declare Laser_Base_H = 10*cm;
#declare Laser_Base_R = Comm_Laser_Housing_R - Comm_Laser_Housing_T - 2*cm;
#declare Laser_Base = cylinder {0 <0,Laser_Base_H,0> Laser_Base_R pigment {White}}

#declare Laser_Mount_H = Comm_Laser_Housing_R*.5;
#declare Laser_Mount_R = 2*cm;


#declare Detector_R = 10*cm;
#declare Detector = sphere {
  0 Detector_R
  texture {
    pigment { Gray50 }
    //normal { facets  size .3  }
  } // texture
} // sphere


#declare Comm_Laser = union {
  // Protective Housing
  
  difference {
    sphere {0 Comm_Laser_Housing_R}
    sphere {0 Comm_Laser_Housing_R - Comm_Laser_Housing_T}
    clipped_by {
      plane { -y, 0}
    }
    texture {T_Glass1}
  }
  
  object {Laser_Base}
  object {Laser_Body translate Laser_Mount_H*y}
  // connecting rod
  cylinder {0, <0,Laser_Mount_H+Laser_Body_H/4,0> Laser_Mount_R
    texture {T_Silver_3C}
  }
  union {
    object {Detector translate 30*cm*x}
    object {Detector translate 30*cm*x rotate -90*y}
    object {Detector translate 30*cm*x rotate 90*y}
    object {Detector translate 30*cm*x rotate -180*y}
    object {Detector translate 30*cm*x rotate 180*y}
    object {Detector translate 30*cm*x rotate -45*y}
    object {Detector translate 30*cm*x rotate 45*y}
    object {Detector translate 30*cm*x rotate -135*y}
    object {Detector translate 30*cm*x rotate 135*y}
    translate Laser_Base_H*y
  }
}


////////////////-----------------------------------------------------
#end // ifndef __comm_laser__