// Persistence of Vision Ray Tracer Scene Description File
// File: LCD_Monitor.pov
// Vers: 3.5
// Desc: LCD Monitor model
// Date: 8/29/2004
// Auth: Ray Benjamin

#ifndef (__lcd_monitor__)
#declare __lcd_monitor__ = 1;

#ifndef (LCD_Frame_Only)
  #declare LCD_Frame_Only = false;
#end

/////// LCD Monitor ////////////////////

#declare P_Computer_Beige = rgb <1,1,.8>;

#declare T_Office_Scuffs = texture {
    pigment {
        granite
        color_map {
          /*
            [0.0 White]
            [0.08 White]
            [0.08 Clear]
            [0.8 Clear]
            [0.92 Black]
            [1.0 Gray10]
          */
          [0 White]
          [.15 Clear]
          [.75 Clear]
          [1 Gray50]
        } // color_map
        turbulence 1
        scale 10
        scale <3,1,1>
    } // pigment
} // texture T_Scuffs

#declare T_Computer_Plastic = 
texture {
  pigment { P_Computer_Beige }
  normal { bumps .01 }
}
texture {T_Office_Scuffs}

#declare T_Monitor_Bezel = texture { T_Computer_Plastic normal {bumps .1} translate 5*inch*y}

#local Screen_W = 23*inch;
#local Screen_H = 20*inch;
#local Screen_D = .25*inch;

#declare T_Screen = texture {
  pigment {
    image_map {
      png "CalculatedDreamsWallpaper.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 0        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
      // once           // for no repetitive tiling
      //filter all 1.0
      //transmit all .1
      // [filter N V]   // N=all or color index # (0...N), V= value (0.0...1.0)
      // [transmit N V] // N=all or color index # (0...N), V= value (0.0...1.0)
      // [use_color | use_index]
    } // image_map
      scale <Screen_W*.87, Screen_H*.85,1>
      translate <-0*inch,-0*inch,0>
  }
  finish {
    diffuse 0
    ambient 1.0
    //conserve_energy
  }
}

#declare LCD_Monitor_Back = union {
  #local Monitor_Back_W = Screen_W-1;
  #local Monitor_Back_H = Screen_H-1;
  prism {
    conic_sweep
    linear_spline
    .5,
    1,
    5,
    <Monitor_Back_W/2,Monitor_Back_H/2>,
    <-Monitor_Back_W/2, Monitor_Back_H/2>,
    <-Monitor_Back_W/2, -Monitor_Back_H/2>,
    <Monitor_Back_W/2,-Monitor_Back_H/2>,
    <Monitor_Back_W/2,Monitor_Back_H/2>
    rotate 180*x
    translate 1*y
    scale <1,1.5,1>
    rotate 90*x
    translate Screen_H/2*y
    texture {T_Computer_Plastic}
  }
  box {0 <Monitor_Back_W, Monitor_Back_H, Screen_D> texture {T_Computer_Plastic}
    translate <-Monitor_Back_W/2,.5,-Screen_D>
  }
}


#declare T_LCD_Button = texture {T_Monitor_Bezel}

#declare LCD_Button_R = .25*inch;
#declare LCD_Power_Button_R = LCD_Button_R * 1.5;
#declare LCD_Button_Hole_R = LCD_Button_R + .02;
#declare LCD_Power_Button_Hole_R = LCD_Button_Hole_R * 1.5;
#declare LCD_Button_Gap = 1*inch;
#declare LCD_PB_Mark_Width = .05;
#declare LCD_PB_Mark_Z = -LCD_Power_Button_R*.85;

#declare LCD_Button = sphere {
  <0,0,0>, LCD_Button_R
  scale <1,1,.5>
  texture {T_LCD_Button}
}

#declare Triangle_Cutout = prism {
  linear_sweep
  0,
  1,
  4
  <-1,-1>,<1,-1>,<-0,1>, <-1,-1>
  scale .5
  rotate -90*x
  translate 2*mm*y
}

#declare LCD_Up_Button = difference {
  sphere { <0,0,0>, 1}
  object {Triangle_Cutout translate <0,0,-.7> pigment {Blue} }
  scale LCD_Button_R
  scale <1,1,.25>
  texture {T_LCD_Button}
}

#declare LCD_Down_Button = difference {
  sphere { <0,0,0>, 1}
  object {Triangle_Cutout rotate -180*z translate <0,0,-.7> pigment {Blue}}
  scale LCD_Button_R
  scale <1,1,.25>
  texture {T_LCD_Button}
}

#local One_Cutout = text {
  ttf             // font type (only TrueType format for now)
  "crystal.ttf",  // Microsoft Windows-format TrueType font file name
  "1",      // the string to create
  1,              // the extrusion depth
  0               // inter-character spacing
  //pigment {Red}
  translate <-.25,-.25,0>
  scale 1.5
  translate 1*mm*x
}

#local Two_Cutout = text {
  ttf             // font type (only TrueType format for now)
  "crystal.ttf",  // Microsoft Windows-format TrueType font file name
  "2",      // the string to create
  1,              // the extrusion depth
  0               // inter-character spacing
  //pigment {Red}
  translate <-.25,-.25,0>
  scale 1.5
  translate 1*mm*x
}

#declare LCD_One_Button = difference {
  sphere { <0,0,0>, 1}
  object {One_Cutout translate <0,0,-1.25> pigment {Black} }
  scale LCD_Button_R
  scale <1,1,.25>
  texture {T_LCD_Button}
}

#declare LCD_Two_Button = difference {
  sphere { <0,0,0>, 1}
  object {Two_Cutout translate <0,0,-1.25> pigment {Black} }
  scale LCD_Button_R
  scale <1,1,.25>
  texture {T_LCD_Button}
}

#declare LCD_Power_Button = union {

  difference {
    sphere { <0,0,0>, LCD_Power_Button_R texture {T_LCD_Button}}
    difference {
      #local Circle_Outer_R = LCD_Power_Button_R * .6;
      #local Circle_Inner_R = Circle_Outer_R - LCD_PB_Mark_Width;
      
      cylinder {<0,0,-1>, <0,0,0> Circle_Outer_R}
      cylinder {<0,0,-1>, <0,0,0> Circle_Inner_R}
    }
    #local Box_H = Circle_Inner_R*2 - LCD_PB_Mark_Width*2;
    box {<-LCD_PB_Mark_Width/2,-Box_H/2,-1>, <LCD_PB_Mark_Width/2,Box_H/2,0>}
  }
  
  sphere { <0,0,0> LCD_Power_Button_R*.95 texture {pigment {Green} finish {Luminous}}}
  scale <1,1,.25>
}

#declare LCD_Button_Hole = sphere {<0,0,0>, LCD_Button_Hole_R }
#declare LCD_Power_Button_Hole = sphere {<0,0,0>, LCD_Power_Button_Hole_R }

/// Dream View Sticker ///

#declare DreamView_Decal = height_field {
  png               // the file type to read (tga/pot/pgm/ppm/png/sys)
  "DreamViewDecalHF.png"     // the file name to read
  smooth        // smooth surface normal
  water_level .1 // truncate/clip below N (0.0 ... 1.0)
  texture {
    pigment {
        image_map {
          png "DreamViewDecal.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
          map_type 0        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
          interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
        } // image_map
        rotate 90*x
    } // pigment
    finish { F_MetalC }
  }
//  translate VECTOR | rotate VECTOR | scale VECTOR
  scale <4,.01,1>
  rotate -90*x
}


#declare LCD_Computer_Screen = union {
  #local Display_W = 20*inch;
  #local Display_H = 17*inch;
  #local Display_D = .5*inch;
  #local Display_Off_X = (Screen_W - Display_W)/2;
  #local Display_Off_Y = (Screen_H - Display_H)/2;
  #local Display_Off_Z = 1/8*inch;
  
  #local Bezel_Rounding = .25*cm;
  #local Z_Offset = -Bezel_Rounding*2/3;
  
  #local Lower_Bezel_Center_Pos = <Display_Off_X+Display_W/2,Display_Off_Y/2,.1*inch>;
  
  // Bezel/Frame
  union {
    difference {
      Round_Box_Merge(<0,0,0>, <Screen_W,Screen_H,Screen_D>,Bezel_Rounding)
      //box {<0,0,0>, <Screen_W,Screen_H,Screen_D>}
      box {<0,0,-1>, <Display_W, Display_H, 1> translate <Display_Off_X,Display_Off_Y,0>} // cutout for display
      
      // button holes
      union {
        object {LCD_Button_Hole translate Lower_Bezel_Center_Pos - LCD_Button_Gap*2*x}
        object {LCD_Button_Hole translate Lower_Bezel_Center_Pos - LCD_Button_Gap*x}
        object {LCD_Button_Hole translate Lower_Bezel_Center_Pos} // center button
        object {LCD_Button_Hole translate Lower_Bezel_Center_Pos + LCD_Button_Gap*x}
        object {LCD_Power_Button_Hole translate Lower_Bezel_Center_Pos + LCD_Button_Gap*2.5*x}
        translate Z_Offset*z
      }
      
      texture {T_Monitor_Bezel}
    }
    // Buttons
    union {
      object {LCD_One_Button translate Lower_Bezel_Center_Pos - LCD_Button_Gap*2*x}
      object {LCD_Up_Button translate Lower_Bezel_Center_Pos - LCD_Button_Gap*x}
      object {LCD_Down_Button translate Lower_Bezel_Center_Pos}
      object {LCD_Two_Button translate Lower_Bezel_Center_Pos + LCD_Button_Gap*x}
      object {LCD_Power_Button translate Lower_Bezel_Center_Pos + LCD_Button_Gap*2.5*x}
      // Mfr Decal
      object {DreamView_Decal translate <Display_Off_X,Display_H+Display_Off_Y+.25*inch,0>}
      translate Z_Offset*z
    }
    //
  } // bezel union
   
 #if (LCD_Frame_Only = false)
  // LCD Screen
  union {
    box { <0,0,0>, <Display_W, Display_H, .01 >
      texture {T_Screen}
    }
    box {<0,0,.01>, <Display_W, Display_H, Display_D>
      pigment {White}
      finish {ambient 1 diffuse 0}
    }
    translate <Display_Off_X, Display_Off_Y, Display_Off_Z>
  }
 #end // if frame only
}

#declare LCD_Display_Assembly = union {
  object {LCD_Computer_Screen translate -Screen_W/2*x}
  #if (LCD_Frame_Only = false)
    object {LCD_Monitor_Back translate Display_D*z}
  #end
}

// LCD Stand

#declare Riser_Major_R = 2;
#declare Riser_Minor_R = .5;
#declare X_Scale = 6;
#declare Riser = difference {
  torus { Riser_Major_R Riser_Minor_R scale <1,X_Scale,1> rotate 90*z
    clipped_by {
      plane {-z,0 rotate 20*x}
      //plane {y,0}
    } // clip
  } // torus
  cylinder {<0,0,-1>, <0,0,Riser_Major_R+Riser_Minor_R+1>, Riser_Major_R translate -X_Scale/2*x}
  cylinder {<0,0,-1>, <0,0,Riser_Major_R+Riser_Minor_R+1>, Riser_Major_R translate X_Scale/2*x}
  translate <0,Riser_Major_R+Riser_Minor_R,0>
  texture {T_Computer_Plastic}
}

#declare LCD_Hinge = union {
  #local Hinge_Len = 5;
  #local Hinge_R = .75;
  cylinder {0, Hinge_Len*x, Hinge_R}
  sphere {0 Hinge_R}
  sphere {0 Hinge_R translate Hinge_Len*x}
  texture {T_Computer_Plastic}
  translate -(Hinge_Len/2)*x
}

#declare LCD_Stand = union {
  object {Riser}
  object {LCD_Hinge translate 1.5*z rotate -50*x translate 2.95*y}
  torus {4-.125, .125 translate .125*y scale <1.5,1,1> texture {T_Chrome_3D}}
  cone {<0,.125,0> 4, <0,1.1,0> 1.8  scale <1.5,1,1> texture {T_Computer_Plastic}}
}

#declare LCD_Monitor = union {
  #if (LCD_Frame_Only = false)
    object {LCD_Stand}
  #end
  object {LCD_Display_Assembly translate 2*y}
}


////////////----------------------------
#end // __lcd_monitor__