// Persistence of Vision Ray Tracer Scene Description File
// File: ThumbTack.pov
// Vers: 3.5
// Desc: Basic Scene Example
// Date: 9/5/2004
// Auth: Ray Benjamin
//

#version 3.5;

#ifndef (__paperclip__)
#declare __paperclip__ = 1;

#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end
#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end

#declare R1 = seed(3939);

global_settings {
  assumed_gamma 1.0
}

// ----------------------------------------

//////////// Paper Clip //////////////////

#local PC_R = .35;
#local PC_C1_L = 14;
#local PC_T1_Maj_R = 2.5*(PC_R*2);
#local PC_C2_L = 12.5;
#local PC_T2_Maj_R = 2.0 * (PC_R*2);
#local PC_C3_L = 11 + PC_R;
#local PC_T3_Maj_R = 1.5 * (PC_R*2);
#local PC_C4_L = 8;

#local PC_C1_Pos = <5,0,PC_R>;
#local PC_T1_Pos = <PC_C1_L, 0, PC_T1_Maj_R + PC_R>;

#local PC_C2_Pos = <PC_T2_Maj_R + PC_R,0,PC_T1_Maj_R*2 + PC_R>;
#local PC_T2_Pos = <PC_T2_Maj_R + PC_R,0,PC_T2_Maj_R+PC_R*3>;

#local PC_C3_Pos = <PC_T2_Pos.x, 0, PC_R*3>;
#local PC_T3_Pos = <13,0, PC_T3_Maj_R+PC_R+PC_R*2>;

#local PC_C4_Pos = <5,0,PC_T3_Pos.z + PC_T3_Maj_R>;

#local Paper_Clip_L = PC_C1_L + PC_T1_Maj_R;
#local Paper_Clip_D = (PC_T1_Maj_R + PC_R)*2;

#declare T_Paper_Clip = texture { T_Chrome_3D }

#declare Paper_Clip = union {
  cylinder {<4,0,0>, (PC_C1_L-1)*x, PC_R translate PC_C1_Pos} // C1 - Most External Straight Run
  torus {PC_T1_Maj_R PC_R                           // T1
    clipped_by { plane {-x,0} } 
    translate PC_T1_Pos
  }
  cylinder {0, PC_C2_L*x, PC_R translate PC_C2_Pos } // C2
  torus {PC_T2_Maj_R PC_R                          // T2
    clipped_by { plane {x,0} }
    translate PC_T2_Pos
  }
  cylinder {0, PC_C3_L*x, PC_R translate PC_C3_Pos } // C3
  torus {PC_T3_Maj_R PC_R                          // T3
    clipped_by { plane {-x,0} }
    translate PC_T3_Pos
  }
  cylinder {0, PC_C4_L*x, PC_R translate PC_C4_Pos } // C4

  texture {T_Paper_Clip}
  translate <-Paper_Clip_L/2,PC_R,-Paper_Clip_D/2>
  scale 1/15
}

#declare Bent_Paper_Clip = union {
  #local T1_Outward_Bend_Angle = 20;
  #local T1_Upward_Bend_Angle = 8;
  
  //sphere {0, .5 pigment {Red}}
  
  torus {PC_T1_Maj_R PC_R // T1-A non-rotated section of largest bend in paper clip
    clipped_by {
      plane {-z,0}
      plane {-x,0}
    }
    //pigment {Blue}
    translate PC_T1_Pos
  }
  union { // outer bend and outer cylinder which are bent out and up
    cylinder {-(PC_C1_L-5)*x, 0, PC_R
      translate -PC_T1_Maj_R*z
      rotate -T1_Outward_Bend_Angle*y // Bend the wire outward
      //pigment {Orange}
    }
    difference {
      torus {PC_T1_Maj_R PC_R                       // T1-B large bend of paper clip cut at angle where it's bent outward
        clipped_by { plane {-x,0} } 
      }
      box {<0,-1,-1>, <-1,1,-3> rotate -10*y}
      clipped_by {
        plane {z,0} // clip it, since this part is rotated up
      }
      //pigment {White}
      //translate PC_T1_Pos
    }
    translate -PC_T1_Maj_R*x // move to origin
    rotate -T1_Upward_Bend_Angle*z  // bend upward the torus section and the wire
    translate PC_T1_Maj_R*x         // move back
    translate PC_T1_Pos             // put whole thing in place
  } // union
  
  cylinder {0, PC_C2_L*x, PC_R translate PC_C2_Pos } // C2
  torus {PC_T2_Maj_R PC_R                          // T2
    clipped_by { plane {x,0} }
    translate PC_T2_Pos
  }
  cylinder {0, PC_C3_L*x, PC_R translate PC_C3_Pos } // C3
  torus {PC_T3_Maj_R PC_R                         // T3
    clipped_by { plane {-x,0} }
    translate PC_T3_Pos
  }
  cylinder {0, PC_C4_L*x, PC_R translate PC_C4_Pos} // C4
  
  texture {T_Paper_Clip}
  translate <-Paper_Clip_L/2,PC_R,-Paper_Clip_D/2>
  scale 1/15
}



//---------------------------------------
#end // ifndef __paperclip__