// Persistence of Vision Ray Tracer Scene Description File
// File: Pen.inc
// Vers: 3.5
// Desc: Basic Scene Example
// Date: 9/5/2004
// Auth: Ray Benjamin
//

#ifndef (__pen_by_pov__)
#declare __pen_by_pov__ = 1;

#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end
#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end


/////////////////////////////////////////////////////////
//// Pen ////

#declare Pen_L = 5.25*in;
#declare Cap_L = 2.125*in;
#declare Pen_R = 5/32*in;
#declare Cap_Bottom_R = 7/32*in;
#declare Cap_Top_R = 6/32*in;
#declare Cap_T = 1/32*in;
#declare Pen_Body_L = 3.25*in;
#declare Pen_Body_Bottom_R = 5/32*in;
#declare Pen_Body_Top_R = Cap_Bottom_R - 1/32*in;
#declare End_Cap_T = 1/32*in; // end cap thickness


#declare T_Pen_Cap = texture {
  pigment {Gray30}
  finish {
    brilliance 10
    specular .03125
    reflection { .01, .2 }
    conserve_energy
  }
}

#declare T_Pen_Body = texture {
  pigment {
    image_map {
      png "PenDecal.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 2        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
    } // image_map
    scale <1, 3.5, 1>
    //rotate 90*z
  }
  finish {
    brilliance 5
    specular .0625
    reflection { .01, .2 }
    conserve_energy
  }
}

#declare T_End_Cap = texture {
  pigment {Black}
  finish {
    brilliance 5
    specular .0625
    reflection { .01, .2 }
    conserve_energy
  }
}

#local T_CE = texture { T_Glass2 
  finish { 
    reflection { .01, .2 }
    conserve_energy
  }
}

#local T_Interior = texture {
  pigment {Gray30}
  finish {
    brilliance 5
    specular .0625
    reflection { .01, .2 }
    conserve_energy
  }
}

#local T_White_Plastic = texture {
  pigment {White}
  finish {
    brilliance 5
    specular .0625
    reflection { .01, .2 }
    conserve_energy
  }
}


#local Clip_T = 1/32;
#local Clip_W = 3/16;
#local Clip_R = 3/32;
#local Clip_EH = 1/8;
#local Clip_MH = 1/8;
#local Clip_BH = Clip_MH - Clip_T;
#local Clip_Scale = <1,1/6,1>;
#local Clip_Tip_H = 3/32;
#local Clip_L = 7/4;


#declare Pen_Clip = union {
  box {<0,0,-1/32>, <Clip_T,Clip_W,Clip_EH> translate <-Clip_T,0,0>}
  box {<0,0,0>, <-Clip_L,Clip_W,Clip_T> translate <0,0,Clip_EH>}
  difference {
    cylinder {<0,0,0>, <0,Clip_W,0>, 1/16}
    cylinder {<0,-1,0>, <0,Clip_W+1,0>, 1/32}
    box {<0,-.1,0>, <.5,Clip_W+1,Clip_T*2.2> translate -Clip_T*1.2*z}
    translate <-Clip_L,0,Clip_EH-1/32>
  }
  box {<0,0,0>, <Clip_W*1.2,Clip_W,Clip_T> rotate 9*y translate <-Clip_L,0,Clip_EH-3/32>}
  box {<0,0,0>, <4/32,Clip_W,Clip_T> rotate -30*y translate <-Clip_L+7.25/32,0,0>}
  texture {T_Silver_3D}
  rotate 90*z
  rotate -90*y
}



#declare Pen_Cap = union {
  difference {
    cone {<0,0,0>, Cap_Bottom_R, <0,Cap_L-End_Cap_T,0>, Cap_Top_R texture {T_Pen_Cap}}
    cylinder {<0,0,-1>, <0,0,1>, 3/32 translate (Cap_L-4/16)*y}
  }
  cylinder {<0,-.45,0>, <0,End_Cap_T,0>, (Cap_Top_R - 1/64*in) texture {T_End_Cap}
    translate <0,Cap_L-End_Cap_T,0>
  }
    
  object {Pen_Clip translate <-Cap_Top_R,Cap_L-End_Cap_T,0>}
}

#local CE_L = 1.25*in;
#local CE_O_R = Pen_Body_Top_R - 1/32*in;
#local CE_T = 1/32*in;
#local CE_I_R = CE_O_R - CE_T; 

#local T_Metal_Tip = texture {T_Silver_2D}

#local T_Black_Tip = texture {
  pigment { Black }
}

#declare Pen_Body = union {
  // end cap
  cylinder {0, <0,End_Cap_T,0>, Pen_Body_Bottom_R-1/64*in texture {T_End_Cap}}
  // pen body
  cone {<0,End_Cap_T,0>, Pen_Body_Bottom_R, <0,Pen_Body_L,0>, Pen_Body_Top_R texture {T_Pen_Body}}
  // clear plastic section
  difference {
    cylinder {<0,0,0>, <0,CE_L,0>, CE_O_R}
    cylinder {<0,-.1,0>, <0,CE_L+.1,0>, CE_I_R}
    hollow
    interior {I_Glass2}
    texture {T_CE}
    translate Pen_Body_L*y
  }
  union {
    merge {
      cylinder {0, 3/16*y, CE_I_R}
      cylinder {0, CE_L*y, 1/32}
      //
      cylinder {0, .5*y, 2/32 translate 1.125*y}
      // Thin detail cylinders in clear section
      #local Current_Y = 3/16;
      #local End_Y = 1.25;
      #local Delta_Y = 1/64;
      #local Current_Delta_Y = Delta_Y;
      #local Delta_Inc = Delta_Y/20;
      
      #while (Current_Y <  End_Y)
        cylinder {0, Delta_Y*y CE_I_R translate Current_Y*y}
        #local Current_Y = Current_Y + Current_Delta_Y*2;
        #local Current_Delta_Y = Current_Delta_Y + Delta_Inc;
      #end
      // Thicker cylinders making up tip
      #local Thickness = 2/64;
      #local Current_Y = 0;
      #local Gray_Tip_H = 3/8;
      intersection {
        union {
          #while (Current_Y < Gray_Tip_H)
            cylinder {0, Thickness*y, CE_I_R translate Current_Y*y}
            #local Current_Y = Current_Y + Thickness*2;
          #end
        } // union
        cone {0, CE_I_R, .75*y,0}
        translate End_Y*y
      }
      texture {T_Interior}
    }// merge
    // white tip cone
    #local White_Tip_H = 3/32;
    #local White_Tip_Start_R = 3/64;
    #local White_Tip_End_R = 1/32;
    cone {0, White_Tip_Start_R, White_Tip_H*y, White_Tip_End_R texture {T_White_Plastic} translate (End_Y + Gray_Tip_H)*y}
    // Metal Tip
    #local Metal_Tip_R = White_Tip_End_R/2;
    #local Metal_Tip_L = 1/8;
    #local Black_Tip_R = Metal_Tip_R*.8;
    cylinder { 0, Metal_Tip_L*y, Metal_Tip_R texture {T_Metal_Tip} translate (End_Y + Gray_Tip_H + White_Tip_H)*y}
    sphere {0, Black_Tip_R texture {T_Black_Tip} translate (End_Y + Gray_Tip_H + White_Tip_H + Metal_Tip_L) *y}
    translate Pen_Body_L*y
  } // union
}

#declare Pen = union {
  object {Pen_Body}
  object {Pen_Cap translate Pen_Body_L*y}
}

#declare Horizontal_Pen = object {
  Pen
  rotate 90*z
  rotate 47*x
  translate <Pen_Body_L,Cap_Bottom_R+1/32,0>
  rotate -2*z
  //translate <Cap_L,0,0>
}

#declare Horizontal_Pen_Body = object {
  Pen_Body
  rotate -70*y
  rotate 89.5*z
  translate <0,Pen_Body_Bottom_R,0>
}


////-----------------------------------------
#end  // ifndef