// Persistence of Vision Ray Tracer Scene Description File
// File: Pencil.inc
// Vers: 3.5
// Desc: Pencil Model
// Date: 9/7/2004
// Auth: Ray Benjamin

#ifndef (__pencil__)

#declare __pencil__ = 1;

#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end

#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end

#ifndef (ShowPencilText)
  #declare ShowPencilText = true;
#end

#declare Pencil_T = 4*mm;

// Eraser Crimp
#declare Eraser_Crimp_L = 1.4*cm;
#declare Eraser_Crimp_T = .025*mm;
#declare Eraser_Crimp_R = Pencil_T;
#declare Eraser_Crimp_Ring_R = .35*mm;
#declare T_Eraser_Crimp = texture {
  T_Chrome_2C
  finish {
    irid { .25 thickness .25 turbulence .5 }
  }
}
 
#declare Eraser_Crimp = difference {
  union {
    cylinder {0, Eraser_Crimp_L*x, Eraser_Crimp_R}
    #declare Eraser_Crimp_Ring = torus {Eraser_Crimp_R Eraser_Crimp_Ring_R rotate 90*z}
    #local Eraser_Rings = union {
      object {Eraser_Crimp_Ring translate 0*mm*x}
      object {Eraser_Crimp_Ring translate 1*mm*x}
    }
    object {Eraser_Rings translate 3.25*mm*x}
    object {Eraser_Rings translate 1*cm*x}
  }
  #local Eraser_Crimp_Cyl_R = .17*mm;
  #local Eraser_Crimp_Cylinder = cylinder {0, 4.5*mm*x Eraser_Crimp_Cyl_R translate <4.75*mm,Eraser_Crimp_R,0>}
  #local Count = 0;
  #local Angle = 0;
  #local Delta = 12;
  #while (Count < 30)
    object {Eraser_Crimp_Cylinder rotate Angle*x}
    #local Count = Count + 1;
    #local Angle = Angle + 12;
  #end // while
  object {Eraser_Crimp_Cylinder}
  texture {T_Eraser_Crimp}
}

#declare Eraser_L = 6*mm;
#declare Eraser_R = Pencil_T - Eraser_Crimp_T*2;
#declare T_Eraser = texture {
  pigment { rgb <.9,.5,.5> }
  finish {
    diffuse .5
    roughness 1
  }
}
texture {
  pigment {
    bozo
    color_map {
      [0 Clear]
      [.7 Clear]
      [.85 rgbf <.7,.7,.7,.5>]
      [1.0 rgbf <1,1,1,.9>]
    } // color_map
    turbulence .1
    scale 4*mm
  }
  finish {
    diffuse .5
    roughness 1
  }
}
// Eraser
#declare Eraser = object {
  Round_Cylinder_Merge(<0,0,0>, <Eraser_L,0,0>, Eraser_R, .85*mm)
  texture {T_Eraser}
}

// Eraser Assembly

#declare Eraser_Assembly = union {
  object {Eraser_Crimp}
  object {Eraser translate Eraser_Crimp_L*x}
}

#declare Eraser_Assembly_L = Eraser_Crimp_L + Eraser_L;

// Pencil Lead

#declare Pencil_L = 6.5*inch - Eraser_Assembly_L;
#declare Paint_T = Pencil_T - .1*mm;
#declare Pencil_Lead_R = .06*inch;
#declare Pencil_Tip_L = .5*inch;

#declare T_Pencil_Lead = texture {
  pigment {Gray15}
  finish {
    diffuse .5
    roughness .3
  } // finish
} // texture

// Pencil Sharpener - used to cut away pencil at tip to "sharpen" it.

#declare Pencil_Sharpener = union {
  #local Blade = plane {-y,0}
  #local AngleZ = 20;
  #local AngleZ_Delta = 5;
  #local AngleX = 0;
  #local AngleX_Delta = 3;
  
  #while (AngleX < 360)
    object { Blade rotate AngleZ+rand(R1)*AngleZ_Delta*z rotate AngleX*x}
    #local AngleX = AngleX + rand(R1)*AngleX_Delta;
  #end // while
/*  
  plane {x, 0 translate 3*mm*x 
    //rotate -2*z
  }
*/
}

// Cutout for Lead

#declare Lead_Cylinder = cylinder {-(Pencil_L/2 + .1*mm)*x Pencil_L/2*x Pencil_Lead_R } 

#declare Unsharpened_Pencil_Lead = cylinder {-(Pencil_L/2 + .1*mm)*x Pencil_L/2*x Pencil_Lead_R texture {T_Pencil_Lead}}
#declare Raw_Pencil_Lead = union {
  cylinder {-(Pencil_L/2-3*mm)*x Pencil_L/2*x Pencil_Lead_R}
  sphere {3*mm*x, .68*mm translate -(Pencil_L/2)*x}
  texture {T_Pencil_Lead}
}
#declare Pencil_Lead = union {
  difference {
    object {Raw_Pencil_Lead}
    object {Pencil_Sharpener translate -Pencil_L/2*x}
    cutaway_textures
  }
} // union


#if (ShowPencilText)

// create a TrueType text shape
#local Pencil_Text = " DREAMERS    Scrivener  2004   (No. 2)";
#declare Pencil_Lettering = text {
  ttf             // font type (only TrueType format for now)
  "crystal.ttf",  // Microsoft Windows-format TrueType font file name
  Pencil_Text,      // the string to create
  1,              // the extrusion depth
  0               // inter-character spacing
  translate <0,-.4,0>
  //scale .125
  scale .19 // .15
  pigment {Black}
  translate <-Text_Space("crystal.ttf",Pencil_Text,.2,0)/2+2*cm,0,-Pencil_T>
}

#end // if showpenciltext

// Pencil Color
#local P_Pencil = rgb <1,221/255, 63/255>;

// Make the Wood portion of the Pencil

#macro Make_Pencil_Wood(WoodTexture)
  difference {
      union {
        // Paint
        
        difference {
          object {Hexagon scale <Pencil_L/2,Pencil_T,Pencil_T>}
          object {Hexagon scale <Pencil_L/2*1.1,Pencil_T-.1*mm,Pencil_T - .1*mm>}
          pigment { P_Pencil }
        } // diff
        // Wood
        object {Hexagon scale <Pencil_L/2,Pencil_T-.1*mm,Pencil_T - .1*mm> texture {WoodTexture rotate 90*y}}
        // lead
      } // union
      
      object {Pencil_Sharpener translate -Pencil_L/2*x}
      //object {Pencil_Lettering_Decal }
      cutaway_textures
      
  } // difference
#end

#macro Make_Unsharpened_Pencil_Wood(WoodTexture)
  union {
    // Paint
    
    difference {
      object {Hexagon scale <Pencil_L/2,Pencil_T,Pencil_T>}
      object {Hexagon scale <Pencil_L/2*1.1,Pencil_T-.1*mm,Pencil_T - .1*mm>}
      pigment { P_Pencil }
    } // diff
    // Wood
    object {Hexagon scale <Pencil_L/2,Pencil_T-.1*mm,Pencil_T - .1*mm> texture {WoodTexture rotate 90*y}}
    // lead
  } // union
#end

// Macro to make actual pencil given the wood texture  
// Should add length, paint color, lettering color to parameters
#macro Make_Pencil(WoodTexture)
  union {
  
    difference {
      object {Make_Pencil_Wood(WoodTexture)}
      object {Lead_Cylinder}
      #if (ShowPencilText)
        object {Pencil_Lettering translate <0*cm,0,-.01>}  // Plain Lettering
      #end
    }
  
    object {Pencil_Lead}
    //
    object {Eraser_Assembly translate Pencil_L/2*x}
    
    bounded_by {
      cylinder {-Pencil_L/2*x, (Pencil_L/2+Eraser_Assembly_L)*x Pencil_T + .03}
    }
    
    rotate 30*x
    translate (Pencil_T)*y
  }
#end

#macro Make_Unsharpened_Pencil(WoodTexture)
  union {
  
    difference {
      object {Make_Unsharpened_Pencil_Wood(WoodTexture)}
      object {Lead_Cylinder}
      #if (ShowPencilText)
        object {Pencil_Lettering translate <0*cm,0,-.01>}  // Plain Lettering
      #end
    }
    //object {Pencil_Lettering translate <0*cm,0,-.01>}  // Plain Lettering
    object {Unsharpened_Pencil_Lead}
    //
    object {Eraser_Assembly translate Pencil_L/2*x}

    rotate 30*x
    translate (Pencil_T)*y
  }
#end

#declare Old_Pencil = Make_Pencil(T_Wood6)
#declare New_Pencil = Make_Pencil(T_Wood19)
#declare Old_Unsharpened_Pencil = Make_Unsharpened_Pencil(T_Wood6)


//////-----------------------------------------------
#end // ifndef __pencil__