// Persistence of Vision Ray Tracer Scene Description File
// File: Pencil.inc
// Vers: 3.5
// Desc: Pencil Model
// Date: 9/7/2004
// Auth: Ray Benjamin

#ifndef (__popcan__)
#declare __popcan__ = 1;

#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end

#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end


#local Main_Body_H = 9.7*cm;
#local Main_Body_R = 3.25*cm;
#local PopCan_T = .015*mm;
#local Transition_Minor_R = 1*mm;
#local Transition_Major_R = Main_Body_R - Transition_Minor_R;

#local Bottom_Torus_Major_R = 2.5*cm;
#local Bottom_Torus_Minor_R = 2*mm;

#local Inner_Bottom_H = 3*mm;
#local Inner_Bottom_Upper_R = Bottom_Torus_Major_R-Bottom_Torus_Minor_R - 1*mm;

#local Bottom_H = 6*mm;

#local Top_Cone_H = 1.5*cm;
#local Top_Ring_H = 2*mm;
#local Top_Cone_R = 55/2*mm;

#declare PopCan_H = Main_Body_H + Bottom_H + Top_Cone_H + Top_Ring_H;
#declare PopCan_R = Main_Body_R;

#local T_PopCan_Body = texture {
  pigment {
    image_map {
      png "DreamColaDecal.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 2        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
      // once           // for no repetitive tiling
      // [filter N V]   // N=all or color index # (0...N), V= value (0.0...1.0)
      // [transmit N V] // N=all or color index # (0...N), V= value (0.0...1.0)
      // [use_color | use_index]
    } // image_map
    scale <PopCan_R,Main_Body_H+Top_Cone_H,1>
    translate Bottom_H*y
  } // pigment
  finish {
    metallic
    reflection .08
  }
}

#local T_PopCan_Bottom = texture {T_Silver_3A}


#declare PopCan_Bottom = union {
  // top part of bottom section
  torus {Transition_Major_R Transition_Minor_R translate Bottom_H*y}
  // connection cone
  cone {Bottom_Torus_Minor_R*y, Bottom_Torus_Major_R+Bottom_Torus_Minor_R,
        Bottom_H*y, Transition_Major_R+Transition_Minor_R open}
  // bottom torus which contacts the table top
  torus {Bottom_Torus_Major_R Bottom_Torus_Minor_R translate Bottom_Torus_Minor_R*y}
  // cone from bottom torus to bottom hemisphere
  cone {Bottom_Torus_Minor_R*y, Bottom_Torus_Major_R-Bottom_Torus_Minor_R
        Bottom_H*y, Inner_Bottom_Upper_R open}
  // bottom hemisphere
  difference {
    sphere {0, Inner_Bottom_Upper_R}
    sphere {0, Inner_Bottom_Upper_R - PopCan_T}
    clipped_by {
      plane {-y,0}
    }
    scale <1,.3,1>
    translate Bottom_H*y
  }
  texture {T_PopCan_Bottom}
}

#declare T_PopCan_Top = texture {T_Silver_3B}

#local PopTop_Ring_Indent_H = 5*mm;
#local PopTop_Ring_Indent_Big_R = 1.1*cm; 
#local PopTop_Ring_Indent_Small_R = .75*cm;
#local PopTop_Ring_Indent_Separation = 2*cm;
 
#declare PopTop_Ring_Indent = union {

  cylinder {0 PopTop_Ring_Indent_H*y PopTop_Ring_Indent_Big_R}
  cylinder {0 PopTop_Ring_Indent_H*y PopTop_Ring_Indent_Small_R
    translate PopTop_Ring_Indent_Separation*z
  }
  prism {                                
    linear_sweep  // or conic_sweep for tapering to a point
    linear_spline // linear_spline | quadratic_spline | cubic_spline | bezier_spline 
    0,         // height 1
    PopTop_Ring_Indent_H,         // height 2
    5,           // number of points
    // (--- the <u,v> points ---)
    <-PopTop_Ring_Indent_Big_R,0>,
    <PopTop_Ring_Indent_Big_R,0>,
    <PopTop_Ring_Indent_Small_R,PopTop_Ring_Indent_Separation>,
    <-PopTop_Ring_Indent_Small_R,PopTop_Ring_Indent_Separation>,
    <-PopTop_Ring_Indent_Big_R,0>
  }
}

#local PT_Outer_R = 6*mm;
#local PT_Inner_R = PT_Outer_R - 1.2*mm;
#local PT_Rivet_Mount_R = PT_Inner_R - 1.5*mm;
#local PT_Rivet_R = PT_Rivet_Mount_R - .5*mm;
#local PT_T = 1*mm;
#local Offset = 12*mm;

#declare PT_Cutout = cylinder {-1*mm*y, 2*mm*y, PT_Inner_R} 

#declare PT_Bottom = union {
  cylinder {0, PT_T*y, PT_Outer_R
    clipped_by {
      plane {z,0}
    }
  }
  cylinder {0, PT_T*y, PT_Rivet_Mount_R}
  sphere {0, PT_Rivet_R scale <1,.75,1>}
}

#declare Pull_Tab = union {
  difference {
    union {
      cylinder {0, PT_T*y, PT_Outer_R} 
      cylinder {0, PT_T*y, PT_Outer_R translate Offset*z}
      box {<-PT_Outer_R,0,0>, <PT_Outer_R,PT_T,Offset>}
    }
    object {PT_Cutout translate Offset*z}
    object {PT_Cutout}
    //cylinder {-1*mm*y, 2*mm*y, PT_Inner_R translate Offset*z}
  }
  
  object {PT_Bottom}  
  
  texture {T_PopCan_Bottom}
}

#declare PopCan_Top = union {
  torus {Top_Cone_R 1*mm scale <1,2,1>}
  cone {0, Top_Cone_R+1*mm,
        -5*mm*y, Top_Cone_R open}
  #local Crease_Major_R = Top_Cone_R;
  #local Crease_Minor_R = 1.5*mm;
  #local Crease_H = -5*mm;
  #local Inside_Disc_H = Crease_H + 2*mm;
  // crease
  difference {
    torus {Crease_Major_R Crease_Minor_R}
    torus {Crease_Major_R (Crease_Minor_R - 1*mm)}
    clipped_by {
      plane {y,0}
    }
    translate Crease_H*y
  }
  // other inside rise
  #local Cone_Bottom_R = Crease_Major_R - Crease_Minor_R;
  #local Cone_Top_R = Cone_Bottom_R - 2*mm;
  cone {Crease_H*y, Cone_Bottom_R,
        Inside_Disc_H*y,Cone_Top_R open}
  // Top of can
  cylinder {-5*mm*y, Inside_Disc_H*y, Cone_Top_R}
  difference {
    cylinder {Inside_Disc_H*y, (Inside_Disc_H+1*mm)*y, Cone_Top_R - 2*mm}
    object {PopTop_Ring_Indent translate <0,Inside_Disc_H,-8*mm>}
  }
  object {Pull_Tab translate Inside_Disc_H*y}
  texture {T_PopCan_Top}
}

#declare PopCan = union {

  union {
    difference {
      cylinder {0, Main_Body_H*y, Main_Body_R translate Bottom_H*y}
      cylinder {0, (Main_Body_H+1)*y, Main_Body_R-PopCan_T translate (Bottom_H - 1*mm)*y}
    }
    torus {Transition_Major_R Transition_Minor_R translate (Main_Body_H + Bottom_H)*y}
    cone {0, Transition_Major_R + Transition_Minor_R
          Top_Cone_H*y, Top_Cone_R open translate (Main_Body_H + Bottom_H)*y }
    texture {T_PopCan_Body}
  }

  object {PopCan_Bottom}
  object {PopCan_Top translate (Main_Body_H + Bottom_H + Top_Cone_H)*y}
}

#declare Droplet_R = 2*mm;

#declare T_Droplet = texture {
  T_Glass1
}
#declare Droplet = sphere { 0 Droplet_R hollow texture {T_Droplet}
  interior {
    ior 1.5
  }
}

#declare PopCan_Wet = union {
  object {PopCan}
  // droplets
  #local Drop_Count = 2000;
  #local Count = 0;
  merge {
  #while (Count < Drop_Count)
    object {Droplet scale rand(R1) translate <PopCan_R,rand(R1)*(PopCan_H/2-1*cm),0> rotate rand(R1)*360*y}
    #local Count = Count + 1;
  #end
  }
  //object {Droplet translate <PopCan_R,PopCan_H/2+1*cm,0> rotate 95*y}
  //object {Droplet translate <PopCan_R,PopCan_H/2-1.5*cm,0> rotate 98*y}
}

#local Condensation_T = .05*mm;
#local T_Condensation = texture {
  pigment { Clear }
  finish  {
      ambient 0.1
      diffuse 0.1
      reflection 0.1
      specular 0.8
      //roughness 0.003
      roughness .125
      phong 1
      phong_size 400
  }
  normal {
    bumps .01
  }
}

#declare Condensation_Light = cylinder {
  <0,0,0>
  <0,Main_Body_H/2,0>
  Main_Body_R + Condensation_T
  hollow
  interior {
    ior 1.5
    fade_distance 1.0
    fade_power 2
  }
  texture {T_Condensation}
  translate Bottom_H*y
}

#declare PopCan_Condensation = union {
  object {PopCan}
  object {Condensation_Light}
}

//////-----------------------------------------------
#end // ifndef __pencil__