// Persistence of Vision Ray Tracer Scene Description File
// File: Saturn.pov
// Vers: 3.5
// Desc: This is a model of Saturn with the rings
// Date: 9/5/2004
// Auth: Ray Benjamin

// ----------- Saturn ----------------------

#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end
#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end

#declare Saturn_Actual_Radius = 60268*km;

#ifndef (Saturn_R)
  #declare Saturn_R = Saturn_Actual_Radius;
#end

#local CF = Saturn_R/Saturn_Actual_Radius;


#declare Ring_A_Inner_R = 122200*km*CF;
#declare Ring_A_Outer_R = 136800*km*CF;

#declare Ring_B_Inner_R = 92000*km*CF;
#declare Ring_B_Outer_R = 117500*km*CF;
#declare Ring_B_T = 1*km*CF;

#declare Ring_C_Inner_R = 74500*km*CF;
#declare Ring_C_Outer_R = 92000*km*CF;

#declare Ring_D_Inner_R = 66000*km*CF;
#declare Ring_D_Outer_R = 73150*km*CF;

#declare Ring_E_Inner_R = 180000*km*CF;
#declare Ring_E_Outer_R = 480000*km*CF;

#declare Ring_F_Inner_R = 140210*km*CF;
#declare Ring_F_Outer_R = 140700*km*CF;

#declare Ring_G_Inner_R = 164000*km*CF;
#declare Ring_G_Outer_R = 172000*km*CF;



//#local R1 = seed(399);

#declare CM_Saturn =  color_map {
      [0  rgb <255/256, 194/256, 87/256>] // first band
      [.15 rgb <255/256, 194/256, 87/256>] //
      [.16 rgb <234/256, 177/256, 87/256>] // 1.5
      [.19 rgb <234/256, 177/256, 87/256>] //
      [.2 rgb < 221/256, 165/256, 76/256> ] // second band
      [.26 rgb < 221/256, 165/256, 76/256> ] //
      [.27 rgb < 201/256, 151/256, 77/256 > ] // third band
      [.31 rgb < 201/256, 151/256, 77/256 > ] //
      [.32 rgb <224/256, 169/256, 87/256 > ] // fourth
      [.36 rgb <224/256, 169/256, 87/256 > ] //
      [.37 rgb <221/256, 166/256, 85/256> ] // fifth
      [.38 rgb <221/256, 166/256, 85/256> ] //
      [.39 rgb <227/256, 169/256, 91/256> ] // sixth
      [.40 rgb <227/256, 169/256, 91/256> ] //
      [.41 rgb <219/256, 163/256, 87/256> ] // seventh
      [.44 rgb <219/256, 163/256, 87/256> ] //
      [.45 rgb <218/256, 165/256, 100/256> ] // eight
      [.49 rgb <218/256, 165/256, 100/256> ] //
      [.50 rgb <215/256, 163/256, 87/256> ] // nine
      [.57 rgb <215/256, 163/256, 87/256> ] // 
      [.58 rgb <214/256, 158/256, 84/256> ] // ten
      [.67 rgb <214/256, 158/256, 84/256> ] // 
      [.68 rgb <209/256, 163/256, 101/256>] // eleven
      [.75 rgb <209/256, 163/256, 101/256>] //
      [.76 rgb <188/256, 145/256, 92/256> ] // twelve
      [.85 rgb <188/256, 145/256, 92/256> ] // 
      [.86 rgb <174/256, 131/256, 86/256> ] // thirteen
      [.88 rgb <174/256, 131/256, 86/256> ] // thirteen
      
      [.9 rgb <160/256, 123/256, 90/256> ]
      [.95 rgb <58/256, 46/256, 51/256>]
      [1 rgb <49/256, 38/256, 46/256>]
    }

#declare Band_Func = function(y) { abs(y)}
#local T_Saturn = texture {
  pigment {
    function { Band_Func(y) }
    color_map { CM_Saturn }
    turbulence .1
    scale Saturn_R
    scale <3,1,3>
  } // pigment
} // texture

#local Ring_Color = rgb <.7,.65,.46>;// old => rgb <.59,.445,.359>;

#local Ring_Color_Map = color_map {
      [0 Ring_Color + .9*t ]
      [.1 Ring_Color + .5*t ]
      [.12 Ring_Color + .5*t ]
      [.2 Ring_Color + .8*t ]
      [.2 Ring_Color + .2*t]
      [.3 Ring_Color + .1*t ]
      [.33 Ring_Color + .1*t]
      [.4 Ring_Color + .9*t ]
      [.45 Ring_Color + .7*t ]
      [.48 Ring_Color + .1*t]
      [.5 Ring_Color + .1*t ]
      [.55 Ring_Color + 0*t]
      [.56 Ring_Color + 0*t]
      [.56 Ring_Color + .2*t]
      [.58 Ring_Color + .5*t]
      [.59 Ring_Color + .1*t]
      [.6 Ring_Color + .9*t ]
      [.63 Ring_Color + .95*t]
      [.65 Ring_Color + .5*t]
      [.7 Ring_Color + .1*t ]
      [.75 Ring_Color + .2*t]
      [.78 Ring_Color + .8*t]
      [.8 Ring_Color + .9*t ]
      [.83 Ring_Color + .7*t]
      [.9 Ring_Color + .1*t ]
      [1  rgbt <.59, .445, .359, .9> ]
    }

#local Ring_Color_Map_B = color_map {
      [0 Ring_Color + .9*t ]
      [.1 Ring_Color + .5*t ]
      [.15 Ring_Color + .5*t ]
      [.2 Ring_Color + .8*t ]
      [.25 Ring_Color + .2*t]
      [.3 Ring_Color + .1*t ]
      [.33 Ring_Color + .1*t]
      [.4 Ring_Color + .9*t ]
      [.45 Ring_Color + .7*t ]
      [.48 Ring_Color + .1*t]
      [.5 Ring_Color + .1*t ]
      [.55 Ring_Color + 0*t]
      [.56 Ring_Color + 0*t]
      [.56 Ring_Color + .2*t]
      [.58 Ring_Color + .5*t]
      [.59 Ring_Color + .1*t]
      [.6 Ring_Color + .9*t ]
      [.63 Ring_Color + .95*t]
      [.65 Ring_Color + .5*t]
      [.7 Ring_Color + .1*t ]
      [.75 Ring_Color + .2*t]
      [.78 Ring_Color + .8*t]
      [.8 Ring_Color + .9*t ]
      [.83 Ring_Color + .7*t]
      [.9 Ring_Color + .1*t ]
      [1  rgbt <.59, .445, .359, .9> ]
    }

#local T_Ring_A = texture {
  pigment {
    onion
    color_map {CM_Saturn}
    turbulence .01
    scale Ring_A_Inner_R
    frequency 8
  } // pigment
  finish {
    diffuse .30
    roughness .15 
  }
  //normal { bumps .1 }
}

#local T_Ring_B = texture {
  pigment {
    onion
    color_map {CM_Saturn}
    turbulence .01
    scale Ring_B_Inner_R
    frequency 4
  } // pigment
  finish {
    diffuse .55
    roughness .15 
  }
  //normal { bumps .1 }
}

#local T_Ring_C = texture {
  pigment {
    onion
    color_map {CM_Saturn}
    turbulence .01
    scale Ring_C_Inner_R
    frequency 8
  } // pigment
  finish {
    diffuse .16
    roughness .15 
  }
  //normal { bumps .1 }
}

#local T_Ring_D = texture {
  pigment {
    onion
    color_map {CM_Saturn}
    turbulence .01
    scale Ring_D_Inner_R
    frequency 8
  } // pigment
  finish {
    diffuse .20
    roughness .15 
  }
  //normal { bumps .1 }
}


#declare C_Ring_A = rgb <255/256, 194/256, 87/256>;// old rgb <151/256,141/256,142/256>;

#declare C_Ring_B = rgb <252/256,224/256,212/256>;

#declare C_Ring_C = rgb <174/256, 160/256, 159/256>;

#declare C_Ring_D = rgb <56/256, 54/256, 65/256>;
/*
#declare Ring_Media = media {
  // (---general values---)
  intervals 10           // number of intervals used for sampling [10]
  samples 1,1            // minimum and maximum number of samples taken per interval [1,1]
  confidence 0.9         // statistic parameter higher->better quality [0.9]
  variance 1.0/128       // statistic parameter lower->better quality [1.0/128]
  ratio 0.9              // distribution between lit and unlit areas [0.9]
  
  scattering {           // scattering media
    4,                   // scattering type 1=isotropic; 2=Mie haze; 3=Mie murky
                         // 4=Rayleigh; 5=Henyey-Greenstein
    rgb Ring_Color       // color
  }
  method 3               // adaptive sampling
  aa_threshold 0.1     // accuracy threshold for method 3 [0.1]
  aa_level 4           // maximum recursion depth for method 3 [4]
  
  density {
    onion
    color_map { Ring_Color_Map }
    turbulence .01
    scale Ring_A_Inner_R
  }
}
*/

/*
#local M_Ring_B = material {
  texture { 
    pigment { Clear }
    finish {
      diffuse .5
    }
  }
}



#macro Make_Ring(Outer_R,Inner_R,Thickness,Material)
  #local T2 = Thickness/2;
  #local T3 = T2 + 1;
  difference {
    cylinder {<0,-T2,0>, <0,T2,0> Outer_R }
    cylinder {<0,-T3,0>, <0,T3,0> Inner_R }
    hollow
    material { Material }
  }
#end // macro Make_Ring
*/

#declare Ring_A = disc { 0, <0,1,0>, Ring_A_Outer_R, Ring_A_Inner_R texture {T_Ring_A}} 
#declare Ring_B = disc { 0, <0,1,0>, Ring_B_Outer_R, Ring_B_Inner_R texture {T_Ring_B}}
#declare Ring_C = disc { 0, <0,1,0>, Ring_C_Outer_R, Ring_C_Inner_R texture {T_Ring_C}} 
#declare Ring_D = disc { 0, <0,1,0>, Ring_D_Outer_R, Ring_D_Inner_R texture {T_Ring_D}}
 
#declare Ring_E = disc { 0, <0,1,0>, Ring_E_Outer_R, Ring_E_Inner_R texture {T_Ring_A}} 
#declare Ring_F = disc { 0, <0,1,0>, Ring_F_Outer_R, Ring_F_Inner_R texture {T_Ring_A}} 
#declare Ring_G = disc { 0, <0,1,0>, Ring_G_Outer_R, Ring_G_Inner_R texture {T_Ring_A}} 



#declare Saturn = union {
  sphere { <0,0,0> Saturn_R texture {T_Saturn}}
  object { Ring_A }
  object { Ring_B }
  object { Ring_C }
  object { Ring_D }
//  object { Ring_E }
//  object { Ring_F }
//  object { Ring_G }
}



