// Persistence of Vision Ray Tracer Scene Description File
// File: SpaceShip.inc
// Vers: 3.5
// Desc: Space Ship
// Date: 9/5/2004
// Auth: Ray Benjamin

#ifndef(__spaceship__)
  #declare __spaceship__ = 1;

#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end
#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end


#ifndef (Ship_Lights)
  #declare Ship_Lights = true;
#end

#ifndef (Show_Cockpit_Light)
  #declare Show_Cockpit_Light = true;
#end

#ifndef (Show_Scuffs)
  #declare Show_Scuffs = true;
#end

#ifndef (Show_Dirt)
  #declare Show_Dirt = true;
#end

#ifndef (Engine_On)
  #declare Engine_On = true;
#end

#ifndef (Show_Train)
  #declare Show_Train = true;
#end

#ifndef (R1)
  #declare R1 = seed(3883);
#end

#ifndef (SpaceAmbient)
  #declare SpaceAmbient = 0;
#end 

#include "Airlock.inc"

// ------------- Space Ship --------------------------
#declare T_Scuffs = texture {
#if (Show_Scuffs)
    pigment {
        granite
        color_map {
          /*
            [0.0 White]
            [0.08 White]
            [0.08 Clear]
            [0.8 Clear]
            [0.92 Black]
            [1.0 Gray10]
          */
          [0 White]
          [.15 Clear]
          [.75 Clear]
          [1 Black]
        } // color_map
        turbulence 1
        scale 1000
        scale <30,1,1>
    } // pigment
#else
  pigment {Clear}
#end    
    finish {
     ambient SpaceAmbient
    }
    
#if (Show_Dirt)
} // texture T_Scuffs
texture {
  pigment {
    bozo
    color_map {
     [0 Clear]
     [.8 Clear]
     [.95 Black]
     [1.0 Black]
    } // color_map
  } // pigment
  finish {
    //Dull
    ambient SpaceAmbient
  }
  scale 10
}
#else
}
#end

// Normal Ship Texture
#declare T_Ship_White = 
texture {
  pigment {White}
  finish {
    ambient SpaceAmbient
  }

#if (Show_Scuffs)
}
texture { T_Scuffs }
#else
}
#end

#declare T_Ship_Yellow = 
texture {
  pigment {Yellow}
  finish {
    ambient SpaceAmbient
  }

#if (Show_Scuffs)
}
texture { T_Scuffs }
#else
}
#end

// Yellow Strip texture
#declare T_Caution = texture {
  pigment {
    gradient x
    color_map {
      [0 Yellow]
      [.5 Yellow]
      [.5 Black]
      [1.0 Black]
      [1.0 Yellow]
    }
  }
  finish {
    ambient SpaceAmbient
  }
  scale 2*foot
  
#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
} 
#end

#declare SpaceShip_Pos = <-50*ft,0,100*ft>;

#include "Cargo_Module.inc"

#include "Cargo_Quad.inc"

///////////////////////////////////////////////////////////

#macro Make_Random_Cargo_Quad()
  #local Missing_Cargo_Chance = .99;
  union {
    object {Quad_Cradle}
    #if (rand(R1) < Missing_Cargo_Chance)
      object {Make_Random_Cargo() translate Cargo_Offset*y}
    #end
    #if (rand(R1) < Missing_Cargo_Chance)
      object {Make_Random_Cargo() translate Cargo_Offset*y rotate 90*x}
    #end
    #if (rand(R1) < Missing_Cargo_Chance)
      object {Make_Random_Cargo() translate Cargo_Offset*y rotate -90*x}
    #end
    #if (rand(R1) < Missing_Cargo_Chance)
      object {Make_Random_Cargo() translate Cargo_Offset*y rotate 180*x}
    #end
    rotate 90*y
  }
#end // macro

#macro Make_Cargo_Quad(Cargo1,Cargo2,Cargo3,Cargo4)
  union {
    object {Quad_Cradle}
    object {Cargo1 translate Cargo_Offset*y}
    object {Cargo2 translate Cargo_Offset*y rotate 90*x}
    object {Cargo3 translate Cargo_Offset*y rotate -90*x}
    object {Cargo4 translate Cargo_Offset*y rotate 180*x}
    rotate 90*y
  }
#end // macro

#macro Make_Partial_Cargo_Quad(Cargo1,Cargo2,Cargo3)
  union {
    object {Lit_Quad_Cradle}
    //object {Cargo1 translate Cargo_Offset*y}
    object {Cargo1 translate Cargo_Offset*y rotate 90*x}
    object {Cargo2 translate Cargo_Offset*y rotate -90*x}
    object {Cargo3 translate Cargo_Offset*y rotate 180*x}
    rotate 90*y
  }
#end // macro

#macro Place_Random_Cargo(AngleY,AngleX)
  Place_Cargo_Quad(AngleY,AngleX,Make_Random_Cargo_Quad())    
#end // macro

#macro Place_Bulk_Cargo(AngleY, AngleX)
  #if (rand(R1) < .5)
    #local pngFile = "BrownHeavyEquipmentDecal.png";
  #else
    #local pngFile = "BulkVolitilesDecal.png";
  #end
  Place_Cargo_Quad(AngleY, AngleX, Make_Bulk_Cargo(pngFile))
#end

#macro Place_Cargo_Quad(AngleY,AngleX,CargoQuad)
  #declare Front_Pt = Attach_Pt;
  #declare V_Cargo = vtransform(V_Cargo, transform {rotate <AngleX,AngleY,0>});
  #declare Attach_Pt = Front_Pt + V_Cargo;
  #declare Cargo_RotationX = Cargo_RotationX + AngleX;
  #declare Cargo_RotationY = Cargo_RotationY + AngleY;
  object { 
    CargoQuad
    translate Attach_Pt 
    Rotate_Around_Trans(Cargo_RotationY*y,Attach_Pt) 
    Rotate_Around_Trans(Cargo_RotationX*x,Attach_Pt)
    }
#end

#if (Show_Train)
  #declare Cargo_Train = union {
    #declare Cargo_Dist = Cargo_Quad_Len;
    #declare Cargo_RotationX = 0;
    #declare Cargo_RotationY = 0;
    #declare Front_Pt = <0,0,0>;
    #declare Attach_Pt = <0,0,Cargo_Dist*2>;
    #declare V_Cargo = <0,0, Cargo_Dist> - Front_Pt;
    
    object {Make_Random_Cargo_Quad() translate Attach_Pt}
    
    //Place_Cargo(0,0)
    Place_Random_Cargo(0,0)
    Place_Random_Cargo(0,0)
    Place_Random_Cargo(0,0)
    Place_Random_Cargo(-5,0)
    Place_Bulk_Cargo(-5,0)
    Place_Bulk_Cargo(-5,0)
    Place_Bulk_Cargo(-5,0)
    Place_Random_Cargo(-5,0)
    Place_Random_Cargo(-5,0)
    Place_Random_Cargo(-5,0)
    Place_Random_Cargo(-5,0)
    Place_Bulk_Cargo(-5,0)
    Place_Bulk_Cargo(-10,0)
    Place_Bulk_Cargo(-10,0)
    Place_Bulk_Cargo(-10,0)
    Place_Random_Cargo(-15,0)
    Place_Random_Cargo(-20,0)
    Place_Random_Cargo(-20,0)
    Place_Bulk_Cargo(-10,0)
    Place_Bulk_Cargo(-10,0)
    Place_Bulk_Cargo(-10,0)
    Place_Bulk_Cargo(-5,0)
    Place_Bulk_Cargo(-5,0)
    Place_Random_Cargo(-5,0)
    Place_Random_Cargo(-5,0)
    Place_Random_Cargo(-5,0)
    Place_Random_Cargo(-5,0)
    Place_Random_Cargo(-5,0)
    Place_Bulk_Cargo(-5,0)
    Place_Bulk_Cargo(-5,0)
    Place_Bulk_Cargo(-5,0)
    Place_Bulk_Cargo(-5,0)
    Place_Bulk_Cargo(-5,0)
    Place_Random_Cargo(0,5)
    Place_Random_Cargo(0,5)
    Place_Random_Cargo(0,0)
    Place_Random_Cargo(0,0)
    Place_Random_Cargo(0,0)
    Place_Random_Cargo(0,0)
    Place_Random_Cargo(0,0)
    Place_Random_Cargo(0,0)
    #local RealLights = true;
    Place_Cargo_Quad(0,0,
      Make_Partial_Cargo_Quad(
        MakeCargo("USSDecalFull.png",1,RealLights)
        MakeCargo("AMDecalFull.png",2,RealLights),    
        MakeCargo("He3DecalFull.png",3,RealLights),
      )
    )
    //Place_Random_Cargo(0,0)
  //  Place_Cargo(0,0)
  //  Place_Cargo(0,0)
}
#end

////////////////////////////////////////////////////////
//////// Fusion Drive Engine ///////////////////////////
#declare Engine_L = 150*ft;
#declare Engine_R = 20*ft;
#declare Engine_Shield_L = 2.5*ft;

#declare T_Engine = texture {
  pigment {
     gradient x
     color_map {
       [0 Yellow]
       [.75 rgb <1,0,0>]
       [1 rgb <.5,0,0>]
     }
     scale <Engine_L+Engine_R,1,1>
  } // pigment
  finish {
    Dull
    ambient SpaceAmbient
  }
 
#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end

#declare Spike = lathe {
  cubic_spline
  7
  <-1,33>, <0,15>, <1,7>, <2,3>, <3,1>, <4,0>, <5,1>
  pigment { Gray50 }
  rotate 90*z
  scale (Engine_R/5)
}

#declare Engine_Body = cylinder {
  0, <Engine_L, 0,0>, Engine_R  
  texture {T_Engine}
}

#declare Engine_Decal_Len = 800;
#declare Engine_Decal_H = 100;

#declare T_Engine_Decal = texture {
  pigment {
    image_map {
      png "GeneralFusionDecal.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 0        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
      once           // for no repetitive tiling
    } // image_map
    scale <Engine_Decal_Len, Engine_Decal_H,1>
  }
  finish {ambient SpaceAmbient}

#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end

#declare Engine_Decal = difference {
    cylinder {0, <Engine_Decal_Len,0,0>, Engine_R + .01 }
    cylinder {-1*x, <Engine_Decal_Len + 1,0,0>, Engine_R + .005 }
    texture {T_Engine_Decal}
    clipped_by {
      plane {z,-(Engine_R-4*ft)}
    }
}
#declare Engine_Decals = union {
  object { Engine_Decal translate 17*ft*x}
  object { Engine_Decal
    translate -(Engine_Decal_Len/2)*x
    rotate 180*y
    translate Engine_Decal_Len/2*x
    }
  translate 45*ft*x
}

///// Create Heat Sinks //////
#macro Make_T_Heat_Sink()
 texture {
  pigment {
   #if (Engine_On)    
    gradient y
    color_map {
      [0 Red]
      [.25 Red*.5]
      [.7 Red*.25]
      [.95 Black]
      [1 Black]
    }
    turbulence .15
   #else
    Black
   #end 
  }
  finish {
    Dull
    ambient SpaceAmbient
  }
  translate -.1*y
  scale Engine_L/3.3
  translate Engine_L/2*x
}
#end

#macro Make_Sink_Panel()
  box { 0, <Sink_L,Sink_H,Sink_D> Make_T_Heat_Sink() translate Engine_R*y}
#end

#macro Make_Heat_Sink()
union {
  #local Sink_L = Engine_L;
  #local Sink_H = Engine_L/4;
  #local Sink_D = 1*inch;
  
  object {Make_Sink_Panel() }
  object {Make_Sink_Panel() rotate -5*x}
  object {Make_Sink_Panel() rotate 5*x}
  object {Make_Sink_Panel() rotate -10*x}
  object {Make_Sink_Panel() rotate 10*x}
  object {Make_Sink_Panel() rotate -15*x}
  object {Make_Sink_Panel() rotate 15*x}
  
}
#end

#declare M_Exhaust = material {
  
}

#macro Make_Exhaust()
  #local Exhaust_Plume_L = 500*ft;
  #local Vent_Width = 3*ft;
  #local Exhaust_Start_R = Engine_R - 1*ft;
  #local Exhaust_End_R = 0;
  #local Inner_Exhaust_Start_R = Exhaust_Start_R - Vent_Width;
  #local Inner_Exhaust_End_R = 0;
union {
  difference {
    cone {0, Exhaust_Start_R, <0,Exhaust_Plume_L,0>, Exhaust_End_R}
    cone {<0,-1,0>, Inner_Exhaust_Start_R, <0,Exhaust_Plume_L + 1,0> Inner_Exhaust_End_R - 1*ft}
    hollow
    pigment { Clear } 
    interior {
      media {
        emission .2
        //scattering {.1, 0.5}
        density { 
          cylindrical 
          density_map {
            [0 rgbft<.2,.1,1,.1,.95>]
            [.1 rgbt<0,0,0,1>]
            [1 rgbft<0,0,.5,.1,.95>]
          }// density_map
          turbulence .1
          scale <Inner_Exhaust_End_R,Exhaust_Plume_L,Inner_Exhaust_End_R>
        } // density
        density {
          gradient y
          density_map {
           [0 rgbft<1,1,1,.1,.9>]
           [1 rgbt<0,0,0,1>]
          } // density map
          turbulence .1
          scale <1,Exhaust_Plume_L*1.1,1>
        } // density 
      } // media
    } // interior
    
    rotate 90*z
  }
} // union
#end

#macro Make_Engine()
union {
  union {
    object {Engine_Body}
    sphere {<0,0,0>, Engine_R translate Engine_L*x} // tip
    texture {T_Engine translate <rand(R1)*100,rand(R1)*100,rand(R1)*100>}
  }
  object {Spike}
  // flame shield
  difference {
    cylinder {<0,0,0>, <-Engine_Shield_L,0,0>, Engine_R }
    cylinder {<1,0,0>, <-Engine_Shield_L-1,0,0>, Engine_R - 1}    
    texture {T_Ship_White}
  }
  difference {
    cylinder { <0,0,0>, <6*inch,0,0>, Engine_R + 6*ft }
    cylinder { <-1,0,0>, <7*inch,0,0>, Engine_R }
    texture {T_Caution rotate 90*y}
  }
  object {Engine_Decals}
    
  // Heat Sinks
  object {Make_Heat_Sink() }
  // Exhaust
  #if (Engine_On)
    object {Make_Exhaust()}
  #end
  // position engine
  translate <-20*ft,Engine_R/2,0> 
}
#end  

//////////////// Engine Cradle ////////////////////////////
#local Engine_Cradle_L = Engine_L;
#local Engine_Cradle_T = 1*ft;
#local Engine_Cradle_R = Engine_R + Engine_Cradle_T;
#local Engine_Cradle_Support = torus {Engine_Cradle_R Engine_Cradle_T rotate 90*z}

#local T_Engine_Cradle = texture {
  pigment {Yellow}
  finish {ambient SpaceAmbient}
}

#local Engine_Cradle = merge {
  union {
    //object {Engine_Cradle_Support}
    object {Engine_Cradle_Support translate 1/5*Engine_Cradle_L*x}
    object {Engine_Cradle_Support translate 2/5*Engine_Cradle_L*x}
    object {Engine_Cradle_Support translate 3/5*Engine_Cradle_L*x}
    object {Engine_Cradle_Support translate 4/5*Engine_Cradle_L*x}
    clipped_by {
      plane {y,0}
    }
  }
  #local H_Support = cylinder {0, Engine_Cradle_L*x, Engine_Cradle_T translate -Engine_Cradle_R*y}
  object {H_Support}
  object {H_Support rotate -90*x}
  object {H_Support rotate 90*x}
  object {H_Support rotate -45*x}
  object {H_Support rotate 45*x}
  object {Engine_Cradle_Support translate Engine_Cradle_L*x}
  merge {
    torus {Engine_Cradle_R Engine_Cradle_T}
    torus {Engine_Cradle_R Engine_Cradle_T rotate 90*x}
    clipped_by {
      plane {-x,0}
    }
    translate Engine_Cradle_L*x
  }
  // Add Running Light
  object {
    Make_Real_Running_Light(Yellow)
    rotate -90*z
    translate (Engine_L+Engine_Cradle_R+Engine_Cradle_T)*x
  }
  // Adjust position of cradle to match engine
  translate <-20*ft,Engine_R/2,0>
  texture {T_Engine_Cradle}
}

///////////////////////////////////// End of Engine Section ////////////////////////////

#include "Comm_Laser.inc"

#include "Comm_Dish.inc"

///////////////////////////////////////////////
//////////////////// Torus ////////////////////
#declare Torus_Major_R = 100*ft;
#declare Torus_Minor_R = 15*ft;
#declare Detail_Torus_r = 6*ft;

//#declare T_Torus = texture { T_Ship_White }
#declare T_Torus = texture {
  pigment {
    image_map {
      png "SpaceShipTorusDecal.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 5        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
      once              // for no repetitive tiling
    } // image_map
    scale <1, .25, 1>
  }
  finish { ambient SpaceAmbient }

#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end

#declare T_Detail_Torus = texture { T_Ship_White}

#declare Main_Torus = union {
  
  merge {
    difference {
      torus { Torus_Major_R, Torus_Minor_R texture {T_Torus}}
      cylinder {-(Torus_Minor_R+1)*y (Torus_Minor_R + 1)*y Torus_Major_R}
    }
    intersection {
      torus {Torus_Major_R, Torus_Minor_R texture {T_Ship_White}}
      cylinder {-(Torus_Minor_R+1)*y (Torus_Minor_R + 1)*y Torus_Major_R}
    }
    scale <1,4,1>
  }
  // these tori form attachment points for engine cradles
  #local Detail_Torus = torus {
    Torus_Minor_R-Detail_Torus_r/2 Detail_Torus_r 
    rotate 90*x
    scale <1,4,1>
    translate Torus_Major_R*x
    texture {T_Detail_Torus}
  }
  #local Detail_Angle = 35;
  #local Tori = union {
    object {Detail_Torus rotate Detail_Angle*y}
    object {Detail_Torus rotate -Detail_Angle*y}
  }
  object {Tori }
  object {Tori rotate 90*y }
  object {Tori rotate -90*y}
  object {Tori rotate 180*y}
  rotate 90*z
} // Torus

///////////////////////////////////////////////////
// Zero G Module //////////////////////////////////

#declare Zero_G_R = Torus_Major_R - Torus_Minor_R - 30*ft;
#declare Zero_G_Cyl_L = Zero_G_R;
#declare Zero_G_Wall_T = 1*m;

#local T_Cargo_Door = texture {
  pigment {
    image_map { 
      png "CargoDoorDecal.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 1        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 4     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
      once           // for no repetitive tiling
    } // image_map
    translate <0,0,0>
    rotate 180*y
  }
  finish { ambient SpaceAmbient }

#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end


#local Shell = difference {
  sphere {0, Zero_G_R }
  sphere {0, Zero_G_R - Zero_G_Wall_T} // hollow it out
}

#local Forward_Sphere_Shell = object {
  Shell
  texture {T_Ship_White}
}

#local Door_Shell = object {
  Shell scale .985
  texture {T_Cargo_Door}
}

#local Door_Frame_T = 15*cm;

#local Cutout_W = Zero_G_R - Door_Frame_T*2;
#local Cutout_H = Zero_G_R/2 - Door_Frame_T*2;

#local Door_Cutout = box { <-Zero_G_R,0,0>, <Zero_G_R,Cutout_H,Cutout_W>
    translate <0,Door_Frame_T,Door_Frame_T>
  }


#local Door_Frame_Cutout = difference {
  box { 0, <Zero_G_R,Zero_G_R/2,Zero_G_R>}
  
  object {Door_Cutout}
  translate <0,-Zero_G_R/4,-Zero_G_R/2>
}

#local T_Door_Frame = texture { T_Caution rotate 90*y}

#local Top_Door_Frame = intersection {
  object {Door_Shell texture {T_Door_Frame}}
  object {Door_Frame_Cutout translate Zero_G_R/4*y}
}

#local Bottom_Door_Frame = intersection {
  object {Door_Shell texture {T_Door_Frame}}
  object {Door_Frame_Cutout translate -Zero_G_R/4*y}
}



#local Top_Door = intersection {
  object {Door_Shell }
  object {Door_Cutout translate <0,-Zero_G_R/4,-Zero_G_R/2> translate Zero_G_R/4*y}
} 


#local Bottom_Door = intersection {
  object {Door_Shell}
  object {Door_Cutout translate <0,-Zero_G_R/2,-Zero_G_R/2>}
}

#local T_Large_Hatch = texture { T_Ship_White }

#local T_Hatch_Detail = texture { T_Ship_Yellow }

#local Large_Hatch_R = 10*ft;
#local Large_Hatch_r = 2*ft;

#local Large_Hatch = union {
  sphere {0, Large_Hatch_R }
  torus {Large_Hatch_R Large_Hatch_r}
  merge {
    torus {Large_Hatch_R Large_Hatch_r/2 rotate 90*x scale <1,1,.5>}
    torus {Large_Hatch_R Large_Hatch_r/2 rotate 90*z scale <.5,1,1>}
    texture {T_Hatch_Detail}
  }
  scale <1,.25,1>
  texture {T_Large_Hatch}
} 

#local Forward_Sphere = union {

  difference {
    object {Forward_Sphere_Shell}
    box {0, <Zero_G_R,Zero_G_R,Zero_G_R> translate <Zero_G_R/2,-Zero_G_R/2,-Zero_G_R/2>} 
  }
  object {Large_Hatch translate (Zero_G_R-1*ft)*y rotate -40*z}
  object {Large_Hatch translate (Zero_G_R-1*ft)*y rotate -40*z rotate 90*x}
  object {Large_Hatch translate (Zero_G_R-1*ft)*y rotate -40*z rotate -90*x}
  object {Large_Hatch translate (Zero_G_R-1*ft)*y rotate -40*z rotate 180*x}

  object {Top_Door_Frame}
  object {Top_Door}
  object {Bottom_Door_Frame}
  object {Bottom_Door}
}


#declare Zero_G_Module = union {
  union {
    cylinder {-Zero_G_Cyl_L*x, Zero_G_Cyl_L*x, Zero_G_R}
    sphere {-Zero_G_Cyl_L*x Zero_G_R }
    texture {T_Ship_White}
  }
  object { Forward_Sphere translate Zero_G_Cyl_L*x} 
}


///////////////////////////////////////////////////////////////
///////// Command Bubble //////////////////////////////////////


#declare Command_Bubble_R = 20*ft;
#declare T_Command_Bubble = texture { 
  pigment { rgbf <.95,.8,0,.75> }
  finish {
      specular 1
      roughness 0.001
      ambient SpaceAmbient
      diffuse 0
      reflection 0.5
  }   
}

#declare Command_Bubble = union {
  difference {
    difference {
        sphere {0, Command_Bubble_R}
        sphere {0, Command_Bubble_R - 6*inch}
    } // diff
    // put in window cutout dummy!
    box {<0,0,-(Command_Bubble_R + 1)>, <Command_Bubble_R+1,Command_Bubble_R + 1,Command_Bubble_R + 1>}
    texture {T_Ship_White}
  } // diff
  // put in window glass - gold tint?
  difference {
    sphere {0, Command_Bubble_R - .01*inch}
    sphere {0, Command_Bubble_R - 1*inch}
    texture {T_Command_Bubble}
    interior { I_Glass1 }
  } // difference
  #if (Show_Cockpit_Light)
    light_source {<0,5,0> color White fade_distance Command_Bubble_R fade_power 2 }
  #end
}

////// Airlock and Communications Complex //////////////

#declare Air_Lock_Comms_Complex = union {
  #local Offset = (Torus_Major_R+Torus_Minor_R)-2*ft;
  #local Comm_Laser_Angle = 10;
  #local Comm_Dish_Angle = 10;
  object {Air_Lock translate Offset*y}
  object {Comm_Laser translate (Offset+2*ft)*y rotate -Comm_Laser_Angle*x}
  object {Comm_Dish translate (Offset+2*ft)*y rotate Comm_Dish_Angle*x}
}

#declare Torus_Assembly = union {
  object {Main_Torus}
  // Add Air Locks
  union {
    object {Air_Lock_Comms_Complex }
    object {Air_Lock_Comms_Complex rotate -90*x}
    object {Air_Lock_Comms_Complex rotate 90*x}
    object {Air_Lock_Comms_Complex rotate -180*x}
  } // union 
  // Add Engines
  union {
    object {Make_Engine() translate <-Torus_Minor_R*4,Torus_Major_R+Torus_Minor_R,0>}
    object {Engine_Cradle translate <-Torus_Minor_R*4,Torus_Major_R+Torus_Minor_R,0>} 
    object {Make_Engine() translate <-Torus_Minor_R*4,Torus_Major_R+Torus_Minor_R,0> rotate 90*x}
    object {Engine_Cradle translate <-Torus_Minor_R*4,Torus_Major_R+Torus_Minor_R,0> rotate 90*x}
    object {Make_Engine() translate <-Torus_Minor_R*4,Torus_Major_R+Torus_Minor_R,0> rotate -90*x}
    object {Engine_Cradle translate <-Torus_Minor_R*4,Torus_Major_R+Torus_Minor_R,0> rotate -90*x}
    object {Make_Engine() translate <-Torus_Minor_R*4,Torus_Major_R+Torus_Minor_R,0> rotate -180*x}
    object {Engine_Cradle translate <-Torus_Minor_R*4,Torus_Major_R+Torus_Minor_R,0> rotate -180*x}
    rotate 45*x
  }
  // Zero_G Module
  object {Zero_G_Module}
  // Module Connectors
  #local Module_Connector_R = 5*m;
  #local Module_Connector_L = Torus_Major_R;
  #local Module_Connector = cylinder {0, Module_Connector_L*y Module_Connector_R scale <3,1,1>}
  #local T_Module_Connector = texture {
    pigment { rgb <.3,.3,.7> }
    finish { ambient SpaceAmbient }
  }
  union {
    object {Module_Connector}
    object {Module_Connector rotate 90*x}
    object {Module_Connector rotate -90*x}
    object {Module_Connector rotate 180*x}
    rotate 45*x
    texture { T_Module_Connector }
  }
  // Command Bubble
  object {Command_Bubble translate <Torus_Minor_R*3,-(Torus_Major_R-Torus_Minor_R),0> }
}

#declare Ship = union {
  object {Torus_Assembly rotate 90*y}
  #if (Show_Train)
    object {Cargo_Train translate -100*ft*z}
  #end
}


////////----------------------------------------------------------------------------
#end // ifdef spaceship