// Persistence of Vision Ray Tracer Scene Description File
// File: SpaceTug.pov
// Vers: 3.5
// Desc: Basic Scene Example
// Date: 9/15/2004
// Auth: Ray Benjamin
//

#ifndef(__SpaceTug__)
#declare __SpaceTug__ = 1;

#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end
#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end

#ifndef (R1)
  #declare R1 = seed(3939);
#end

#ifndef(Ship_Lights)
  #declare Ship_Lights = true;
#end
#ifndef(Ship_Scale_Factor)  
  #declare Ship_Scale_Factor = 1;
#end 

#include "Comm_Laser.inc"
#include "Comm_Dish.inc"

// ----------------------------------------

#ifndef(T_Scuffs)
  #declare T_Scuffs = texture {
      pigment {
          granite
          color_map {
            /*
              [0.0 White]
              [0.08 White]
              [0.08 Clear]
              [0.8 Clear]
              [0.92 Black]
              [1.0 Gray10]
            */
            [0 White]
            [.15 Clear]
            [.75 Clear]
            [1 Black]
          } // color_map
          turbulence 1
          scale 1000
          scale <30,1,1>
      } // pigment
      finish {
        ambient 0
      }
  } // texture T_Scuffs
texture {
  pigment {
    bozo
    color_map {
     [0 Clear]
     [.8 Clear]
     [.95 Black]
     [1.0 Black]
    } // color_map
  } // pigment
  finish {
    Dull
    ambient 0
  }
  scale 10
}
#end

#ifndef(T_White)
  #local T_White = texture {
    pigment {White}
    finish {ambient 0}
#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end

#end

#ifndef(T_Caution)
  // Yellow Strip texture
  #declare T_Caution = texture {
    pigment {
      gradient x
      color_map {
        [0 Yellow]
        [.5 Yellow]
        [.5 Black]
        [1.0 Black]
        [1.0 Yellow]
      }
    }
    finish { ambient 0}
    scale 2*foot
  }
#end

#local T_Gold_Glass = texture {
  pigment { Gold + <0,0,0,.5,.1> }
  finish {F_Glass1 ambient 0}
}


#macro Make_Running_Light(LightColor)

#local Light_R = 3*inch;
#local Glass_T = .25*inch;
#local Light_H = 6*inch;

union {
  object {
  #if (Ship_Lights)
    light_source {
      <0,0,0>
      color LightColor
      fade_power 2
      fade_distance 5*ft/Ship_Scale_Factor
      looks_like {
        sphere {<0,0,0>, Light_R
          pigment { LightColor * 2 }
          finish { ambient LightColor * 2 }
          hollow
          no_shadow
        } // sphere
      } // looks_like
    }  // light_source
  #else
    sphere { <0,0,0> Light_R
          pigment { LightColor * 2 }
          finish { ambient LightColor * 2 }
    }
  #end // if
    translate Light_H*y
  }
  // Light Bulb Base
  cylinder { <0,0,0>, <0,Light_H,0>, 1*inch pigment {White}}
  // Glass Cover
  #ifndef (Show_Glass)
    #declare Show_Glass = true;
  #end
  #if (Show_Glass)
    union {
      #local Glass_R = Light_R + 1*inch;
      #local Glass_I_R = Glass_R - Glass_T;
      Ring(Glass_R,Glass_I_R, Light_H)
      difference {
        sphere {0, Glass_R}
        sphere {0, Glass_I_R}
        translate Light_H*y
      }
      hollow
      texture {T_Glass1}
      interior {
        ior 1.32
      }
    }
  #end
}  
#end // macro

// ----------------------------------------

#include "Docking_Ring"

////////////////////////////////////////////////////////////////////
// ROAM a.k.a. "Beachball" -- Remote Observervation And Maintenance
////////////////////////////////////////////////////////////////////
#local Roam_Shell_R = 1*m;
#local Roam_Shell_T = 2*cm;
#local Roam_Camera_R = 10*cm;
#local Roam_Light_R = 10*cm;
#local Jet_Nozzle_R = 5*cm;
#local Jet_Cutout_L = 2.5*m;
#local Jet_T = 5*mm;
#local Jet_H = Roam_Shell_R - Jet_Nozzle_R * 5;
#local Jet_Jut = 5*cm; // how much the nozzle juts out from body
#local Flood_Light_R = 20*cm;
#local Flood_Light_Shell_T = 2*mm;
#local Roam_Window_Cutout_H = 20*cm;

#local T_Roam_Shell = texture {
  //pigment { White }
  pigment {
    image_map {
      png "ROAMDecal.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 1        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
    } // image_map
    rotate -90*y
  }

  finish {ambient .3}

#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end

#local T_Jets = texture { 
 T_Gold_3D  
 finish {ambient 0}

#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end

#local T_Flood_Light_Shell = texture {
  pigment {White}
  finish { Glossy }
}

#local Roam_Shell_Sphere = difference {
  sphere {0, Roam_Shell_R}
  sphere {0, Roam_Shell_R - Roam_Shell_T}
} // difference

////////////// Safety Anchor ////////////
#local Anchor_R = 6*inch;
#local Anchor_r = .5*inch;
#local Anchor_Base_R = Anchor_R*1.3;
#local Anchor_Base_H = .5*inch;


#local T_Ring_Ext = texture {
  pigment {Yellow}
  finish {ambient 0}
}

#local T_Ring_Scrapes = texture {
  //T_Silver_2D
    gradient x
    texture_map {
      [0 T_Ring_Ext]
      [.5 T_Ring_Ext]
      [.5 pigment {Black}]
      [.55 pigment {Black}]
      [.56 T_Silver_2D]
      [1.0 T_Silver_2D]
    }
    turbulence .5
}

#local T_Ring_Int = texture {
  bozo
  texture_map {
    [0 T_Ring_Ext]
    [.5 T_Ring_Ext]
    [.5 T_Ring_Scrapes]
    [1.0 T_Ring_Scrapes]
  }
}

#local Anchor_Ring = union {
  torus {Anchor_R Anchor_r scale <1,1,.5> texture {T_Ring_Int} rotate 90*z}
  cylinder {<0,0,-Anchor_Base_H/2> <0,0,Anchor_Base_H> Anchor_Base_R  scale <.5,1,1> texture {T_Caution scale .15 rotate 45*z}}
}

#local Anchor = object {
  Anchor_Ring
  translate Roam_Shell_R*z
}

#local Anchor_Assembly = union {
  #local Anchor_Angle = <0,30,0>;
  object {Anchor rotate Anchor_Angle}
  object {Anchor rotate -Anchor_Angle}
}

////////////// Rear Hatch ///////////////
#local Rear_Hatch_R = Roam_Shell_R/3;
#local Rear_Hatch_Hole = cylinder {<0,0,0>, <0,0,Roam_Shell_R+1>, Rear_Hatch_R}
#local Detail_r = .5*inch;
#local Rear_Hatch = union {
  sphere {0, Rear_Hatch_R texture {T_White}}
  union {
    torus {Rear_Hatch_R-Detail_r Detail_r texture {T_Silver_2D}}
    torus {Rear_Hatch_R-Detail_r Detail_r texture {T_Silver_2D} rotate 90*z}
    translate Detail_r*z
  }
  scale <1,1,.125>
  translate (Roam_Shell_R-2.5)*z
}

#local Roam_Shell = difference {
  object {Roam_Shell_Sphere}
  object {Rear_Hatch_Hole}
  texture {T_Roam_Shell}
}

#local Roam_Window_Cutout = cylinder {-Roam_Window_Cutout_H/2*y, Roam_Window_Cutout_H/2*y, Roam_Shell_R+1}

#local Roam_Glass_Shell = intersection {
  object {Roam_Shell_Sphere}
  union {
    object {Roam_Window_Cutout translate Roam_Shell_R/2*y}  
    object {Roam_Window_Cutout translate -Roam_Shell_R/2*y}  
  }
  hollow
  interior {I_Glass1}
  texture {T_Gold_Glass}
}

#local Jet_Cutout = cylinder { -Jet_Cutout_L/2*y, Jet_Cutout_L/2*y, Jet_Nozzle_R}

#local Jet_Cutout_Assembly = union {
  object {Jet_Cutout}
  #local H_Jets_Cutout = union {
    object {Jet_Cutout rotate 90*z}
    object {Jet_Cutout rotate 90*z translate Roam_Shell_R/4*z}
    object {Jet_Cutout rotate 90*z translate -Roam_Shell_R/4*z}
    object {Jet_Cutout rotate 90*x}
    object {Jet_Cutout rotate 90*x translate Roam_Shell_R/4*x}
    object {Jet_Cutout rotate 90*x translate -Roam_Shell_R/4*x}
  } // union
  object {H_Jets_Cutout translate Jet_H*y}
  object {H_Jets_Cutout translate -Jet_H*y}
} // union

#local Jet = difference {
  cylinder {-(Jet_Cutout_L/2)*y, (Jet_Cutout_L/2)*y, Jet_Nozzle_R}
  cylinder {-(Jet_Cutout_L/2 + 1)*y, (Jet_Cutout_L/2 + 1)*y, Jet_Nozzle_R - Jet_T}
}

#local Jet_Assembly = intersection {
  union {
    object {Jet}
    #local H_Jets = union {
      object {Jet rotate 90*z}
      object {Jet rotate 90*z translate Roam_Shell_R/4*z}
      object {Jet rotate 90*z translate -Roam_Shell_R/4*z}
      object {Jet rotate 90*x}
      object {Jet rotate 90*x translate Roam_Shell_R/4*x}
      object {Jet rotate 90*x translate -Roam_Shell_R/4*x}
    } // union
    object {H_Jets translate Jet_H*y}
    object {H_Jets translate -Jet_H*y}
    texture {T_Jets}
  } // union
  sphere {0, Roam_Shell_R + Jet_Jut}
} // intersection

#local FL_Cutout_L = 1.5*Roam_Shell_R;
#local Flood_Light_Cutout = cylinder {0,  -FL_Cutout_L*z, Flood_Light_R}
#local Flood_Light_Depth = Flood_Light_R*2;
#declare Flood_Light = union {
  // temp pointing stick
  //cylinder {0, -200*m*z, 10*cm pigment {Red}}
  //
  difference {
    cylinder { 0, Flood_Light_Depth*z, Flood_Light_R}
    cylinder { 0, Flood_Light_Depth*z, Flood_Light_R - Flood_Light_Shell_T
      translate -Flood_Light_Shell_T*z
    } 
    texture {T_Flood_Light_Shell}
  } // difference
  #local LightColor = rgb<.98,1,.95>;
  #local Light_R = Flood_Light_R * .8;
  
  light_source {
    <0,0,0>
    color LightColor
    spotlight               // this kind of light source
    point_at <0,0,-1>      // direction of spotlight
    radius 30
    tightness 0
    falloff 35
    fade_power 2
    fade_distance 20*m/Ship_Scale_Factor
    looks_like {
      sphere {<0,0,0>, Light_R
        pigment { LightColor * 2 }
        finish { ambient LightColor * 2 }
        hollow
        no_shadow
      } // sphere
    } // looks_like
    translate Light_R*z
  }  // light_source
  
  translate -Roam_Shell_R*z
}

#declare Flood_Light_Unlit = union {
  difference {
    cylinder { 0, Flood_Light_Depth*z, Flood_Light_R}
    cylinder { 0, Flood_Light_Depth*z, Flood_Light_R - Flood_Light_Shell_T
      translate -Flood_Light_Shell_T*z
    } 
    texture {T_Flood_Light_Shell}
  } // difference
  #local LightColor = rgb<.98,1,.95>;
  #local Light_R = Flood_Light_R * .8;
  
  sphere {<0,0,0>, Light_R
    pigment { LightColor }
    finish { ambient 0 }
    translate Light_R*z
  }
  
  translate -Roam_Shell_R*z
}

#declare ROAM_Craft = union {
  difference {
    object {Roam_Shell}
    object {Jet_Cutout_Assembly }
    object {Roam_Window_Cutout translate Roam_Shell_R/2*y}  
    object {Roam_Window_Cutout translate -Roam_Shell_R/2*y}  
    object {Flood_Light_Cutout}
  }
  object {Jet_Assembly}
  object {Flood_Light}
  object {Roam_Glass_Shell}
  object {Rear_Hatch}
  object {Anchor_Assembly}
  object {Anchor_Assembly rotate 180*y}
}

#declare ROAM_Craft_Unlit = union {
  difference {
    object {Roam_Shell}
    object {Jet_Cutout_Assembly }
    object {Roam_Window_Cutout translate Roam_Shell_R/2*y}  
    object {Roam_Window_Cutout translate -Roam_Shell_R/2*y}  
    object {Flood_Light_Cutout}
  }
  object {Jet_Assembly}
  object {Flood_Light_Unlit}
  object {Roam_Glass_Shell}
  object {Rear_Hatch}
  object {Anchor_Assembly}
  object {Anchor_Assembly rotate 180*y}
}

/////////////////////////////////////////////////////////////////////

/////////////////////// Space Tug /////////////////////////////////


#declare Space_Tug_R = 20*ft;
#local Space_Tug_Shell_T = 6*inch;
#local Space_Tug_I_R = Space_Tug_R - Space_Tug_Shell_T;
#local Space_Tug_Torus_Min_R = 5*ft;
#local Space_Tug_Torus_Maj_R = Space_Tug_R + Space_Tug_Torus_Min_R;
 

#local T_Space_Tug = texture {
  //pigment {rgb <1,1,.3>}
  
  pigment {
    image_map {
      png "SpaceTugDecal.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 1        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 2     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
    } // image_map
    rotate 180*y
  }
  
  finish {ambient 0}

#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end


#local T_Space_Tug_Torus = texture {

  pigment {
    image_map {
      png "TugTorusDecal.png" // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
      map_type 2        // 0=planar, 1=spherical, 2=cylindrical, 5=torus
      interpolate 4     // 0=none, 1=linear, 2=bilinear, 4=normalized distance
    } // image_map
    translate .5*y
    scale <1,Space_Tug_R,1>
    //scale <Space_Tug_Torus_Maj_R,Space_Tug_R,1>
    //rotate 20*y
    rotate 90*z
  }
  
  finish {ambient 0}

}


#local Torus_Inner_R = Space_Tug_R + 1*ft;
#local Torus_T = 5*ft;
#local Torus_Outer_R = Torus_Inner_R + Torus_T;

#declare Space_Tug_Radius = Torus_Outer_R;

//////////////////////// Space Tug Arms /////////////////////////

#declare Upper_Arm_R = 10*cm;
#declare Upper_Arm_L = 5*m;
#declare Shoulder_R = 40*cm;
#declare Elbow_R = 15*cm;
#declare Wrist_R = 15*cm;
#declare Clamp_R = 11*cm;
#declare Clamp_W = 1*m;
#declare Lower_Arm_R = 10*cm;
#declare Lower_Arm_L = 5*m;

#local Elbow_Pos = <Upper_Arm_L,0,0>;
#local Wrist_Pos = Elbow_Pos + <Lower_Arm_L,0,0>;

#declare T_Arm_Joint = texture { 
  pigment {White}
  finish {ambient 0}
}

#declare T_Arm = texture {
  pigment {White}
  finish {ambient 0}
}

#declare T_Clamp = texture {
  pigment {rgb <.1,.1,.8>}
  finish {
    Dull
    ambient 0
  }
}

#macro Make_SpaceTug_Arm(WristAngle,
 ElbowAngle,
 ShoulderAngle)
 
  // Hand -- Clamp
  #local Clamp = cylinder {<0,0,-Clamp_W/2>, <0,0,Clamp_W/2>, Clamp_R texture {T_Clamp}}
  #local Hand = sphere {<0,0,0>, Wrist_R texture {T_Arm_Joint}}
  // Lower Arm
  #local Lower_Arm = union {
    sphere {0, Elbow_R texture {T_Arm_Joint}}
    cylinder {0,Lower_Arm_L*x, Lower_Arm_R texture {T_Arm}}
  }
  #local Upper_Arm = union { // Upper Arm}
    sphere {0, Shoulder_R texture {T_Arm_Joint}}
    cylinder {0, Upper_Arm_L*x, Upper_Arm_R texture {T_Arm}}
  }
  
  union {
    union {
      object {Clamp rotate WristAngle translate Lower_Arm_L*x}
      object {Lower_Arm}
      rotate ElbowAngle
      translate Upper_Arm_L*x
    } // Union Lower Arm
    object {Upper_Arm}
    rotate ShoulderAngle
  } // union Upper_Arm
#end // Macro Make_Space_Tug_Arm

////////////////// Search Light //////////////////////////////////

#local SL_R = 30*cm;
#local SL_L = 50*cm;
#local SL_T = 2*cm;
#local T_SL_Interior = texture {
  pigment {White}
}
#local SL_Bulb_Offset = 20*cm;

#local T_SL_Exterior = texture {
  pigment {Gray50}
  finish {
    ambient 0
    metallic
  }
}

#declare SearchLight_Mount_Angle = 60;
#declare SpaceTug_SearchLight = union {
  union {
    light_source {
      SL_Bulb_Offset*x                     // light's position (translated below)
      color rgb <1,1,1> / 3      // light's color
      spotlight               // this kind of light source
      point_at <SL_Bulb_Offset*2, 0, 0>      // direction of spotlight
      radius 30
      tightness 0
      falloff 35
      fade_power 2
      fade_distance 20*m/Ship_Scale_Factor
      looks_like {
        sphere {SL_Bulb_Offset*x, SL_R/2
          hollow
          no_shadow
          texture {
            pigment {White}
            finish {Luminous}
          } // texture
        }
      }
    }
    // Body of Search Light
    difference {
      cylinder {-20*cm*x, SL_L*x, SL_R}
      cylinder {0, SL_L*x, (SL_R - SL_T) translate SL_T*x texture { T_SL_Interior }}
      texture {T_SL_Exterior}
    }
    translate SL_R*y
    // Counter act the rotation done when mounting the light
    rotate -(SearchLight_Mount_Angle-5)*z
    // Point Light
    rotate -15*z
  }
  // Mount
  #local Mount_R = 5*cm;
  #local Mount_Leg = cylinder {<0,0,0>, <-3*m,0,0>, Mount_R texture {T_SL_Exterior}}
  object {Mount_Leg}
  object {Mount_Leg rotate <0,30,45>}
  object {Mount_Leg rotate <0,-30,45>}
}


/////////////////// Make RCS Motors /////////////////////////////////
#local T_Nozzle = texture {
  T_Silver_2D
}
texture {
  pigment {
    gradient y
    color_map {
     [0 Clear]
     [.5 Black]
     [1 Black]
    }
  }
  finish {
    Dull
    ambient 0
  }
}

#declare Nozzle = lathe {
    quadratic_spline
    5, // # of points
    <-1,.5>
    <0,0> // list of <u,v> points
    <1,.5>
    <2,2>
    <3,4.5>
    texture {T_Nozzle scale 6}
    scale 6
}

#local Nozzle_Cutout = cylinder {
 0, 4.6*y, 3
 scale 6
}

#local T_Engine_Module = texture {
  pigment { rgb<.2,.2,1> }
  finish {ambient 0}

#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end

#local T_Reaction_Control_Nozzle = texture {
  T_Gold_3C
  finish { ambient 0 }

#if (Show_Scuffs)
}
texture {T_Scuffs}
#else
}
#end

#local RCS_Motor_Nozzle = object {Nozzle scale .5 texture {T_Reaction_Control_Nozzle}}

#local RCS_Thruster_R = 10*cm;
#local RCS_Thruster_L = 20*cm;
#local RCS_Thruster_T = 1*cm;
#local RCS_Thruster_I_R = RCS_Thruster_R - RCS_Thruster_T;

#local RCS_Thruster = difference {
  union {
    sphere {0, RCS_Thruster_R}
    cylinder {0, RCS_Thruster_L*x, RCS_Thruster_R}
  }
  union {
   sphere {0, RCS_Thruster_I_R}
   cylinder {0, (RCS_Thruster_L+1)*x, RCS_Thruster_I_R}
  }
  texture {T_Reaction_Control_Nozzle}
}

#local RCS_Thruster_Cutout = union {
    sphere {0, RCS_Thruster_R}
    cylinder {0, RCS_Thruster_L*x, RCS_Thruster_R}
} 


#local Reaction_Control_R = 1*ft;
#local Reaction_Control_H = 2*ft;
#local Reaction_Control_Assembly = union {
  cylinder { -Reaction_Control_H/2*y, Reaction_Control_H/2*y, Reaction_Control_R 
    texture {T_Engine_Module}
  }
  object {RCS_Motor_Nozzle translate Reaction_Control_H/2*y}
}

//////////////////////////////////////////////
// Engines
//////////////////////////////////////////////

#local Engine_Module_R = 2*ft;
#local Engine_Module_H = 2*ft;

#local Engine_Module_Front = union {
  difference {
    union {
      cylinder {Engine_Module_H/2*y, -Engine_Module_H/2*y, Engine_Module_R }
      sphere {Engine_Module_H/2*y, Engine_Module_R}
      texture {T_Engine_Module}
    }
    object {Nozzle_Cutout scale .5 translate Engine_Module_H*y}
    union {
      object {RCS_Thruster_Cutout translate Engine_Module_R*x rotate 90*y}
      object {RCS_Thruster_Cutout translate Engine_Module_R*x rotate -90*y}
      object {RCS_Thruster_Cutout translate Engine_Module_R*x rotate 180*y}
      translate Engine_Module_H/2*y
    }
  }
  object {Nozzle scale .5 texture {T_Reaction_Control_Nozzle} translate Engine_Module_H*y}
  union {
    object {RCS_Thruster translate Engine_Module_R*x rotate 90*y}
    object {RCS_Thruster translate Engine_Module_R*x rotate -90*y}
    object {RCS_Thruster translate Engine_Module_R*x rotate 180*y}
    translate Engine_Module_H/2*y
  }
}

#local Engine_Module_Rear = union {
  object {Nozzle translate 1*ft*y}
  difference {
    cylinder {Engine_Module_H/2*y, -Engine_Module_H/2*y, Engine_Module_R texture {T_Engine_Module}}
    union {
      object {RCS_Thruster_Cutout translate Engine_Module_R*x rotate 90*y}
      object {RCS_Thruster_Cutout translate Engine_Module_R*x rotate -90*y}
      object {RCS_Thruster_Cutout translate Engine_Module_R*x}
      translate Engine_Module_H*4/16*y
    }
  }
  union {
    object {RCS_Thruster translate Engine_Module_R*x rotate 90*y}
    object {RCS_Thruster translate Engine_Module_R*x rotate -90*y}
    object {RCS_Thruster translate Engine_Module_R*x}
    translate Engine_Module_H*4/16*y
  }
}

#local Engine_Grouping = union {
  object { Engine_Module_Front rotate -90*z  translate <Space_Tug_R/4+2*ft,-Torus_T/2,0>} // front
  // Engine Cowling
  #local Engine_Cowling_L = Space_Tug_R/2 + 4*ft;
  #local Engine_Cowling_R = Engine_Module_R+40*cm;
  union {
    difference {
      cylinder {0, Engine_Cowling_L*x, Engine_Cowling_R}
      box {<0,-5*inch,-1*inch> <Engine_Cowling_L,1*inch,1*inch> translate Engine_Cowling_R*y}
      texture {T_Ship_White}
    }
    #local Anchor_Group = union {
      object {Anchor_Ring rotate <90,90,0> translate Engine_Cowling_R*y rotate -15*x}
      object {Anchor_Ring rotate <90,90,0> translate Engine_Cowling_R*y rotate 15*x}
    }
    object {Anchor_Group translate Engine_Cowling_L/5*x}
    //object {Anchor_Group translate Engine_Cowling_L*2/5*x}
    //object {Anchor_Group translate Engine_Cowling_L*3/5*x}
    object {Anchor_Group translate Engine_Cowling_L*4/5*x}
    translate <-Engine_Cowling_L/2,-Torus_T/2,0>
  }
  object { Engine_Module_Rear rotate 90*z  translate <-(Space_Tug_R/4+3*ft),-Torus_T/2,0>} // rear
}

///////////////////// Engine Torus ///////////////////////////////////////////

#local Space_Tug_Torus = union {
  union {
   Ring(Torus_Outer_R, Torus_Inner_R,Space_Tug_R/2)
   torus {Torus_Outer_R-Torus_T/2,Torus_T/2}
   torus {Torus_Outer_R-Torus_T/2,Torus_T/2 translate Space_Tug_R/2*y}
   translate -Space_Tug_R/4*y
   rotate 90*z
   texture {T_Space_Tug_Torus}
  }
  object {Engine_Grouping translate Torus_Outer_R*y}
  object {Engine_Grouping translate Torus_Outer_R*y rotate -90*x}
  object {Engine_Grouping translate Torus_Outer_R*y rotate 90*x}
  object {Engine_Grouping translate Torus_Outer_R*y rotate 180*x}
  //
  object {Comm_Laser translate Torus_Outer_R*y rotate 20*x}
  object {Comm_Laser translate Torus_Outer_R*y rotate 110*x}
  object {Comm_Laser translate Torus_Outer_R*y rotate 200*x}
  object {Comm_Laser translate Torus_Outer_R*y rotate 290*x}
  //
  object {Comm_Dish scale .5 rotate rand(R1)*360*y translate Torus_Outer_R*y rotate -20*x}
  object {Comm_Dish scale .5 rotate rand(R1)*360*y translate Torus_Outer_R*y rotate -110*x}
  object {Comm_Dish scale .5 rotate rand(R1)*360*y translate Torus_Outer_R*y rotate -200*x}
  object {Comm_Dish scale .5 rotate rand(R1)*360*y translate Torus_Outer_R*y rotate -290*x}
  
}

//////////////////////////////// space Tug Shell ///////////////////////////////

// window cutout box
#local Window_W = Space_Tug_R/3;
#local Window_H = Space_Tug_R/2;
#local Window_D = Space_Tug_R*2;
#local Cutout_Box = box {0, <Window_W,Window_H,Window_D> translate <Space_Tug_R*2/3,0,-Space_Tug_R>}

#local T_Space_Tug_Window = texture { T_Gold_Glass }

#local Space_Tug_Window = intersection {
  difference {
      sphere {0, Space_Tug_R - 5*cm}
      sphere {0, Space_Tug_R - 10*cm} 
  }
  object {Cutout_Box}
  hollow
  texture { T_Space_Tug_Window }
  interior { I_Glass1 }
}

#local T_Roam_Dock = texture {
  T_Silver_2D
  finish {
    ambient 0
  }
}

#local Space_Tug_Shell = union {
  difference {
    difference { // spherical hollow shell
      sphere {0, Space_Tug_R}
      sphere {0, Space_Tug_I_R}
      #declare Roam_Cutout_R = Roam_Shell_R + 10*cm;
      sphere {0, Roam_Cutout_R translate Space_Tug_R*y rotate 45*z rotate 45*x} 
      sphere {0, Roam_Cutout_R translate Space_Tug_R*y rotate 45*z rotate -45*x} 
      sphere {0, Roam_Cutout_R translate Space_Tug_R*y rotate 45*z rotate 135*x} 
      sphere {0, Roam_Cutout_R translate Space_Tug_R*y rotate 45*z rotate -135*x} 
      texture {T_Space_Tug}
    }
    // cutout window
    object {Cutout_Box}
  }
  // ROAM Docks
  #local Roam_Dock_T = 10*cm;
  #local Roam_Dock_R = Roam_Cutout_R + Roam_Dock_T;
  #local Roam_Craft_Dock = torus {Roam_Dock_R Roam_Dock_T texture {T_Roam_Dock} scale <1,.25,1>}
  #local ROAM_Dock_Elevation = Space_Tug_R - 4*inch;
  object {Roam_Craft_Dock translate ROAM_Dock_Elevation*y rotate 45*z rotate 45*x}
  object {Roam_Craft_Dock translate ROAM_Dock_Elevation*y rotate 45*z rotate -45*x}
  object {Roam_Craft_Dock translate ROAM_Dock_Elevation*y rotate 45*z rotate 135*x}
  object {Roam_Craft_Dock translate ROAM_Dock_Elevation*y rotate 45*z rotate -135*x}

  // Park the ROAMers
  
  object {ROAM_Craft_Unlit rotate 90*x translate Space_Tug_R*y rotate 45*z rotate 45*x}
  object {ROAM_Craft_Unlit rotate 90*x translate Space_Tug_R*y rotate 45*z rotate -45*x}
  object {ROAM_Craft_Unlit rotate 90*x translate Space_Tug_R*y rotate 45*z rotate 135*x}
  object {ROAM_Craft_Unlit rotate 90*x translate Space_Tug_R*y rotate 45*z rotate -135*x}
  
  
  // Add Window
  object {Space_Tug_Window}
  object {Docking_Ring rotate 90*z translate -Space_Tug_R*x}
  // rails for torus
  #local Rail_R = 6*inch;
  #local Rail_Maj_R = Space_Tug_R + Rail_R;
  torus {Rail_Maj_R Rail_R rotate 90*z texture {T_Silver_3D finish { ambient 0}}}
}

#declare Space_Tug_Body = union {
  object {SpaceTug_SearchLight 
    //rotate -50*z 
    translate (Space_Tug_R + 1*m)*x  
    rotate SearchLight_Mount_Angle*z
  }
  object {SpaceTug_SearchLight 
    //rotate -50*z 
    translate (Space_Tug_R + 1*m)*x  
    rotate SearchLight_Mount_Angle*z
    rotate 110*x
  }
  object {SpaceTug_SearchLight 
    //rotate -50*z 
    translate (Space_Tug_R + 1*m)*x  
    rotate SearchLight_Mount_Angle*z
    rotate -110*x
  }
  object {Space_Tug_Torus}
  object {Space_Tug_Shell}
}

#macro Make_SpaceTug(UpperRightArm, UpperLeftArm, LowerRightArm, LowerLeftArm)
union {
  object {Space_Tug_Body}
  // Arm 1 -- Upper Right
  object {UpperRightArm
    translate Space_Tug_R*x
    rotate 50*y
    rotate 50*x
  }
  // Arm2 -- Upper Left
  object {UpperLeftArm
    translate Space_Tug_R*x
    rotate -50*y
    rotate -50*x
  }   
  // Arm3 -- Lower Right
  object {LowerRightArm
    translate Space_Tug_R*x
    rotate 50*y
    rotate -50*x
  }
  
  // Arm4 -- Lower Left
  object {LowerLeftArm
    translate Space_Tug_R*x
    rotate -50*y
    rotate 50*x
  }
     
}
#end // Make_Space_Tug



///////-----------------------------------------------
#end // ifndef(__SpaceTug__)