// Persistence of Vision Ray Tracer Scene Description File
// File: Speakers.inc
// Vers: 3.5
// Desc: Speakers
// Date: 9/13/2004
// Auth: Ray Benjamin
//

#ifndef (__speakers__)
  #declare __speakers__ = 1;


#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end
#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end

#include "logo.inc"

////////////////// Speakers /////////////////////////////////

#declare T_Speaker = texture {
  pigment {Gray10}
}
#declare T_Speaker_Cloth = texture {
  //pigment {rgbt<.3,.3,.3,.1>}
  pigment {
    agate
    color_map {
     [0 rgbt <.3,.3,.3,.2> ]
     [.5 rgbt <.4,.4,.4,.5> ]
     [1 rgbt <.3,.3,.3,.2> ]
    }
    turbulence 1
    scale .125
  }
  finish {
    roughness 1
  }
  normal { granite .1 scale <1,1,2> }
  normal { quilted .05 }
}

#local T_Speaker_Trim = texture { T_Chrome_3C }

/////////////////// Speaker ////////////////////////////////
#macro MakeSpeaker(Spk_Radius,Spk_Depth,Spk_Frame_Width, Spk_Frame_Depth)
  union {
    cone {Origin, Spk_Radius, <0,0,Spk_Depth>, 0 open texture {T_Speaker_Cloth}}
    union {
      object {
        //Ring(Spk_Radius + Spk_Frame_Width, Spk_Radius,Spk_Frame_Depth)
        torus {Spk_Radius - Spk_Frame_Width/2, Spk_Frame_Width/2 scale <1,.5,1>}
        rotate -90*x
      }
      sphere {<0,0,0>, Spk_Radius/4 translate Spk_Depth*z}
      texture {
        pigment {Gray50}
        finish {
          reflection .25
          specular .5
          roughness .01
          metallic
        }
      }
    }
  }
#end // macro MakeSpeaker

#declare Base_Speaker = union {
  #local Base_Width = 1*foot;
  #local Base_Height = 1*foot;
  #local Base_Depth = 1*foot;
  #local Base_Speaker_Radius = 4*inch;
  #local Mid_Speaker_Radius = 1.5*inch;
  #local Tweeter_Radius = 1*inch;
  
  #local Base_Spk = object { MakeSpeaker(Base_Speaker_Radius,Base_Speaker_Radius/2,.5*inch,1/16*inch) }
  #local Mid_Spk = object {MakeSpeaker(Mid_Speaker_Radius,Mid_Speaker_Radius/2,.5*inch,1/16*inch)}
  #local Tweeter = object {MakeSpeaker(Tweeter_Radius,Tweeter_Radius/2,.5*inch,1/16*inch)}
  
  #local Base_Pos = <Base_Width/2,Base_Height/2-1.25*inch,0>;
  #local Mid_Pos = <Mid_Speaker_Radius + .5*inch,Base_Height-Mid_Speaker_Radius-.5*inch,0>;
  #local Tweeter_Pos = <Base_Width - Tweeter_Radius - 1*inch,Base_Height-Tweeter_Radius-1*inch,0>;
  difference {
    box {<0,0,0>, <Base_Width,Base_Height,Base_Depth> texture {T_Speaker}}
    box {<1*inch,1*inch,1*inch>,<Base_Width-1*inch,Base_Height-1*inch,Base_Depth-1*inch> texture {T_Speaker}} // make it hollow
    cylinder {<0,0,-.1*inch>, <0,0,1.1*inch>, Base_Speaker_Radius translate Base_Pos}
    cylinder {<0,0,-.1*inch>, <0,0,1.1*inch>, Mid_Speaker_Radius translate Mid_Pos}
    cylinder {<0,0,-.1*inch>, <0,0,1.1*inch>, Tweeter_Radius translate Tweeter_Pos}
  }
  object {Base_Spk translate Base_Pos}
  object {Mid_Spk translate Mid_Pos}
  object {Tweeter translate Tweeter_Pos}
  #local Center_Pos = <Base_Width/2,Base_Height/2,-1*inch>;
  #local Origin_Pos = <0,0,0>;
  #local UL_Pos = <0,Base_Height,0>;
  #local UR_Pos = <Base_Width,Base_Height,0>;
  #local LR_Pos = <Base_Width,0,0>;
  
  mesh {
    triangle { Origin_Pos, Center_Pos, UL_Pos } // left side
    triangle { UL_Pos, Center_Pos, UR_Pos } // top
    triangle { UR_Pos, Center_Pos, LR_Pos } // right
    triangle { LR_Pos, Center_Pos, Origin_Pos } // bottom
    texture {T_Speaker_Cloth}
    translate -1/16*inch*z
  }
  
  translate 1*cm*y
}


#declare Cube_Speaker = union {
  #local Speaker_Width = 3*inch;
  #local Speaker_Height = 3*inch;
  #local Speaker_Depth = 3*inch;

  box {<0,0,0>, <Speaker_Width,Speaker_Height,Speaker_Depth> texture {T_Speaker}}
  #local Center_Pos = <Speaker_Width/2,Speaker_Height/2,-.25*inch>;
  #local Origin_Pos = <0,0,0>;
  #local UL_Pos = <0,Speaker_Height,0>;
  #local UR_Pos = <Speaker_Width,Speaker_Height,0>;
  #local LR_Pos = <Speaker_Width,0,0>;
  
  mesh {
    triangle { Origin_Pos, Center_Pos, UL_Pos } // left side
    triangle { UL_Pos, Center_Pos, UR_Pos } // top
    triangle { UR_Pos, Center_Pos, LR_Pos } // right
    triangle { LR_Pos, Center_Pos, Origin_Pos } // bottom
    texture {T_Speaker_Cloth}
    translate -1/16*inch*z
  }
  
  translate .5*in*y
}


///////////////////// Rectangular Speaker ///////////////////////////////////////////


// Pov Logo Plate

#local T_Logo = texture {T_Gold_3D}
#local Logo_Plate = object {
  Povray_Logo_Prism
  scale .5
  texture {T_Logo} 
}


#declare Rectangular_Speaker_Box = union {
  #local Speaker_Width = 3*inch;
  #local Speaker_Height = 5*inch;
  #local Speaker_Depth = 1*inch;
  #local Speaker_T = 5*mm;
  #local Speaker_Trim_R = 1.5*mm;
  
  #local Cutout_W = .35*inch;
  #local Cutout_L = Speaker_Width - (Cutout_W*2)-.5*inch;
  #local Cutout = union {
    box {<0,-Cutout_W,-1>, <Cutout_L,Cutout_W,1>}
    cylinder {-1*z, 1*z, Cutout_W}
    cylinder {-1*z, 1*z, Cutout_W translate Cutout_L*x}
  }
  #local speaker_pos = <Speaker_Width/2,Speaker_Height/2,0>;
  #local speaker_R = Speaker_Width/2-.25*inch;
  #local speaker = object{ MakeSpeaker(speaker_R,speaker_R/2,.25*inch,1/16*inch )}
  difference {
    HollowBox(<0,0,0>, <Speaker_Width,Speaker_Height,Speaker_Depth>, Speaker_T)
    cylinder {<0,0,-.1*inch>, <0,0,1.1*inch>, speaker_R translate speaker_pos} // speaker cutout
    object {Cutout translate <.25*inch+Cutout_W,(Speaker_Height - Cutout_W - .25*inch),0>}
    object {Cutout translate <.25*inch+Cutout_W,(Cutout_W+.25*inch),0>}
    texture {T_Speaker}
  }
  object {speaker translate speaker_pos}       
  #local Center_Pos = <Speaker_Width/2,Speaker_Height/2,-.25*inch>;
  #local Origin_Pos = <0,0,0>;
  #local UL_Pos = <0,Speaker_Height,0>;
  #local UR_Pos = <Speaker_Width,Speaker_Height,0>;
  #local LR_Pos = <Speaker_Width,0,0>;
  
  mesh {
    triangle { Origin_Pos, Center_Pos, UL_Pos } // left side
    triangle { UL_Pos, Center_Pos, UR_Pos } // top
    triangle { UR_Pos, Center_Pos, LR_Pos } // right
    triangle { LR_Pos, Center_Pos, Origin_Pos } // bottom
    texture {T_Speaker_Cloth}
    translate -1/16*inch*z
  }
  // Logo
  object {Logo_Plate rotate -5*x translate <Speaker_Width/2,.75*inch,-2/16*inch>}
  
  // Chrome Trim
  
  merge {
    #local Horiz_Trim = merge {
      cylinder {0, Speaker_Width*x, Speaker_Trim_R}
      sphere {0, Speaker_Trim_R}
      sphere {0, Speaker_Trim_R translate Speaker_Width*x}
    }
    #local Vert_Trim = cylinder {0, Speaker_Height*y, Speaker_Trim_R}
    object {Horiz_Trim}
    object {Horiz_Trim translate Speaker_Height*y}
    object {Vert_Trim}
    object {Vert_Trim translate Speaker_Width*x}
    texture {T_Speaker_Trim}
  } // trim
  
  /////////// Back of Speaker Box /////////////////
  difference {
    object {
      Round_Cylinder_Merge(<0,0,0>, <0,Speaker_Height,0>,Speaker_Width/2,Speaker_Trim_R)
      scale <1,1,1.25>
      clipped_by {
        plane {-z,0}
      }
    }
    #local Clip_Cyl = cylinder {0, Speaker_Width*x, Speaker_Width/2 scale <1,.5,1> }
    object {Clip_Cyl translate <-Speaker_Width/2,Speaker_Height,Speaker_Width/2>}
    object {Clip_Cyl translate <-Speaker_Width/2,0,Speaker_Width/2>}
    texture {T_Speaker}
    translate <Speaker_Width/2,0,Speaker_Depth>
  }
  
} // Rectangular_Speaker_Box

#local Speaker_Stand = union {
  #local Stand_W = Speaker_Width;
  #local Stand_H = .75*inch;
  #local Stand_T = .2*inch;
  #local Stand_Base_R = 3*inch;
  #local Connect_W = 1*inch;
  #local Connect_D = .2*inch;
  #local Connect_H = 3*inch;
  difference {
    difference {
      cylinder {0, Stand_W*x, Stand_Base_R}
      cylinder {-.1*x,(Stand_W+.1)*x, (Stand_Base_R - Stand_T)}
      clipped_by {
        plane {-y,Stand_Base_R-Stand_H}
      } // clip
      translate -(Stand_Base_R-Stand_H)*y
    }
    #local Trim_Cyl = cylinder {0, (Stand_Base_R+1)*y, Stand_W}
    object {Trim_Cyl translate <-(Stand_W/2+1*inch),0,0>}
    object {Trim_Cyl translate <(Stand_W+Stand_W/2+1*inch),0,0>}
  }
  box {0, <Connect_W,Connect_H,Connect_D>
    translate <(Stand_W-Connect_W)/2,.5,Stand_Base_R/3.5>
  }
  translate <0,0,Stand_Base_R/2>
  texture {T_Speaker}
}

#declare Rectangular_Speaker = union {
  object {Rectangular_Speaker_Box translate 1*in*y}
  object {Speaker_Stand}
}



////////----------------------------------------------------
#end // ifndef __speakers__