// Persistence of Vision Ray Tracer Scene Description File
// File: Pencil.inc
// Vers: 3.5
// Desc: Pencil Model
// Date: 9/7/2004
// Auth: Ray Benjamin

#ifndef (__stapler__)
#declare __stapler__ = 1;

#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end

#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end

////////////////// Utility Macros /////////////////////

#macro Make_Phillips_Screw_Head(Radius, Height)
  difference {
    cone {<0,0,0>, Radius, <0,-Height,0>, Radius/2}
    #local Cutout = box {<0,0,0>, <Radius*3/2,Height/2,Radius/4> translate <-Radius*3/4,-Height/3,-Radius/8> }
    object { Cutout }
    object { Cutout rotate 90*y }
  }  
#end // Phillips_Screw_Head macro

/////////////////////////////////////////////////////


#local ST_Staple_Holder_W = 12.1*cm;
#local ST_Staple_Holder_H = 1.4*cm;
#local ST_Staple_Holder_D = 1.6*cm;
#local ST_Staple_Holder_R = 1.0*mm;

#local ST_Shell_W = ST_Staple_Holder_W + 20*mm;
#local ST_Shell_H = ST_Staple_Holder_H + 6*mm;
#local ST_Shell_D = ST_Staple_Holder_D + 10*mm;
#local ST_Shell_Edge_R = 2*mm;
#local ST_Shell_T = 2*mm;

#local SB_W = ST_Shell_W;
#local SB_H = 7*mm;
#local SB_D = ST_Shell_D + 4*mm;

#local Support_T = 2*mm;
#local Support_H = 2.2*cm;
#local Support_W = 5*cm;
#local Support_Top_Indent = .5*cm;

#local Decoration_T = .5*mm;
#local Decoration_H = Support_H - 4*mm;
#local Decoration_W = Support_W - 4*mm;
#local Decoration_Top_Indent = .5*cm;

#local CP_W = 1.7*cm;
#local CP_D = ST_Staple_Holder_D+4*mm;//1.4*cm;
#local CP_H = 1*mm;

#local T_Crimp_Plate = texture { T_Silver_3C}

#local CP_Screw_R = 2*mm;


#local Crimp_Plate = union {
  
  difference {
    Round_Box_Merge(<0,0,0>, <CP_W,CP_H,CP_D>,.5*mm)
    // staple guides
    #local Guide_Hole = sphere {0, .5*mm scale <1.5,1.5,5>}
    union {
      union {
        object {Guide_Hole translate -3*mm*z }
        object {Guide_Hole translate 3*mm*z }
        translate (CP_W-2*mm)*x
      }
      union {
        object {Guide_Hole translate -5*mm*z }
        object {Guide_Hole translate 5*mm*z }
        translate 2*mm*x
      }
      translate <0,CP_H,CP_D/2>
    }
    // central screw hole
    sphere {0, CP_Screw_R*1.1 translate <CP_W/2,CP_H,CP_D/2>}
  }
  
  object {Make_Phillips_Screw_Head(CP_Screw_R,1*mm) translate <CP_W/2,CP_H-.05*mm,CP_D/2> texture {T_Brass_3C}}
  translate -CP_H*y
  texture {T_Crimp_Plate}
}

#local Crimp_Plate_Cutout = object {
  Round_Box_Merge(<0,0,0>, <CP_W,CP_H*2,CP_D>,.5*mm)
  translate -CP_H*.95*y
}


#local T_Stapler_Shell = texture {
  pigment { Black }
  finish {
    diffuse .5
    reflection .05
  }
}


#local Decoration_Text = height_field {
  png               // the file type to read (tga/pot/pgm/ppm/png/sys)
  "StaplerDecal.png"     // the file name to read
//  [smooth]        // smooth surface normal
  water_level .1
//  [water_level N] // truncate/clip below N (0.0 ... 1.0)
//  texture {...}
  texture {T_Gold_1A}
  scale <Support_W, 1*mm,Support_H>
//  translate VECTOR | rotate VECTOR | scale VECTOR
  rotate -90*x
  translate <0*mm,0*mm,.25*mm>
}


#local T_Support_Decoration = texture {
  pigment {Black}
  finish {
    diffuse .5
    reflection .05
  }
}

#local Support_Decoration_Panel =   prism {
    linear_sweep
    linear_spline
    0,
    Decoration_T,
    5,
    <1*mm,0>, <Decoration_W,0>,
    <Decoration_W, Decoration_H>,
    <Decoration_Top_Indent, Decoration_H>,
    <1*mm,0>
    rotate -90*x
    translate <2*mm,2*mm,0>
    texture {T_Support_Decoration}
}


#local Support_Decoration_Left = union {
  object {Support_Decoration_Panel translate .25*mm*z}
  object {Decoration_Text}
}

#local Support_Decoration_Right = union {
  object {Support_Decoration_Panel}
  object {Decoration_Text
    translate -Support_W/2*x
    rotate 180*y
    translate Support_W/2*x
    translate .25*mm*z
  }
}

#local Support = prism {                                
  linear_sweep  // or conic_sweep for tapering to a point
  linear_spline // linear_spline | quadratic_spline | cubic_spline | bezier_spline 
  0,         // height 1
  Support_T,         // height 2
  5,           // number of points
  <0,0>, <Support_W,0>
  <Support_W,Support_H>
  <Support_Top_Indent, Support_H>,
  <0,0>
  rotate -90*x
  translate Support_T*z
  texture {T_Stapler_Shell}
}

#local Support_Assembly = union {
  union {
    object {Support}
    object {Support_Decoration_Left}
  }
  union {
    object {Support}
    object {Support_Decoration_Right translate (Decoration_T+Support_T)*z}
    translate (SB_D-Support_T)*z
  }
  translate <1*mm,SB_H - .25*mm,0>
}


#local Stapler_Base = union {
  #local Crimp_Plate_Pos = <(SB_W - CP_W - .5*cm),SB_H,(SB_D-CP_D)/2>;
  difference {
    Round_Box_Merge( <0,0,0>, <SB_W,SB_H,SB_D>, 1*mm )
    object {Crimp_Plate_Cutout translate Crimp_Plate_Pos}
  }
  object {Crimp_Plate translate Crimp_Plate_Pos} 
  object {Support_Assembly}
  texture {T_Stapler_Shell}
}

#local Stapler_Top_Shell = difference {
  Round_Box_Merge ( <0,0,0>, <ST_Shell_W,ST_Shell_H,ST_Shell_D>, ST_Shell_Edge_R ) 
  Round_Box_Merge ( <ST_Shell_T,-ST_Shell_T,ST_Shell_T>, <ST_Shell_W-ST_Shell_T,ST_Shell_H-ST_Shell_T,ST_Shell_D-ST_Shell_T>, ST_Shell_Edge_R)
  texture {T_Stapler_Shell}
}

#local Holder_Tip_W = 6*mm;
#local Holder_Tip_H = 1*cm;
#local Holder_Tip_D = ST_Staple_Holder_D + 2*mm;
#local Holder_Tip_R = 1*mm;

// Staple View Slot Cutout
#local SV_W = .5*cm;
#local SV_H = 2*mm;
#local SV_D = ST_Staple_Holder_D + 2*mm;

#local Staple_View_Slot_Cutout = union {
  cylinder {0, SV_D*z SV_H/2}
  box {<0,-SV_H/2,0>, <SV_W,SV_H/2,SV_D> }
  cylinder {<SV_W,0,0>, <SV_W,0,SV_D>, SV_H/2 }
  translate -1*mm*z
}
 

// Staple Holder

#local T_Staples = texture {
  pigment {
    gradient x
    color_map {
      [0 Black]
      [.15 Black]
      [.45 White]
      [1 White]
    } // color_map
    sine_wave
    scale <1*mm,1,1>
  } // pigment
  finish { metallic }
} // texture

#local Staple_Holder = union {
  union {
    // main staple holder
    difference {
      Round_Box_Merge( <0,0,0>, <ST_Staple_Holder_W,ST_Staple_Holder_H,ST_Staple_Holder_D>, ST_Staple_Holder_R)
      Round_Box_Merge(<.5*mm,.5*mm,.5*mm>, <ST_Staple_Holder_W - .5*mm, ST_Staple_Holder_H+.5*mm, ST_Staple_Holder_D-.5*mm>, ST_Staple_Holder_R)
      // Staple View Slot
      object {Staple_View_Slot_Cutout translate <ST_Staple_Holder_W - 1.3*cm,5*mm,0>}
      //  Staple Holder Guide Notches
      #local STG_W = 5*mm;
      #local STG_D = 2*mm;
      #local STG_H = 2*mm;
      #local STG_X = ST_Staple_Holder_W - 1.2*cm;
      
      box {<0,0,0>, <STG_W, STG_H, STG_D> translate <STG_X,0,0>}
      box {<0,0,0>, <STG_W, STG_H, STG_D> translate <STG_X,0,ST_Staple_Holder_D-STG_D>}
    }
    // front piece
    object {
      difference {
        Round_Box_Merge( <0,0,0>, <Holder_Tip_W,Holder_Tip_H,Holder_Tip_D>, Holder_Tip_R)
        // The crimp that holds this piece in place...
        #local Crimp_W = 2*mm;
        #local Crimp_H = 4*mm;
        #local Crimp_D = 1*mm;
        #local Crimp_Angle = 15;
        
        #local R_Crimp = Round_Box_Merge(<0,0,-Crimp_D> <Crimp_W, Crimp_H, 0>,.5*mm)
        #local L_Crimp = Round_Box_Merge(<0,0,0> <Crimp_W, Crimp_H, Crimp_D>,.5*mm)
        object {R_Crimp rotate -Crimp_Angle*y translate <1.5*mm,3*mm,0>}
        object {L_Crimp rotate Crimp_Angle*y translate <1.5*mm,3*mm,Holder_Tip_D>}
        
      }
      translate <(ST_Staple_Holder_W - (Holder_Tip_W - 1*mm)), 0, -1*mm>
    }
    // Slide Guides
    #local SG_R = 1*mm;
    #local SG_L = 6*cm;
    #local SG_Y = 4*mm;
    #local SG_X = 2*mm;
    
    #local Slide_Guide = union {
      sphere {0, SG_R}
      cylinder {0, SG_L*x, SG_R}
      sphere {SG_L*x, SG_R}
      translate <SG_X,SG_Y,0>
    }
    object {Slide_Guide}
    object {Slide_Guide translate ST_Staple_Holder_D*z}
    ///////////// end of staple holder body //////////////
    texture {T_Silver_3D}
  }
  // Staples
  box {<1*mm,1*mm,1*mm>, <ST_Staple_Holder_W - 1*mm, ST_Staple_Holder_H+1*mm, ST_Staple_Holder_D-1*mm>
    texture {T_Staples}
  }
  //
  #local X_Offset = (ST_Shell_W-ST_Staple_Holder_W)/2;
  #local Z_Offset = (ST_Shell_D - ST_Staple_Holder_D)/2;
  translate <X_Offset,0,Z_Offset>
}


#local Top_Offset_Y = 14*mm;
#local Top_Angle = 0*mm;
#local Shell_Offset = 9*mm;

#declare Stapler_Top = union {
  object {Stapler_Top_Shell translate <0,Shell_Offset,0>}
  object {Staple_Holder translate 4*mm*x}
  translate Support_T*z
  //rotate 90*x
  translate Top_Offset_Y*y
}

#declare Stapler = union {
  object {Stapler_Top translate 0*cm*y}
  object {Stapler_Base}
}

#declare Stapler_L = SB_W;
#declare Stapler_D = SB_D;

//////-----------------------------------------------
#end // ifndef __stapler__