// Persistence of Vision Ray Tracer Scene Description File
// File: ThumbTack.pov
// Vers: 3.5
// Desc: Basic Scene Example
// Date: 9/5/2004
// Auth: Ray Benjamin
//

#version 3.5;

#ifndef(__thumb_tack__)
#declare __thumb_tack__ = 1;

#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end
#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end


/////////////////////// Thumb Tack /////////////////////////

#declare TT_Top_R = 4*mm;//7/64;
#declare TT_Bottom_R = 5*mm;//4/32;
#declare TT_Top_H = 2*mm;//1/16;
#declare TT_Bottom_H = 4*mm;//1/8;
#declare TT_H = 13*mm;//1/2;
#declare TT_Spike_Cylinder_L= 8*mm;//1/4;
#declare TT_Spike_Cone_L = 3*mm;//1/8;
#declare TT_Spike_R = .5*mm;//1/64;
#declare TT_Bottom_Torus_Minor_R = 3*mm;//1/8;

#declare T_Thumb_Tack = texture {
  pigment {Red}
  finish {
    diffuse .5
    specular .7
    reflection {.1}
    conserve_energy
  }
}

#macro Make_Thumb_Tack(TackColor)
union {
    // top part
    #local Top_Torus_Pos = <0,TT_H - 1/32,0>;
    #local TT_r = 1*mm;//1/32;
    merge {
      torus {TT_Top_R TT_r // forms undercurve of top piece
        //pigment {Red}
        clipped_by {
          plane {y,0}
        }
        translate Top_Torus_Pos
      }
      cylinder {<0,0,0>, <0,1*mm,0>, TT_Top_R+1*mm translate Top_Torus_Pos 
        //pigment {Yellow}
      } // top cylinder
      cylinder {Top_Torus_Pos, (Top_Torus_Pos - TT_r)*y, TT_Top_R 
        //pigment {Blue}
      }
      intersection {
        torus {TT_Bottom_R 3*mm } // bottom part, top curve
        plane {-y,0}
        //pigment {Green}
      }
      cone {<0,0,0>, TT_Bottom_R-1*mm, <0,TT_H,0> TT_Top_R-.5*mm
        //pigment {Orange}
      } // cone between top and bottom
      cone {<0,2.8*mm,0>, TT_Bottom_R+1*mm, <0,4*mm,0>, TT_Bottom_R-1.1*mm 
        //pigment {White}
        //translate 2.5*mm*y
      } // cone at top of bottom section

      texture {
        pigment {TackColor}
        finish {
          diffuse .5
          specular .7
          reflection {.1}
          conserve_energy
        }
      }
      interior { ior 1.33 }
    }
  // spike pike
  union {
    cylinder {<0,-TT_Spike_Cylinder_L,0>, <0,TT_Spike_Cylinder_L,0>, TT_Spike_R}
    cone {<0,0,0>, TT_Spike_R, <0,-TT_Spike_Cone_L,0>, 0 translate -TT_Spike_Cylinder_L*y}
    texture {T_Silver_3C}
  }

}
#end // macro

#macro Make_Horizontal_Thumb_Tack(TackColor)
    object {Make_Thumb_Tack(TackColor) rotate -90*z translate (TT_Bottom_R + TT_Bottom_Torus_Minor_R+.1*mm)*y rotate -12.85*z}    

#end

#declare Thumb_Tack_Red = object {
  Make_Thumb_Tack(Red)
}

#declare Thumb_Tack_White = object {
  Make_Thumb_Tack(White)
}

#declare Thumb_Tack_Blue = object {
  Make_Thumb_Tack(Blue)
}

#declare Thumb_Tack_Clear = object {
  Make_Thumb_Tack(Clear)
}

#declare Thumb_Tack_Yellow = object {
  Make_Thumb_Tack(Yellow)
}

///////-------------------------------------------------------
#end // ifndef __thumb_tack__
