// Persistence of Vision Ray Tracer Scene Description File
// File: utils.inc
// Vers: 3.5
// Desc: Utility functions and Macros
// Date: 3/21/2004
// Auth: Ray Benjamin

#ifndef(__utils__)
#declare __utils__=1;

//
// This macro produces a triangle constructed of spheres, cylinders and
// two flat mesh triangles.  It can be used to construct surfaces of
// triangles with smooth transitions rather than sharp edges.
// 
// Args: 
// p2,p2,p3 - the points
// Radius - radius of the cylinders and spheres
//
#macro Round_Triangle(p1,p2,p3,Radius)
  union {
    sphere {p1 Radius}
    sphere {p2 Radius}
    sphere {p3 Radius}
    cylinder {p1,p2,Radius}
    cylinder {p2,p3,Radius}
    cylinder {p3,p1,Radius}
    #local v1 = p2-p1;
    #local v2 = p3-p1;
    #local vp = vcross(v1,v2); // get vector perpendicular to plane of triangle
    #local vu = vnormalize(vp); // get unit vector
    #local vr = vu * Radius; // scale vector
    #local myTriangle = mesh {
      triangle {p1,p2,p3}
    }
    object {myTriangle translate vr}
    object {myTriangle translate -vr}
  }
#end // macro Round_Triangle

#macro Round_Triangle3(p1,r1,p2,r2,p3,r3)
union {
  sphere {p1 r1}
  sphere {p2 r2}
  sphere {p3 r3}
  cone {p1 r1 p2 r2}
  cone {p2 r2 p3 r3}
  cone {p3 r3 p1 r1}
  #local v1 = p2 - p1;
  #local v2 = p3 - p1;
  #local vc = vcross(v1,v2);
  #local vu = vnormalize(vc);
  mesh {
    triangle { p1+vu*r1,p2+vu*r2,p3+vu*r3 }
    triangle { p1-vu*r1,p2-vu*r2,p3-vu*r3 }
  }
}
#end // macro

#declare Origin = <0,0,0>;
// This macro creates a ring in the X-Z plane that extends from 0 to Thickness in Y
//
#macro Ring(Outer_Radius,Inner_Radius,Thickness)
  difference {
    cylinder {Origin,<0,Thickness,0>,Outer_Radius}
    cylinder {<0,-1,0>,<0,Thickness+1,0>,Inner_Radius}
  }
#end // macro Ring()

#macro HollowBox(Pt1,Pt2,Thickness)
  difference {
    box {Pt1, Pt2}
    box {<Pt1.x + Thickness,Pt1.y + Thickness, Pt1.z + Thickness>
         <Pt2.x - Thickness,Pt2.y - Thickness, Pt2.z - Thickness>
    }
  }
#end

#macro RoundHollowBox(Pt1,Pt2,Thickness,EdgeRadius)
  difference {
    Round_Box_Merge(Pt1,Pt2,EdgeRadius)
    Round_Box_Merge(<Pt1.x + Thickness,Pt1.y + Thickness, Pt1.z + Thickness>,
         <Pt2.x - Thickness,Pt2.y - Thickness, Pt2.z - Thickness>, EdgeRadius)
  }
#end

#end // ifndef(__utils__)
