// Persistence of Vision Ray Tracer Scene Description File
// File: webcam.inc
// Vers: 3.5
// Desc: Web Camera Model
// Date: 9/4/2004
// Auth: Ray Benjamin



///////////// Web Camera ///////////////////

#declare Web_Cam_R = 1*inch;
#declare Web_Cam_Shell_T = .1;
#declare Web_Cam_Seam_W = .01;
#declare Web_Cam_Inner_R = Web_Cam_R - Web_Cam_Shell_T;
#declare Web_Cam_Lens_Mount_R = .5*inch;
#declare Web_Cam_Lens_Mount_D = .75*inch;

#declare Web_Cam_Lens_Mount_R_Step = .05;
#declare Web_Cam_Lens_Mount_D_Step = .05;

#declare F_Plastic = finish {
    diffuse .3
    specular .5
    roughness .001
}

#declare T_Web_Cam = texture {
  pigment {White}
  finish { F_Plastic }
}

#declare T_Web_Cam_Lens_Mount = texture {
  pigment {Gray10}
}

#declare Web_Cam_Lens_Shield_R = Web_Cam_R + .1*inch;
#declare Web_Cam_Lens_Shield_T = .06*inch;
#declare Web_Cam_Lens_Shield_D = .5*inch;

#declare T_Web_Cam_Lens_Shield = texture {
  pigment { rgbft <.2,.2,.2,.5,.1> }
}

#declare Web_Cam_Lens_Shield = intersection {
  difference {
    sphere {0 Web_Cam_Lens_Shield_R + Web_Cam_Lens_Shield_T}
    sphere {0 Web_Cam_Lens_Shield_R}
  }
  union {
    #local L = (Web_Cam_Lens_Shield_R + Web_Cam_Lens_Shield_T + .1*inch)*2;
    box { <-L,0,-Web_Cam_Lens_Shield_D/2>, <L,L,Web_Cam_Lens_Shield_D/2> }
    cylinder {-L*x x*L Web_Cam_Lens_Shield_D/2}
    translate 0
  }
  scale <.95,1.1,1>
  texture {T_Web_Cam_Lens_Shield}
}

#declare Web_Cam_Lens_Mount = union {
  difference {
    cylinder {0, <0,0,Web_Cam_Lens_Mount_D>, Web_Cam_Lens_Mount_R}
    #local Mount_Cut_R = Web_Cam_Lens_Mount_R - .1;
    #local Mount_Cut_D = .05;
    cylinder {<0,0,-1>, <0,0,Mount_Cut_D>, Mount_Cut_R}  
    #local Mount_Cut_R = Mount_Cut_R - Web_Cam_Lens_Mount_R_Step;
    #local Mount_Cut_D = Mount_Cut_D + Web_Cam_Lens_Mount_D_Step; 
    cylinder {<0,0,-1>, <0,0,Mount_Cut_D>, Mount_Cut_R}  
    #local Mount_Cut_R = Mount_Cut_R - Web_Cam_Lens_Mount_R_Step;
    #local Mount_Cut_D = Mount_Cut_D + Web_Cam_Lens_Mount_D_Step; 
    cylinder {<0,0,-1>, <0,0,Mount_Cut_D>, Mount_Cut_R}  
    #local Mount_Cut_R = Mount_Cut_R - Web_Cam_Lens_Mount_R_Step;
    #local Mount_Cut_D = Mount_Cut_D + Web_Cam_Lens_Mount_D_Step; 
    cylinder {<0,0,-1>, <0,0,Mount_Cut_D>, Mount_Cut_R}
    // Reduce Step Size
    #declare Web_Cam_Lens_Mount_R_Step = Web_Cam_Lens_Mount_R_Step / 2;
    #declare Web_Cam_Lens_Mount_D_Step = Web_Cam_Lens_Mount_D_Step / 2;
    //
    #local Mount_Cut_R = Mount_Cut_R - Web_Cam_Lens_Mount_R_Step;
    #local Mount_Cut_D = Mount_Cut_D + Web_Cam_Lens_Mount_D_Step; 
    cylinder {<0,0,-1>, <0,0,Mount_Cut_D>, Mount_Cut_R}
    #local Mount_Cut_R = Mount_Cut_R - Web_Cam_Lens_Mount_R_Step;
    #local Mount_Cut_D = Mount_Cut_D + Web_Cam_Lens_Mount_D_Step; 
    cylinder {<0,0,-1>, <0,0,Mount_Cut_D>, Mount_Cut_R}
    #local Mount_Cut_R = Mount_Cut_R - Web_Cam_Lens_Mount_R_Step;
    #local Mount_Cut_D = Mount_Cut_D + Web_Cam_Lens_Mount_D_Step; 
    cylinder {<0,0,-1>, <0,0,Mount_Cut_D>, Mount_Cut_R}
    #local Mount_Cut_R = Mount_Cut_R - Web_Cam_Lens_Mount_R_Step;
    #local Mount_Cut_D = Mount_Cut_D + Web_Cam_Lens_Mount_D_Step; 
    cylinder {<0,0,-1>, <0,0,Mount_Cut_D>, Mount_Cut_R}
    #local Mount_Cut_R = Mount_Cut_R - Web_Cam_Lens_Mount_R_Step;
    #local Mount_Cut_D = Mount_Cut_D + Web_Cam_Lens_Mount_D_Step; 
    cylinder {<0,0,-1>, <0,0,Mount_Cut_D>, Mount_Cut_R}
    #local Mount_Cut_R = Mount_Cut_R - Web_Cam_Lens_Mount_R_Step;
    #local Mount_Cut_D = Mount_Cut_D + Web_Cam_Lens_Mount_D_Step; 
    // Lens Hole
    cylinder {<0,0,-1>, <0,0,Web_Cam_R+.25>, Mount_Cut_R}

    texture {T_Web_Cam_Lens_Mount}
  }
  // lens
  sphere {<0,0,0> Mount_Cut_R
    texture { T_Glass1 }
    scale <1,1,.2>
    translate Mount_Cut_D*z
  }
  translate -(Web_Cam_R+.08*inch)*z
}

#declare Web_Cam_Base_R = .5*in;
#declare Web_Cam_Base_H = .75*in;
#declare Web_Cam_Base_Top_R = .125*in;

#declare T_Web_Cam_Base = texture {
  pigment { Gray20 }
}

#declare Web_Cam_Base = cone { 
  <0,0,0> Web_Cam_Base_R <0,Web_Cam_Base_H,0> Web_Cam_Base_Top_R 
    scale <2,1,1>
    texture {T_Web_Cam_Base}
}

#declare WC_Clip_Base_W = 2*inch;
#declare WC_Clip_Base_H = .25*inch;
#declare WC_Clip_Base_D = 2*inch;
#declare WC_Clip_Leg_H = .5*inch;
#declare WC_Clip_Leg_T = .25*inch;
#declare WC_Clip_Mount_Ball_R = .2*inch;
#declare WC_Clip_Mount_Rod_R = .125*inch;
#declare WC_Clip_Mount_Rod_H = 1*inch;

#declare T_WC_Clip = texture {
  pigment { Gray20 }
}

#declare WC_Clip = union {
  // base 
  union {
    box {0 <WC_Clip_Base_W, WC_Clip_Base_H, WC_Clip_Base_D>}
    #declare WC_Clip_Leg = box {0, <WC_Clip_Leg_T,WC_Clip_Leg_H,WC_Clip_Leg_T> }
    object {WC_Clip_Leg translate <0,-WC_Clip_Base_H,0>}
    object {WC_Clip_Leg translate <WC_Clip_Base_W-WC_Clip_Leg_T,-WC_Clip_Base_H,0>}
    texture {T_WC_Clip}
  }
  // mounting ball
  sphere {0, WC_Clip_Mount_Ball_R
    texture {T_Chrome_3D}
    translate <WC_Clip_Base_W/2,WC_Clip_Base_H,WC_Clip_Base_D/2>
  }
  // mounting Rod
  cylinder {0, <0,WC_Clip_Mount_Rod_H,0>, WC_Clip_Mount_Rod_R translate <WC_Clip_Base_W/2,WC_Clip_Base_H,WC_Clip_Base_D/2>
    texture {T_Chrome_3D}
  }
  translate <-WC_Clip_Base_W/2,0,0>
}


#declare Web_Cam_Ball = union {
  difference {
    // Create the shell with a seam showing on outside, but sealed on inside with second sphere
    union {
      difference {
        sphere {<0,0,0>, Web_Cam_R}
        // seam
        box { <-Web_Cam_Seam_W/2,-(Web_Cam_R+.1),-(Web_Cam_R+.1)>, <Web_Cam_Seam_W/2,Web_Cam_R+.1,Web_Cam_R+.1> }
      }
      // smaller sphere inside shell to seal seam
      sphere {<0,0,0>, Web_Cam_R-Web_Cam_Shell_T/4}
    }
    //
    sphere {0, Web_Cam_Inner_R} // cutout inside of sphere
    // lens hole
    cylinder {<0,0,-(Web_Cam_R+.1)>, <0,0,0>, Web_Cam_Lens_Mount_R}
    texture {T_Web_Cam}
  }
  object {Web_Cam_Lens_Mount}
  object {Web_Cam_Lens_Shield rotate -40*x}
  rotate <-20,5,0>
  translate 1*inch*y
  translate (Web_Cam_Base_H - .125*in)*y
}

#declare Web_Cam = union {
  object {Web_Cam_Ball}
  object {Web_Cam_Base}
}

#declare Web_Cam_Clipped = union {
  object {WC_Clip}
  object {Web_Cam_Ball translate <0,WC_Clip_Mount_Rod_H-.5*inch,WC_Clip_Base_D/2>}
  translate -WC_Clip_Leg_T*z
}

