// Persistence of Vision Ray Tracer Scene Description File
// File: Docking_Ring.inc
// Vers: 3.5
// Desc: Spaceship Docking Ring
// Date: 9/6/2004
// Auth: Ray Benjamin
//

#ifndef (__docking_ring__)
  #declare __docking_ring__ = 1;


#ifndef (inch)
  #declare inch = 1;
  #declare in = inch;
#end
#ifndef (foot)
  #declare foot = 12;
  #declare ft = foot;
#end
#ifndef (cm)
  #declare cm = 0.393701*inch;
  #declare m = 100*cm;
  #declare km = 1000*m;
  #declare mm = cm/10;
#end

#declare Docking_Ring_R = 10*ft;
#declare Docking_Ring_Cone_H = Docking_Ring_R/2;
#declare Docking_Ring_Petal_H = Docking_Ring_R/8;
#declare Docking_Ring_T = 6*inch;
#declare Docking_Capture_Ring_H = 1*ft;
#declare Docking_Ring_Structure_R = Docking_Ring_R * 1.1;
#declare Docking_Ring_Structure_H = Docking_Ring_R/2;
#declare Docking_Ring_H = 1*ft; // height above pressure door

#local Cutout_W = Docking_Ring_R/2;

#declare T_Docking_Ring_Petals = texture {
  pigment {White}
  finish {ambient 0}
}
texture {T_Scuffs}

#declare T_Docking_Ring_Cylinder = texture { T_Silver_3C finish {ambient 0}}

#declare T_Docking_Ring_Structure = texture { 
  pigment { rgb<.2,.2,.5> }
  finish {ambient 0} 
}
texture {T_Scuffs rotate 90*y}

#declare Docking_Ring_Petals = intersection {
    difference {
      #local Docking_Cone = cone {0, Docking_Ring_R, Docking_Ring_Cone_H*y, 0  }
      object {Docking_Cone}
      object {Docking_Cone translate -Docking_Ring_T*y}
    }

  #local Intersection_Sphere = sphere {0, Docking_Ring_R/2.5 scale <1,1,1.5>}
  union {
    object {Intersection_Sphere translate -Docking_Ring_R*x}  
    object {Intersection_Sphere translate -Docking_Ring_R*x rotate 90*y}  
    object {Intersection_Sphere translate -Docking_Ring_R*x rotate -90*y}  
    object {Intersection_Sphere translate -Docking_Ring_R*x rotate -180*y}  
  }

  texture {T_Docking_Ring_Petals}  
} // Docking_Ring_Petals


#declare Docking_Capture_Ring = union {
  object {Docking_Ring_Petals translate Docking_Capture_Ring_H*y}
  difference {
    cylinder {0, Docking_Capture_Ring_H*y, Docking_Ring_R}
    cylinder {-1*mm*y, (Docking_Capture_Ring_H+1*mm)*y, (Docking_Ring_R - Docking_Ring_T)}
    texture {T_Docking_Ring_Cylinder}
  }
}

#local Upper_Door = cylinder { 0, Docking_Ring_T*y, Docking_Ring_R
  clipped_by {
    plane {-z,0}
  }
  texture {T_Caution translate <100*ft,20*ft,420*ft> rotate -45*y}
} 

#local Lower_Door = cylinder { 0, Docking_Ring_T*y, Docking_Ring_R
  clipped_by {
    plane {z,0}
  }
  texture {T_Caution rotate 45*y}
} 

#declare Docking_Ring = union {
  object {Docking_Capture_Ring translate Docking_Ring_H*y}
  //
  object {
    Ring(Docking_Ring_Structure_R,Docking_Ring_R, Docking_Ring_Structure_H)
    texture {T_Docking_Ring_Structure}
    translate (Docking_Ring_H-Docking_Ring_Structure_H)*y
  }
  object {Upper_Door}
  object {Lower_Door}
  object {Make_Fake_Running_Light(Red) rotate 90*z translate <-Docking_Ring_Structure_R,Docking_Ring_H/2,0>}
  object {Make_Fake_Running_Light(Green) rotate 90*z translate <-Docking_Ring_Structure_R,Docking_Ring_H/2,0> rotate -180*y}
  object {Make_Fake_Running_Light(Yellow) rotate 90*z translate <-Docking_Ring_Structure_R,Docking_Ring_H/2,0> rotate -90*y}
  object {Make_Fake_Running_Light(Blue) rotate 90*z translate <-Docking_Ring_Structure_R,Docking_Ring_H/2,0> rotate 90*y}
}

////////////////////////////////////////////////////////////
// Pressurized Gas/Liquid Connector Docking Ring


#local Cone_R = 20*cm;
#local Ring_T = 1*cm;
#local Inner_Ring_R = Cone_R + Ring_T;
#local Structure_Ring_R = Inner_Ring_R + Cone_R;
#local Cone_Depth = Cone_R;
#local Cone_H = 1.5*Cone_R;
#local Structure_Ring_H = Cone_R;

#local T_Cone = texture { 
  T_Silver_3D
  finish {ambient 0}
}
texture {T_Scuffs}

#local T_Ring = texture { 
  pigment {Yellow}
  finish {ambient 0} 
}
texture {T_Scuffs}

#declare Nozzle_Docking_Ring = union {
  // inner cone
  difference {
    cone {0, Cone_R, -Cone_Depth*y, 0}
    cone {0, Cone_R, -Cone_Depth*y, 0 translate 1*cm*y}
    texture {T_Cone}
    translate Cone_H*y
  }
  // ring outside cone
  object {Ring(Inner_Ring_R,Cone_R,Cone_H) texture {T_Ring}}
  // structure ring
  object {Ring(Structure_Ring_R, Inner_Ring_R, Structure_Ring_H) texture {T_Ring}}
  // lights
  /*
  object {Make_Fake_Running_Light(Yellow) rotate 90*z translate <-Structure_Ring_R,Structure_Ring_H/2,0>}
  object {Make_Fake_Running_Light(Yellow) rotate 90*z translate <-Structure_Ring_R,Structure_Ring_H/2,0> rotate -180*y}
  object {Make_Fake_Running_Light(Yellow) rotate 90*z translate <-Structure_Ring_R,Structure_Ring_H/2,0> rotate -90*y}
  object {Make_Fake_Running_Light(Yellow) rotate 90*z translate <-Structure_Ring_R,Structure_Ring_H/2,0> rotate 90*y}
  */
}

////////----------------------------------------------------
#end // ifndef __docking_ring__